@@ -19,19 +19,8 @@
 #include <unistd.h>
 #include <time.h>
 #include "knot/common/fdset.h"
-#include "libknot/libknot.h"
-
-/* Workarounds for clock_gettime() not available on some platforms. */
-#ifdef HAVE_CLOCK_GETTIME
-#define time_now(x) clock_gettime(CLOCK_MONOTONIC, (x))
-typedef struct timespec timev_t;
-#elif HAVE_GETTIMEOFDAY
-#include <sys/time.h>
-#define time_now(x) gettimeofday((x), NULL)
-typedef struct timeval timev_t;
-#else
-#error Neither clock_gettime() nor gettimeofday() found. At least one is required.
-#endif
+#include "contrib/time.h"
+#include "libknot/errcode.h"
 
 /* Realloc memory or return error (part of fdset_resize). */
 #define MEM_RESIZE(tmp, p, n) \
