@@ -26,18 +26,13 @@
 #include "knot/common/log.h"
 #include "knot/nameserver/query_module.h"
 #include "knot/nameserver/internet.h"
-#include "libknot/internal/mem.h"
-#include "libknot/internal/namedb/namedb_lmdb.h"
-#include "libknot/internal/sockaddr.h"
+#include "libknot/libknot.h"
 #include "libknot/yparser/ypformat.h"
 #include "libknot/yparser/yptrafo.h"
-
-/*! Configuration specific logging. */
-#define CONF_LOG(severity, msg, ...) do { \
-	log_msg(severity, "config, " msg, ##__VA_ARGS__); \
-	} while (0)
-
-#define MAX_INCLUDE_DEPTH	5
+#include "contrib/mempattern.h"
+#include "contrib/sockaddr.h"
+#include "contrib/string.h"
+#include "contrib/ucw/mempool.h"
 
 // The active configuration.
 conf_t *s_conf;
@@ -56,7 +51,7 @@ static void rm_dir(const char *path)
 
 	// Prepare own dirent structure (see NOTES in man readdir_r).
 	size_t len = offsetof(struct dirent, d_name) +
-		     fpathconf(dirfd(dir), _PC_NAME_MAX) + 1;
+	             fpathconf(dirfd(dir), _PC_NAME_MAX) + 1;
 
 	struct dirent *entry = malloc(len);
 	if (entry == NULL) {
@@ -93,10 +88,82 @@ static void rm_dir(const char *path)
 	}
 }
 
+static int init_and_check(
+	conf_t *conf,
+	conf_flag_t flags)
+{
+	if (conf == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	knot_db_txn_t txn;
+	unsigned txn_flags = (flags & CONF_FREADONLY) ? KNOT_DB_RDONLY : 0;
+	int ret = conf->api->txn_begin(conf->db, &txn, txn_flags);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
+	// Initialize the database.
+	if (!(flags & CONF_FREADONLY)) {
+		ret = conf_db_init(conf, &txn, false);
+		if (ret != KNOT_EOK) {
+			conf->api->txn_abort(&txn);
+			return ret;
+		}
+	}
+
+	// Check the database.
+	if (!(flags & CONF_FNOCHECK)) {
+		ret = conf_db_check(conf, &txn);
+		if (ret < KNOT_EOK) {
+			conf->api->txn_abort(&txn);
+			return ret;
+		}
+	}
+
+	return conf->api->txn_commit(&txn);
+}
+
+int conf_refresh_txn(
+	conf_t *conf)
+{
+	if (conf == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	// Close previously opened transaction.
+	conf->api->txn_abort(&conf->read_txn);
+
+	return conf->api->txn_begin(conf->db, &conf->read_txn, KNOT_DB_RDONLY);
+}
+
+void conf_refresh_hostname(
+	conf_t *conf)
+{
+	if (conf == NULL) {
+		return;
+	}
+
+	free(conf->hostname);
+	conf->hostname = sockaddr_hostname();
+}
+
+static void init_cache(
+	conf_t *conf)
+{
+	conf->cache.srv_nsid = conf_get(conf, C_SRV, C_NSID);
+	conf->cache.srv_max_udp_payload = conf_get(conf, C_SRV, C_MAX_UDP_PAYLOAD);
+	conf->cache.srv_max_tcp_clients = conf_get(conf, C_SRV, C_MAX_TCP_CLIENTS);
+	conf->cache.srv_tcp_hshake_timeout = conf_get(conf, C_SRV, C_TCP_HSHAKE_TIMEOUT);
+	conf->cache.srv_tcp_idle_timeout = conf_get(conf, C_SRV, C_TCP_IDLE_TIMEOUT);
+	conf->cache.srv_tcp_reply_timeout = conf_get(conf, C_SRV, C_TCP_REPLY_TIMEOUT);
+}
+
 int conf_new(
 	conf_t **conf,
 	const yp_item_t *scheme,
-	const char *db_dir)
+	const char *db_dir,
+	conf_flag_t flags)
 {
 	if (conf == NULL) {
 		return KNOT_EINVAL;
@@ -115,73 +182,84 @@ int conf_new(
 		return ret;
 	}
 
-	// Prepare namedb api.
-	out->mm = malloc(sizeof(mm_ctx_t));
+	// Initialize a config mempool.
+	out->mm = malloc(sizeof(knot_mm_t));
+	if (out->mm == NULL) {
+		yp_scheme_free(out->scheme);
+		free(out);
+		return KNOT_ENOMEM;
+	}
 	mm_ctx_mempool(out->mm, MM_DEFAULT_BLKSIZE);
-	struct namedb_lmdb_opts lmdb_opts = NAMEDB_LMDB_OPTS_INITIALIZER;
-	lmdb_opts.mapsize = 500 * 1024 * 1024;
-	lmdb_opts.flags.env = NAMEDB_LMDB_NOTLS;
 
-	// Open database.
+	// Set the DB api.
+	out->api = knot_db_lmdb_api();
+	struct knot_db_lmdb_opts lmdb_opts = KNOT_DB_LMDB_OPTS_INITIALIZER;
+	lmdb_opts.mapsize = (size_t)CONF_MAPSIZE * 1024 * 1024;
+	lmdb_opts.flags.env = KNOT_DB_LMDB_NOTLS;
+
+	// Open the database.
 	if (db_dir == NULL) {
-		// A temporary solution until proper trie support is available.
+		// Prepare a temporary database.
 		char tpl[] = "/tmp/knot-confdb.XXXXXX";
 		lmdb_opts.path = mkdtemp(tpl);
 		if (lmdb_opts.path == NULL) {
-			CONF_LOG(LOG_ERR, "failed to create temporary directory");
+			CONF_LOG(LOG_ERR, "failed to create temporary directory (%s)",
+			         knot_strerror(knot_map_errno()));
 			ret = KNOT_ENOMEM;
 			goto new_error;
 		}
-		out->api = namedb_lmdb_api();
+
 		ret = out->api->init(&out->db, out->mm, &lmdb_opts);
 
-		// Remove opened database to ensure it is temporary.
-		rm_dir(tpl);
+		// Remove the database to ensure it is temporary.
+		rm_dir(lmdb_opts.path);
 	} else {
+		// Set the specified database.
 		lmdb_opts.path = db_dir;
-		out->api = namedb_lmdb_api();
+
+		// Set the read-only mode.
+		if (flags & CONF_FREADONLY) {
+			lmdb_opts.flags.env |= KNOT_DB_LMDB_RDONLY;
+		}
+
 		ret = out->api->init(&out->db, out->mm, &lmdb_opts);
 	}
-
-	// Check database opening.
 	if (ret != KNOT_EOK) {
 		goto new_error;
 	}
 
-	// Initialize/check database.
-	namedb_txn_t txn;
-	ret = out->api->txn_begin(out->db, &txn, 0);
-	if (ret != KNOT_EOK) {
-		out->api->deinit(out->db);
-		goto new_error;
-	}
-
-	ret = conf_db_init(out, &txn);
-	if (ret != KNOT_EOK) {
-		out->api->txn_abort(&txn);
-		out->api->deinit(out->db);
-		goto new_error;
-	}
-
-	ret = out->api->txn_commit(&txn);
+	// Initialize and check the database.
+	ret = init_and_check(out, flags);
 	if (ret != KNOT_EOK) {
 		out->api->deinit(out->db);
 		goto new_error;
 	}
 
 	// Open common read-only transaction.
-	ret = out->api->txn_begin(out->db, &out->read_txn, NAMEDB_RDONLY);
+	ret = conf_refresh_txn(out);
 	if (ret != KNOT_EOK) {
 		out->api->deinit(out->db);
 		goto new_error;
 	}
 
+	// Initialize query modules list.
+	init_list(&out->query_modules);
+
+	// Cache the current hostname.
+	if (!(flags & CONF_FNOHOSTNAME)) {
+		out->hostname = sockaddr_hostname();
+	}
+
+	// Initialize cached values.
+	init_cache(out);
+
 	*conf = out;
 
 	return KNOT_EOK;
 new_error:
-	yp_scheme_free(out->scheme);
+	mp_delete(out->mm->ctx);
 	free(out->mm);
+	yp_scheme_free(out->scheme);
 	free(out);
 
 	return ret;
@@ -211,40 +289,38 @@ int conf_clone(
 	out->api = s_conf->api;
 	out->mm = s_conf->mm;
 	out->db = s_conf->db;
-	out->filename = s_conf->filename;
 
 	// Open common read-only transaction.
-	ret = out->api->txn_begin(out->db, &out->read_txn, NAMEDB_RDONLY);
+	ret = conf_refresh_txn(out);
 	if (ret != KNOT_EOK) {
 		yp_scheme_free(out->scheme);
 		free(out);
 		return ret;
 	}
 
+	// Copy the filename.
+	if (s_conf->filename != NULL) {
+		out->filename = strdup(s_conf->filename);
+	}
+
+	// Initialize query modules list.
+	init_list(&out->query_modules);
+
+	// Reuse the hostname.
+	if (s_conf->hostname != NULL) {
+		out->hostname = strdup(s_conf->hostname);
+	}
+
+	// Initialize cached values.
+	init_cache(out);
+
+	out->is_clone = true;
+
 	*conf = out;
 
 	return KNOT_EOK;
 }
 
-int conf_post_open(
-	conf_t *conf)
-{
-	if (conf == NULL) {
-		return KNOT_EINVAL;
-	}
-
-	conf->hostname = sockaddr_hostname();
-
-	int ret = conf_activate_modules(conf, NULL, &conf->query_modules,
-	                                &conf->query_plan);
-	if (ret != KNOT_EOK) {
-		free(conf->hostname);
-		return ret;
-	}
-
-	return KNOT_EOK;
-}
-
 void conf_update(
 	conf_t *conf)
 {
@@ -252,19 +328,21 @@ void conf_update(
 		return;
 	}
 
+	conf->is_clone = false;
+
 	conf_t **current_conf = &s_conf;
 	conf_t *old_conf = rcu_xchg_pointer(current_conf, conf);
 
 	synchronize_rcu();
 
 	if (old_conf) {
-		conf_free(old_conf, true);
+		old_conf->is_clone = true;
+		conf_free(old_conf);
 	}
 }
 
 void conf_free(
-	conf_t *conf,
-	bool is_clone)
+	conf_t *conf)
 {
 	if (conf == NULL) {
 		return;
@@ -272,30 +350,35 @@ void conf_free(
 
 	yp_scheme_free(conf->scheme);
 	conf->api->txn_abort(&conf->read_txn);
+	free(conf->filename);
 	free(conf->hostname);
 
-	if (conf->query_plan != NULL) {
-		conf_deactivate_modules(conf, &conf->query_modules,
-		                        conf->query_plan);
+	if (conf->io.txn != NULL) {
+		conf->api->txn_abort(conf->io.txn_stack);
 	}
 
-	if (!is_clone) {
+	conf_deactivate_modules(&conf->query_modules, &conf->query_plan);
+
+	if (!conf->is_clone) {
 		conf->api->deinit(conf->db);
+		mp_delete(conf->mm->ctx);
 		free(conf->mm);
-		free(conf->filename);
 	}
 
 	free(conf);
 }
 
-int conf_activate_modules(
+void conf_activate_modules(
 	conf_t *conf,
-	knot_dname_t *zone_name,
+	const knot_dname_t *zone_name,
 	list_t *query_modules,
 	struct query_plan **query_plan)
 {
+	int ret = KNOT_EOK;
+
 	if (conf == NULL || query_modules == NULL || query_plan == NULL) {
-		return KNOT_EINVAL;
+		ret = KNOT_EINVAL;
+		goto activate_error;
 	}
 
 	conf_val_t val;
@@ -304,19 +387,22 @@ int conf_activate_modules(
 	if (zone_name != NULL) {
 		val = conf_zone_get(conf, C_MODULE, zone_name);
 	} else {
-		val = conf_default_get(conf, C_MODULE);
+		val = conf_default_get(conf, C_GLOBAL_MODULE);
 	}
 
+	// Check if a module is configured at all.
 	if (val.code == KNOT_ENOENT) {
-		return KNOT_EOK;
+		return;
 	} else if (val.code != KNOT_EOK) {
-		return val.code;
+		ret = val.code;
+		goto activate_error;
 	}
 
 	// Create query plan.
 	*query_plan = query_plan_create(conf->mm);
 	if (*query_plan == NULL) {
-		return KNOT_ENOMEM;
+		ret = KNOT_ENOMEM;
+		goto activate_error;
 	}
 
 	if (zone_name != NULL) {
@@ -331,21 +417,51 @@ int conf_activate_modules(
 	while (val.code == KNOT_EOK) {
 		conf_mod_id_t *mod_id = conf_mod_id(&val);
 		if (mod_id == NULL) {
-			return KNOT_ENOMEM;
+			ret = KNOT_ENOMEM;
+			goto activate_error;
 		}
 
 		// Open the module.
 		struct query_module *mod = query_module_open(conf, mod_id, conf->mm);
 		if (mod == NULL) {
-			conf_free_mod_id(mod_id);
-			return KNOT_ENOMEM;
+			ret = KNOT_ENOMEM;
+			goto activate_error;
+		}
+
+		// Check the module scope.
+		if ((zone_name == NULL && (mod->scope & MOD_SCOPE_GLOBAL) == 0) ||
+		    (zone_name != NULL && (mod->scope & MOD_SCOPE_ZONE) == 0)) {
+			if (zone_name != NULL) {
+				log_zone_warning(zone_name,
+				                 "out of scope module '%s/%.*s'",
+				                 mod_id->name + 1, (int)mod_id->len,
+				                 mod_id->data);
+			} else {
+				log_warning("out of scope module '%s/%.*s'",
+				            mod_id->name + 1, (int)mod_id->len,
+				            mod_id->data);
+			}
+			query_module_close(mod);
+			conf_val_next(&val);
+			continue;
 		}
 
 		// Load the module.
-		int ret = mod->load(*query_plan, mod);
+		ret = mod->load(*query_plan, mod, zone_name);
 		if (ret != KNOT_EOK) {
+			if (zone_name != NULL) {
+				log_zone_error(zone_name,
+				               "failed to load module '%s/%.*s' (%s)",
+				               mod_id->name + 1, (int)mod_id->len,
+				               mod_id->data, knot_strerror(ret));
+			} else {
+				log_error("failed to load global module '%s/%.*s' (%s)",
+				          mod_id->name + 1, (int)mod_id->len,
+				          mod_id->data, knot_strerror(ret));
+			}
 			query_module_close(mod);
-			return ret;
+			conf_val_next(&val);
+			continue;
 		}
 
 		add_tail(query_modules, &mod->node);
@@ -353,183 +469,179 @@ int conf_activate_modules(
 		conf_val_next(&val);
 	}
 
-	return KNOT_EOK;
+	return;
+activate_error:
+	CONF_LOG(LOG_ERR, "failed to activate modules (%s)", knot_strerror(ret));
 }
 
 void conf_deactivate_modules(
-	conf_t *conf,
 	list_t *query_modules,
-	struct query_plan *query_plan)
+	struct query_plan **query_plan)
 {
-	if (conf == NULL || query_modules == NULL || query_plan == NULL) {
+	if (query_modules == NULL || query_plan == NULL) {
 		return;
 	}
 
+	// Free query plan.
+	query_plan_free(*query_plan);
+	*query_plan = NULL;
+
 	// Free query modules list.
 	struct query_module *mod = NULL, *next = NULL;
 	WALK_LIST_DELSAFE(mod, next, *query_modules) {
 		mod->unload(mod);
 		query_module_close(mod);
 	}
-
-	// Free query plan.
-	query_plan_free(query_plan);
+	init_list(query_modules);
 }
 
-static int exec_callbacks(
-	const yp_item_t *item,
-	conf_check_t *args)
-{
-	for (size_t i = 0; i < YP_MAX_MISC_COUNT; i++) {
-		conf_check_f *fcn = (conf_check_f *)item->misc[i];
-		if (fcn == NULL) {
-			break;
-		}
-		int ret;
-		if ((ret = fcn(args)) != KNOT_EOK) {
-			return ret;
-		}
-	}
-
-	return KNOT_EOK;
-}
-
-static int previous_block_calls(
-	conf_t *conf,
-	namedb_txn_t *txn,
-	conf_previous_t *prev,
-	const char **err_str)
-{
-	if (prev->id_len > 0) {
-		assert(prev->key0 != NULL);
-
-		conf_check_t args = {
-			.conf = conf,
-			.txn = txn,
-			.previous = prev,
-			.err_str = err_str
-		};
-
-		// Execute previous block callbacks.
-		int ret = exec_callbacks(prev->key0, &args);
-		if (ret != KNOT_EOK) {
-			return ret;
-		}
-	}
-
-	return KNOT_EOK;
-}
-
-static int item_calls(
-	conf_t *conf,
-	namedb_txn_t *txn,
-	yp_parser_t *parser,
-	yp_check_ctx_t *ctx,
-	conf_previous_t *prev,
-	size_t *incl_depth,
-	const char **err_str)
-{
-	conf_check_t args = {
-		.conf = conf,
-		.txn = txn,
-		.parser = parser,
-		.check = ctx,
-		.include_depth = incl_depth,
-		.previous = prev,
-		.err_str = err_str
-	};
-
-	const yp_item_t *item;
-
-	// Prepare previous context.
-	switch (ctx->event) {
-	case YP_EKEY0:
-		// Reset previous context id.
-		prev->id_len = 0;
-		// Set previous context key0 if group item.
-		if (ctx->key0->type == YP_TGRP) {
-			prev->key0 = ctx->key0;
-			return KNOT_EOK;
-		}
-		item = ctx->key0;
-		break;
-	case YP_EID:
-		memcpy(prev->id, ctx->id, ctx->id_len);
-		prev->id_len = ctx->id_len;
-		prev->file = parser->file.name;
-		prev->line = parser->line_count;
-		item = ctx->key1;
-		break;
-	default:
-		assert(ctx->event == YP_EKEY1);
-		item = ctx->key1;
-		break;
-	}
-
-	// Execute item callbacks.
-	return exec_callbacks(item, &args);
-}
-
-#define CONF_LOG_LINE(input, is_file, line, msg, ...) do { \
+#define CONF_LOG_LINE(file, line, msg, ...) do { \
 	CONF_LOG(LOG_ERR, "%s%s%sline %zu, " msg, \
-	         (is_file ? "file '" : ""), (is_file ? input : ""), \
-	         (is_file ? "', " : ""), line, ##__VA_ARGS__); \
+	         (file != NULL ? "file '" : ""), (file != NULL ? file : ""), \
+	         (file != NULL ? "', " : ""), line, ##__VA_ARGS__); \
 	} while (0)
 
-static void log_current_err(
-	const char *input,
-	bool is_file,
+static void log_parser_err(
 	yp_parser_t *parser,
-	int ret,
-	const char *err_str)
+	int ret)
 {
-	CONF_LOG_LINE(input, is_file, parser->line_count,
-	             "item '%.*s', value '%.*s' (%s)",
-	             (int)parser->key_len, parser->key,
-	             (int)parser->data_len, parser->data,
-	             (err_str != NULL ? err_str : knot_strerror(ret)));
+	CONF_LOG_LINE(parser->file.name, parser->line_count,
+	              "item '%.*s', value '%.*s' (%s)",
+	              (int)parser->key_len, parser->key,
+	              (int)parser->data_len, parser->data,
+	              knot_strerror(ret));
+}
+
+static void log_call_err(
+	yp_parser_t *parser,
+	conf_check_t *args,
+	int ret)
+{
+	CONF_LOG_LINE(args->file_name, args->line,
+	              "item '%s', value '%.*s' (%s)", args->item->name + 1,
+	              (int)parser->data_len, parser->data,
+	              args->err_str != NULL ? args->err_str : knot_strerror(ret));
 }
 
 static void log_prev_err(
-	const char *input,
-	bool is_file,
-	struct conf_previous *prev,
-	int ret,
-	const char *err_str)
+	conf_check_t *args,
+	int ret)
 {
-	char buff[512];
+	char buff[512] = { 0 };
 	size_t len = sizeof(buff);
 
-	// Get textual previous identifier.
-	if (yp_item_to_txt(prev->key0->var.g.id, prev->id, prev->id_len,
-	                   buff, &len, YP_SNOQUOTE) != KNOT_EOK) {
-		buff[0] = '\0';
+	// Get the previous textual identifier.
+	if ((args->item->flags & YP_FMULTI) != 0) {
+		if (yp_item_to_txt(args->item->var.g.id, args->id, args->id_len,
+		                   buff, &len, YP_SNOQUOTE) != KNOT_EOK) {
+			buff[0] = '\0';
+		}
 	}
 
-	CONF_LOG_LINE(input, is_file, prev->line, "%s '%s' (%s)",
-	              prev->key0->name + 1, buff,
-	              (err_str != NULL ? err_str : knot_strerror(ret)));
+	CONF_LOG_LINE(args->file_name, args->line,
+	              "%s '%s' (%s)", args->item->name + 1, buff,
+	              args->err_str != NULL ? args->err_str : knot_strerror(ret));
+}
+
+static int parser_calls(
+	conf_t *conf,
+	knot_db_txn_t *txn,
+	yp_parser_t *parser,
+	yp_check_ctx_t *ctx)
+{
+	static const yp_item_t *prev_item = NULL;
+	static size_t prev_id_len = 0;
+	static uint8_t prev_id[YP_MAX_ID_LEN] = { 0 };
+	static const char *prev_file_name = NULL;
+	static size_t prev_line = 0;
+
+	// Zero ctx means the final previous processing.
+	yp_node_t *node = (ctx != NULL) ? &ctx->nodes[ctx->current] : NULL;
+	bool is_id = false;
+
+	// Preprocess key0 item.
+	if (node == NULL || node->parent == NULL) {
+		// Execute previous block callbacks.
+		if (prev_item != NULL) {
+			conf_check_t args = {
+				.conf = conf,
+				.txn = txn,
+				.item = prev_item,
+				.id = prev_id,
+				.id_len = prev_id_len,
+				.file_name = prev_file_name,
+				.line = prev_line
+			};
+
+			int ret = conf_exec_callbacks(prev_item, &args);
+			if (ret != KNOT_EOK) {
+				log_prev_err(&args, ret);
+				return ret;
+			}
+
+			prev_item = NULL;
+		}
+
+		// Stop if final processing.
+		if (node == NULL) {
+			return KNOT_EOK;
+		}
+
+		// Store block context.
+		if (node->item->type == YP_TGRP) {
+			// Ignore alone group without identifier.
+			if ((node->item->flags & YP_FMULTI) != 0 &&
+			    node->id_len == 0) {
+				return KNOT_EOK;
+			}
+
+			prev_item = node->item;
+			memcpy(prev_id, node->id, node->id_len);
+			prev_id_len = node->id_len;
+			prev_file_name = parser->file.name;
+			prev_line = parser->line_count;
+
+			// Defer group callbacks to the beginning of the next block.
+			if ((node->item->flags & YP_FMULTI) == 0) {
+				return KNOT_EOK;
+			}
+
+			is_id = true;
+		}
+	}
+
+	conf_check_t args = {
+		.conf = conf,
+		.txn = txn,
+		.item = is_id ? node->item->var.g.id : node->item,
+		.id = node->id,
+		.id_len = node->id_len,
+		.data = node->data,
+		.data_len = node->data_len,
+		.file_name = parser->file.name,
+		.line = parser->line_count
+	};
+
+	int ret = conf_exec_callbacks(is_id ? node->item->var.g.id : node->item,
+	                              &args);
+	if (ret != KNOT_EOK) {
+		log_call_err(parser, &args, ret);
+	}
+
+	return ret;
 }
 
 int conf_parse(
 	conf_t *conf,
-	namedb_txn_t *txn,
+	knot_db_txn_t *txn,
 	const char *input,
 	bool is_file,
-	size_t *incl_depth,
-	struct conf_previous *prev)
+	void *data)
 {
-	if (conf == NULL || txn == NULL || input == NULL ||
-	    incl_depth == NULL || prev == NULL) {
+	if (conf == NULL || txn == NULL || input == NULL || data == NULL) {
 		return KNOT_EINVAL;
 	}
 
-	// Check for include loop.
-	if ((*incl_depth)++ > MAX_INCLUDE_DEPTH) {
-		CONF_LOG(LOG_ERR, "include loop detected");
-		return KNOT_EPARSEFAIL;
-	}
-
 	yp_parser_t *parser = malloc(sizeof(yp_parser_t));
 	if (parser == NULL) {
 		return KNOT_ENOMEM;
@@ -558,50 +670,50 @@ int conf_parse(
 	}
 
 	int check_ret = KNOT_EOK;
-	const char *err_str = NULL;
 
 	// Parse the configuration.
 	while ((ret = yp_parse(parser)) == KNOT_EOK) {
 		check_ret = yp_scheme_check_parser(ctx, parser);
 		if (check_ret != KNOT_EOK) {
-			log_current_err(input, is_file, parser, check_ret, NULL);
+			log_parser_err(parser, check_ret);
 			break;
 		}
-		check_ret = conf_db_set(conf, txn, ctx);
+
+		yp_node_t *node = &ctx->nodes[ctx->current];
+		yp_node_t *parent = node->parent;
+
+		if (parent == NULL) {
+			check_ret = conf_db_set(conf, txn, node->item->name,
+			                        NULL, node->id, node->id_len,
+			                        node->data, node->data_len);
+		} else {
+			check_ret = conf_db_set(conf, txn, parent->item->name,
+			                        node->item->name, parent->id,
+			                        parent->id_len, node->data,
+			                        node->data_len);
+		}
 		if (check_ret != KNOT_EOK) {
-			log_current_err(input, is_file, parser, check_ret, NULL);
+			log_parser_err(parser, check_ret);
 			break;
 		}
-		if (ctx->event != YP_EKEY1) {
-			check_ret = previous_block_calls(conf, txn, prev, &err_str);
-			if (check_ret != KNOT_EOK) {
-				log_prev_err(input, is_file, prev, check_ret, err_str);
-				break;
-			}
-		}
-		check_ret = item_calls(conf, txn, parser, ctx, prev, incl_depth,
-		                       &err_str);
+
+		check_ret = parser_calls(conf, txn, parser, ctx);
 		if (check_ret != KNOT_EOK) {
-			log_current_err(input, is_file, parser, check_ret, err_str);
 			break;
 		}
 	}
 
 	if (ret == KNOT_EOF) {
 		// Call the last block callbacks.
-		ret = previous_block_calls(conf, txn, prev, &err_str);
-		if (ret != KNOT_EOK) {
-			log_prev_err(input, is_file, prev, ret, err_str);
-		}
+		ret = parser_calls(conf, txn, NULL, NULL);
 	} else if (ret != KNOT_EOK) {
-		log_current_err(input, is_file, parser, ret, NULL);
+		log_parser_err(parser, ret);
 	} else {
 		ret = check_ret;
 	}
 
 	yp_scheme_check_deinit(ctx);
 parse_error:
-	(*incl_depth)--;
 	yp_deinit(parser);
 	free(parser);
 
@@ -617,77 +729,130 @@ int conf_import(
 		return KNOT_EINVAL;
 	}
 
-	namedb_txn_t txn;
-	int ret = conf->api->txn_begin(conf->db, &txn, 0);
+	int ret;
+
+	knot_db_txn_t txn;
+	ret = conf->api->txn_begin(conf->db, &txn, 0);
 	if (ret != KNOT_EOK) {
-		return ret;
+		goto import_error;
 	}
 
-	// Drop the current DB content.
-	ret = conf->api->clear(&txn);
+	// Initialize the DB.
+	ret = conf_db_init(conf, &txn, true);
 	if (ret != KNOT_EOK) {
 		conf->api->txn_abort(&txn);
-		return ret;
+		goto import_error;
 	}
 
-	// Initialize new DB.
-	ret = conf_db_init(conf, &txn);
-	if (ret != KNOT_EOK) {
-		conf->api->txn_abort(&txn);
-		return ret;
-	}
-
-	size_t depth = 0;
-	conf_previous_t prev = { NULL };
+	conf_check_t args = { NULL };
 
 	// Parse and import given file.
-	ret = conf_parse(conf, &txn, input, is_file, &depth, &prev);
+	ret = conf_parse(conf, &txn, input, is_file, &args);
 	if (ret != KNOT_EOK) {
 		conf->api->txn_abort(&txn);
-		return ret;
+		goto import_error;
 	}
 
 	// Commit new configuration.
 	ret = conf->api->txn_commit(&txn);
 	if (ret != KNOT_EOK) {
-		return ret;
+		goto import_error;
 	}
 
 	// Update read-only transaction.
-	conf->api->txn_abort(&conf->read_txn);
-	ret = conf->api->txn_begin(conf->db, &conf->read_txn, NAMEDB_RDONLY);
+	ret = conf_refresh_txn(conf);
+	if (ret != KNOT_EOK) {
+		goto import_error;
+	}
+
+	// Update cached values.
+	init_cache(conf);
+
+	// Reset the filename.
+	free(conf->filename);
+	conf->filename = NULL;
+	if (is_file) {
+		conf->filename = strdup(input);
+	}
+
+	ret = KNOT_EOK;
+import_error:
+
+	return ret;
+}
+
+static int export_group_name(
+	FILE *fp,
+	const yp_item_t *group,
+	char *out,
+	size_t out_len,
+	yp_style_t style)
+{
+	int ret = yp_format_key0(group, NULL, 0, out, out_len, style, true, true);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
 
+	fprintf(fp, "%s", out);
+
 	return KNOT_EOK;
 }
 
 static int export_group(
 	conf_t *conf,
 	FILE *fp,
-	yp_item_t *group,
-	uint8_t *id,
+	const yp_item_t *group,
+	const uint8_t *id,
 	size_t id_len,
 	char *out,
 	size_t out_len,
-	yp_style_t style)
+	yp_style_t style,
+	bool *exported)
 {
-	yp_item_t *item;
-	for (item = group->sub_items; item->name != NULL; item++) {
+	// Export the multi-group name.
+	if ((group->flags & YP_FMULTI) != 0 && !(*exported)) {
+		int ret = export_group_name(fp, group, out, out_len, style);
+		if (ret != KNOT_EOK) {
+			return ret;
+		}
+		*exported = true;
+	}
+
+	// Iterate through all possible group items.
+	for (yp_item_t *item = group->sub_items; item->name != NULL; item++) {
+		// Export the identifier.
+		if (group->var.g.id == item && (group->flags & YP_FMULTI) != 0) {
+			int ret = yp_format_id(group->var.g.id, id, id_len, out,
+			                       out_len, style);
+			if (ret != KNOT_EOK) {
+				return ret;
+			}
+			fprintf(fp, "%s", out);
+			continue;
+		}
+
 		conf_val_t bin;
-		bin.code = conf_db_get(conf, &conf->read_txn, group->name,
-		                       item->name, id, id_len, &bin);
+		conf_db_get(conf, &conf->read_txn, group->name, item->name,
+		            id, id_len, &bin);
 		if (bin.code == KNOT_ENOENT) {
 			continue;
 		} else if (bin.code != KNOT_EOK) {
 			return bin.code;
 		}
 
+		// Export the single-group name if an item is set.
+		if ((group->flags & YP_FMULTI) == 0 && !(*exported)) {
+			int ret = export_group_name(fp, group, out, out_len, style);
+			if (ret != KNOT_EOK) {
+				return ret;
+			}
+			*exported = true;
+		}
+
 		// Format single/multiple-valued item.
 		size_t values = conf_val_count(&bin);
 		for (size_t i = 1; i <= values; i++) {
-			conf_db_val(&bin);
+			conf_val(&bin);
 			int ret = yp_format_key1(item, bin.data, bin.len, out,
 			                         out_len, style, i == 1,
 			                         i == values);
@@ -702,7 +867,9 @@ static int export_group(
 		}
 	}
 
-	fprintf(fp, "\n");
+	if (*exported) {
+		fprintf(fp, "\n");
+	}
 
 	return KNOT_EOK;
 }
@@ -726,41 +893,26 @@ int conf_export(
 	FILE *fp = fopen(file_name, "w");
 	if (fp == NULL) {
 		free(buff);
-		return KNOT_EFILE;
+		return knot_map_errno();
 	}
 
+	fprintf(fp, "# Configuration export (Knot DNS %s)\n\n", PACKAGE_VERSION);
+
 	int ret;
 
-	// Iterate over the current scheme.
-	yp_item_t *item;
-	for (item = conf->scheme; item->name != NULL; item++) {
+	// Iterate over the scheme.
+	for (yp_item_t *item = conf->scheme; item->name != NULL; item++) {
+		bool exported = false;
+
 		// Skip non-group items (include).
 		if (item->type != YP_TGRP) {
 			continue;
 		}
 
-		// Check if the item is ever stored in DB.
-		uint8_t item_code;
-		ret = conf_db_code(conf, &conf->read_txn, CONF_CODE_KEY0_ROOT,
-		                   item->name, true, &item_code);
-		if (ret == KNOT_ENOENT) {
-			continue;
-		} else if (ret != KNOT_EOK) {
-			goto export_error;
-		}
-
-		// Export group name.
-		ret = yp_format_key0(item, NULL, 0, buff, buff_len, style,
-		                     true, true);
-		if (ret != KNOT_EOK) {
-			goto export_error;
-		}
-		fprintf(fp, "%s", buff);
-
 		// Export simple group without identifiers.
-		if (item->var.g.id == NULL) {
+		if ((item->flags & YP_FMULTI) == 0) {
 			ret = export_group(conf, fp, item, NULL, 0, buff,
-			                   buff_len, style);
+			                   buff_len, style, &exported);
 			if (ret != KNOT_EOK) {
 				goto export_error;
 			}
@@ -770,14 +922,18 @@ int conf_export(
 
 		// Iterate over all identifiers.
 		conf_iter_t iter;
-		ret = conf_db_iter_begin(conf, &conf->read_txn, item->name,
-		                         &iter);
-		if (ret != KNOT_EOK) {
+		ret = conf_db_iter_begin(conf, &conf->read_txn, item->name, &iter);
+		switch (ret) {
+		case KNOT_EOK:
+			break;
+		case KNOT_ENOENT:
+			continue;
+		default:
 			goto export_error;
 		}
 
 		while (ret == KNOT_EOK) {
-			uint8_t *id;
+			const uint8_t *id;
 			size_t id_len;
 			ret = conf_db_iter_id(conf, &iter, &id, &id_len);
 			if (ret != KNOT_EOK) {
@@ -785,18 +941,9 @@ int conf_export(
 				goto export_error;
 			}
 
-			// Export identifier.
-			ret = yp_format_id(item->var.g.id, id, id_len, buff,
-			                   buff_len, style);
-			if (ret != KNOT_EOK) {
-				conf_db_iter_finish(conf, &iter);
-				goto export_error;
-			}
-			fprintf(fp, "%s", buff);
-
-			// Export other items.
+			// Export group with identifiers.
 			ret = export_group(conf, fp, item, id, id_len, buff,
-			                   buff_len, style);
+			                   buff_len, style, &exported);
 			if (ret != KNOT_EOK) {
 				conf_db_iter_finish(conf, &iter);
 				goto export_error;
@@ -804,8 +951,9 @@ int conf_export(
 
 			ret = conf_db_iter_next(conf, &iter);
 		}
-
-		conf_db_iter_finish(conf, &iter);
+		if (ret != KNOT_EOF) {
+			goto export_error;
+		}
 	}
 
 	ret = KNOT_EOK;
