@@ -17,8 +17,6 @@
 #include <assert.h>
 
 #include "dnssec/nsec.h"
-#include "libknot/internal/base32hex.h"
-#include "libknot/internal/macros.h"
 #include "knot/dnssec/nsec3-chain.h"
 #include "libknot/dname.h"
 #include "libknot/packet/wire.h"
@@ -28,17 +26,9 @@
 #include "knot/dnssec/nsec-chain.h"
 #include "knot/dnssec/zone-sign.h"
 #include "knot/dnssec/zone-nsec.h"
-
-/* - Forward declarations --------------------------------------------------- */
-
-static int create_nsec3_rrset(knot_rrset_t *rrset,
-                              knot_dname_t *dname,
-                              const knot_nsec3_params_t *,
-                              const dnssec_nsec_bitmap_t *,
-                              const uint8_t *,
-                              uint32_t);
-
-/* - Helper functions ------------------------------------------------------- */
+#include "contrib/base32hex.h"
+#include "contrib/macros.h"
+#include "contrib/wire_ctx.h"
 
 /* - NSEC3 node comparison -------------------------------------------------- */
 
@@ -208,9 +198,10 @@ static size_t nsec3_rdata_size(const knot_nsec3_params_t *params,
  *
  * \note Content of next hash field is not changed.
  */
-static void nsec3_fill_rdata(uint8_t *rdata, const knot_nsec3_params_t *params,
-                             const dnssec_nsec_bitmap_t *rr_types,
-                             const uint8_t *next_hashed, uint32_t ttl)
+static int nsec3_fill_rdata(uint8_t *rdata, size_t rdata_len,
+                            const knot_nsec3_params_t *params,
+                            const dnssec_nsec_bitmap_t *rr_types,
+                            const uint8_t *next_hashed)
 {
 	assert(rdata);
 	assert(params);
@@ -218,24 +209,28 @@ static void nsec3_fill_rdata(uint8_t *rdata, const knot_nsec3_params_t *params,
 
 	uint8_t hash_length = dnssec_nsec3_hash_length(params->algorithm);
 
-	*rdata = params->algorithm;                       // hash algorithm
-	rdata += 1;
-	*rdata = 0;                                       // flags
-	rdata += 1;
-	wire_write_u16(rdata, params->iterations);        // iterations
-	rdata += 2;
-	*rdata = params->salt_length;                     // salt length
-	rdata += 1;
-	memcpy(rdata, params->salt, params->salt_length); // salt
-	rdata += params->salt_length;
-	*rdata = hash_length;                             // hash length
-	rdata += 1;
-	/*memset(rdata, '\0', hash_len);*/                // hash (unknown)
-	if (next_hashed) {
-		memcpy(rdata, next_hashed, hash_length);
+	wire_ctx_t wire = wire_ctx_init(rdata, rdata_len);
+
+	wire_ctx_write_u8(&wire, params->algorithm);			// hash algorithm
+	wire_ctx_write_u8(&wire, 0);					// flags
+	wire_ctx_write_u16(&wire, params->iterations);			// itertions
+	wire_ctx_write_u8(&wire, params->salt_length);			// salt length
+	wire_ctx_write(&wire, params->salt, params->salt_length);	// salt
+	wire_ctx_write_u8(&wire, hash_length);				// hash length
+
+	if (next_hashed != NULL) {
+		wire_ctx_write(&wire, next_hashed, hash_length);	// hash
+	} else {
+		wire_ctx_skip(&wire, hash_length);
 	}
-	rdata += hash_length;
-	dnssec_nsec_bitmap_write(rr_types, rdata);        // RR types bit map
+
+	if (wire.error != KNOT_EOK) {
+		return wire.error;
+	}
+
+	dnssec_nsec_bitmap_write(rr_types, wire.position);		// RR types bit map
+
+	return KNOT_EOK;
 }
 
 /*!
@@ -265,7 +260,11 @@ static int create_nsec3_rrset(knot_rrset_t *rrset,
 
 	size_t rdata_size = nsec3_rdata_size(params, rr_types);
 	uint8_t rdata[rdata_size];
-	nsec3_fill_rdata(rdata, params, rr_types, next_hashed, ttl);
+	int ret = nsec3_fill_rdata(rdata, rdata_size, params, rr_types,
+	                           next_hashed);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
 
 	return knot_rrset_add_rdata(rrset, rdata, rdata_size, ttl, NULL);
 }
