@@ -1,11 +1,3 @@
-/*!
- * \file modules/dnstap.h
- *
- * \author Marek Vavrusa <marek.vavrusa@nic.cz>
- *
- * \addtogroup query_processing
- * @{
- */
 /*  Copyright (C) 2014 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
@@ -21,6 +13,12 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
+/*!
+ * \file
+ *
+ * \addtogroup query_processing
+ * @{
+ */
 
 #pragma once
 
@@ -29,9 +27,11 @@
 /*! \brief Module scheme. */
 #define C_MOD_DNSTAP "\x0A""mod-dnstap"
 extern const yp_item_t scheme_mod_dnstap[];
+int check_mod_dnstap(conf_check_t *args);
 
 /*! \brief Module interface. */
-int dnstap_load(struct query_plan *plan, struct query_module *self);
+int dnstap_load(struct query_plan *plan, struct query_module *self,
+                const knot_dname_t *zone);
 int dnstap_unload(struct query_module *self);
 
 /*! @} */
