@@ -19,6 +19,9 @@
 #include "knot/nameserver/internet.h"
 #include "knot/common/log.h"
 #include "libknot/descriptor.h"
+#include "contrib/mempattern.h"
+#include "contrib/net.h"
+#include "contrib/sockaddr.h"
 
 /* Module configuration scheme. */
 #define MOD_NET		"\x07""network"
@@ -34,37 +37,61 @@ enum synth_template_type {
 	SYNTH_REVERSE = 2
 };
 
-static const lookup_table_t synthetic_types[] = {
+static const knot_lookup_t synthetic_types[] = {
 	{ SYNTH_FORWARD, "forward" },
 	{ SYNTH_REVERSE, "reverse" },
 	{ 0, NULL }
 };
 
+int check_prefix(conf_check_t *args)
+{
+	if (strchr((const char *)args->data, '.') != NULL) {
+		args->err_str = "dot '.' is not allowed";
+		return KNOT_EINVAL;
+	}
+
+	return KNOT_EOK;
+}
+
 const yp_item_t scheme_mod_synth_record[] = {
 	{ C_ID,       YP_TSTR,   YP_VNONE },
 	{ MOD_TYPE,   YP_TOPT,   YP_VOPT = { synthetic_types, SYNTH_NULL } },
-	{ MOD_PREFIX, YP_TSTR,   YP_VNONE },
+	{ MOD_PREFIX, YP_TSTR,   YP_VSTR = { "" }, YP_FNONE, { check_prefix } },
 	{ MOD_ORIGIN, YP_TDNAME, YP_VNONE },
 	{ MOD_TTL,    YP_TINT,   YP_VINT = { 0, UINT32_MAX, 3600, YP_STIME } },
-	{ MOD_NET,    YP_TNET,   YP_VNONE },
+	{ MOD_NET,    YP_TDATA,  YP_VDATA = { 0, NULL, addr_range_to_bin,
+	                                      addr_range_to_txt }, YP_FMULTI },
 	{ C_COMMENT,  YP_TSTR,   YP_VNONE },
 	{ NULL }
 };
 
 int check_mod_synth_record(conf_check_t *args)
 {
-	const char *err_str = "origin with non-reverse type";
-
+	// Check type.
 	conf_val_t type = conf_rawid_get_txn(args->conf, args->txn, C_MOD_SYNTH_RECORD,
-	                                    MOD_TYPE, args->previous->id,
-	                                    args->previous->id_len);
-	conf_val_t origin = conf_rawid_get_txn(args->conf, args->txn, C_MOD_SYNTH_RECORD,
-	                                    MOD_ORIGIN, args->previous->id,
-	                                    args->previous->id_len);
+	                                     MOD_TYPE, args->id, args->id_len);
+	if (type.code != KNOT_EOK) {
+		args->err_str = "no synthesis type specified";
+		return KNOT_EINVAL;
+	}
 
-	// Origin is required only with reverse zone.
-	if (conf_opt(&type) != SYNTH_REVERSE && conf_dname(&origin) != NULL) {
-		*args->err_str = err_str;
+	// Check origin.
+	conf_val_t origin = conf_rawid_get_txn(args->conf, args->txn, C_MOD_SYNTH_RECORD,
+	                                       MOD_ORIGIN, args->id, args->id_len);
+	if (origin.code != KNOT_EOK && conf_opt(&type) == SYNTH_REVERSE) {
+		args->err_str = "no origin specified";
+		return KNOT_EINVAL;
+	}
+	if (origin.code == KNOT_EOK && conf_opt(&type) == SYNTH_FORWARD) {
+		args->err_str = "origin not allowed with forward type";
+		return KNOT_EINVAL;
+	}
+
+	// Check network subnet.
+	conf_val_t net = conf_rawid_get_txn(args->conf, args->txn, C_MOD_SYNTH_RECORD,
+	                                    MOD_NET, args->id, args->id_len);
+	if (net.code != KNOT_EOK) {
+		args->err_str = "no network subnet specified";
 		return KNOT_EINVAL;
 	}
 
@@ -73,7 +100,6 @@ int check_mod_synth_record(conf_check_t *args)
 
 /* Defines. */
 #define ARPA_ZONE_LABELS 2
-#define MODULE_ERR(msg, ...) log_error("module 'synth_record', " msg, ##__VA_ARGS__)
 
 /*!
  * \brief Synthetic response template.
@@ -85,6 +111,7 @@ typedef struct synth_template {
 	char *zone;
 	uint32_t ttl;
 	struct sockaddr_storage addr;
+	struct sockaddr_storage addr_max;
 	int mask;
 } synth_template_t;
 
@@ -311,8 +338,14 @@ static int template_match(int state, synth_template_t *tpl, knot_pkt_t *pkt, str
 	if (ret != KNOT_EOK) {
 		return state;
 	}
-	if (!netblock_match(&tpl->addr, &query_addr, tpl->mask)) {
-		return state; /* Out of our netblock, not applicable. */
+	if (tpl->addr_max.ss_family == AF_UNSPEC) {
+		if (!netblock_match(&query_addr, &tpl->addr, tpl->mask)) {
+			return state; /* Out of our netblock, not applicable. */
+		}
+	} else {
+		if (!netrange_match(&query_addr, &tpl->addr, &tpl->addr_max)) {
+			return state; /* Out of our netblock, not applicable. */
+		}
 	}
 
 	/* Check if the request is for an available query type. */
@@ -367,7 +400,8 @@ static int solve_synth_record(int state, knot_pkt_t *pkt, struct query_data *qda
 	return template_match(state, (synth_template_t *)ctx, pkt, qdata);
 }
 
-int synth_record_load(struct query_plan *plan, struct query_module *self)
+int synth_record_load(struct query_plan *plan, struct query_module *self,
+                      const knot_dname_t *zone)
 {
 	if (plan == NULL || self == NULL) {
 		return KNOT_EINVAL;
@@ -376,7 +410,6 @@ int synth_record_load(struct query_plan *plan, struct query_module *self)
 	/* Create synthesis template. */
 	struct synth_template *tpl = mm_alloc(self->mm, sizeof(struct synth_template));
 	if (tpl == NULL) {
-		MODULE_ERR("not enough memory");
 		return KNOT_ENOMEM;
 	}
 
@@ -384,49 +417,17 @@ int synth_record_load(struct query_plan *plan, struct query_module *self)
 
 	/* Set type. */
 	val = conf_mod_get(self->config, MOD_TYPE, self->id);
-	if (val.code != KNOT_EOK) {
-		if (val.code == KNOT_EINVAL) {
-			MODULE_ERR("no type for '%s'", self->id->data);
-		}
-		mm_free(self->mm, tpl);
-		return val.code;
-	}
 	tpl->type = conf_opt(&val);
 
 	/* Set prefix. */
 	val = conf_mod_get(self->config, MOD_PREFIX, self->id);
-	if (val.code != KNOT_EOK) {
-		if (val.code == KNOT_EINVAL) {
-			MODULE_ERR("no prefix for '%s'", self->id->data);
-		}
-		mm_free(self->mm, tpl);
-		return val.code;
-	}
-
-	const char *prefix = conf_str(&val);
-	if (strchr(prefix, '.') != NULL) {
-		MODULE_ERR("dots '.' are not allowed in the prefix for '%s'",
-		           self->id->data);
-		mm_free(self->mm, tpl);
-		return KNOT_EMALF;
-	}
-	tpl->prefix = strdup(prefix);
+	tpl->prefix = strdup(conf_str(&val));
 
 	/* Set origin if generating reverse record. */
 	if (tpl->type == SYNTH_REVERSE) {
 		val = conf_mod_get(self->config, MOD_ORIGIN, self->id);
-		if (val.code != KNOT_EOK) {
-			if (val.code == KNOT_EINVAL) {
-				MODULE_ERR("no origin for '%s'", self->id->data);
-			}
-			free(tpl->prefix);
-			mm_free(self->mm, tpl);
-			return val.code;
-		}
-
 		tpl->zone = knot_dname_to_str_alloc(conf_dname(&val));
 		if (tpl->zone == NULL) {
-			MODULE_ERR("not enough memory");
 			free(tpl->prefix);
 			mm_free(self->mm, tpl);
 			return KNOT_ENOMEM;
@@ -435,29 +436,11 @@ int synth_record_load(struct query_plan *plan, struct query_module *self)
 
 	/* Set ttl. */
 	val = conf_mod_get(self->config, MOD_TTL, self->id);
-	if (val.code != KNOT_EOK) {
-		if (val.code == KNOT_EINVAL) {
-			MODULE_ERR("no ttl for '%s'", self->id->data);
-		}
-		free(tpl->zone);
-		free(tpl->prefix);
-		mm_free(self->mm, tpl);
-		return val.code;
-	}
 	tpl->ttl = conf_int(&val);
 
 	/* Set address. */
 	val = conf_mod_get(self->config, MOD_NET, self->id);
-	if (val.code != KNOT_EOK) {
-		if (val.code == KNOT_EINVAL) {
-			MODULE_ERR("no network for '%s'", self->id->data);
-		}
-		free(tpl->zone);
-		free(tpl->prefix);
-		mm_free(self->mm, tpl);
-		return val.code;
-	}
-	tpl->addr = conf_net(&val, &tpl->mask);
+	tpl->addr = conf_addr_range(&val, &tpl->addr_max, &tpl->mask);
 
 	self->ctx = tpl;
 
