@@ -1,4 +1,4 @@
-/*  Copyright (C) 2011 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 /*!
- * \file notify.h
- *
- * \author Lubos Slovak <lubos.slovak@nic.cz>
+ * \file
  *
  * \brief NOTIFY request/reply API.
  *
@@ -26,14 +24,9 @@
 
 #pragma once
 
-#include <stdint.h>
-#include <string.h>
-
 #include "libknot/packet/pkt.h"
-#include "knot/zone/contents.h"
-
-struct query_data;
-struct answer_data;
+#include "knot/nameserver/process_answer.h"
+#include "knot/nameserver/process_query.h"
 
 #define NOTIFY_TIMEOUT 3 /*!< Interval between NOTIFY retries. */
 
