@@ -1,7 +1,24 @@
-#include <stdio.h>
+/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+
+    This program is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include <urcu.h>
 
 #include "dnssec/tsig.h"
+#include "knot/common/log.h"
+#include "knot/conf/conf.h"
 #include "knot/nameserver/process_query.h"
 #include "knot/nameserver/query_module.h"
 #include "knot/nameserver/chaos.h"
@@ -11,13 +28,9 @@
 #include "knot/nameserver/update.h"
 #include "knot/nameserver/nsec_proofs.h"
 #include "knot/nameserver/notify.h"
-#include "knot/server/server.h"
-#include "knot/server/rrl.h"
-#include "knot/updates/acl.h"
-#include "knot/conf/conf.h"
-#include "knot/common/debug.h"
 #include "libknot/libknot.h"
-#include "libknot/internal/macros.h"
+#include "contrib/macros.h"
+#include "contrib/mempattern.h"
 
 /*! \brief Accessor to query-specific data. */
 #define QUERY_DATA(ctx) ((struct query_data *)(ctx)->data)
@@ -111,23 +124,22 @@ static int query_internet(knot_pkt_t *pkt, knot_layer_t *ctx)
 {
 	struct query_data *data = QUERY_DATA(ctx);
 	int next_state = KNOT_STATE_FAIL;
-	dbg_ns("%s(%p, %p, pkt_type=%u)\n", __func__, pkt, ctx, data->packet_type);
 
 	switch(data->packet_type) {
 	case KNOT_QUERY_NORMAL:
-		next_state = internet_query(pkt, data);
+		next_state = internet_process_query(pkt, data);
 		break;
 	case KNOT_QUERY_NOTIFY:
 		next_state = notify_process_query(pkt, data);
 		break;
 	case KNOT_QUERY_AXFR:
-		next_state = axfr_query_process(pkt, data);
+		next_state = axfr_process_query(pkt, data);
 		break;
 	case KNOT_QUERY_IXFR:
-		next_state = ixfr_query(pkt, data);
+		next_state = ixfr_process_query(pkt, data);
 		break;
 	case KNOT_QUERY_UPDATE:
-		next_state = update_query_process(pkt, data);
+		next_state = update_process_query(pkt, data);
 		break;
 	default:
 		/* Nothing else is supported. */
@@ -144,7 +156,6 @@ static int query_internet(knot_pkt_t *pkt, knot_layer_t *ctx)
  */
 static int query_chaos(knot_pkt_t *pkt, knot_layer_t *ctx)
 {
-	dbg_ns("%s(%p, %p)\n", __func__, pkt, ctx);
 	struct query_data *data = QUERY_DATA(ctx);
 
 	/* Nothing except normal queries is supported. */
@@ -155,7 +166,6 @@ static int query_chaos(knot_pkt_t *pkt, knot_layer_t *ctx)
 
 	data->rcode = knot_chaos_answer(pkt);
 	if (data->rcode != KNOT_RCODE_NOERROR) {
-		dbg_ns("%s: failed with RCODE=%d\n", __func__, data->rcode);
 		return KNOT_STATE_FAIL;
 	}
 
@@ -220,8 +230,8 @@ static int answer_edns_init(const knot_pkt_t *query, knot_pkt_t *resp,
 	}
 
 	/* Initialize OPT record. */
-	conf_val_t val = conf_get(conf(), C_SRV, C_MAX_UDP_PAYLOAD);
-	int ret = knot_edns_init(&qdata->opt_rr, conf_int(&val), 0,
+	conf_val_t *max_payload = &conf()->cache.srv_max_udp_payload;
+	int ret = knot_edns_init(&qdata->opt_rr, conf_int(max_payload), 0,
 	                     KNOT_EDNS_VERSION, qdata->mm);
 	if (ret != KNOT_EOK) {
 		return ret;
@@ -238,10 +248,12 @@ static int answer_edns_init(const knot_pkt_t *query, knot_pkt_t *resp,
 	}
 
 	/* Append NSID if requested and available. */
-	val = conf_get(conf(), C_SRV, C_NSID);
-	if (knot_edns_has_nsid(query->opt_rr)) {
-		conf_data(&val);
-		if (val.code != KNOT_EOK) {
+	if (knot_edns_has_option(query->opt_rr, KNOT_EDNS_OPTION_NSID)) {
+		conf_val_t *nsid = &conf()->cache.srv_nsid;
+		size_t nsid_len;
+		const uint8_t *nsid_data = conf_bin(nsid, &nsid_len);
+
+		if (nsid->code != KNOT_EOK) {
 			ret = knot_edns_add_option(&qdata->opt_rr,
 			                           KNOT_EDNS_OPTION_NSID,
 			                           strlen(conf()->hostname),
@@ -250,11 +262,10 @@ static int answer_edns_init(const knot_pkt_t *query, knot_pkt_t *resp,
 			if (ret != KNOT_EOK) {
 				return ret;
 			}
-		} else if (val.len > 0) {
+		} else if (nsid_len > 0) {
 			ret = knot_edns_add_option(&qdata->opt_rr,
 			                           KNOT_EDNS_OPTION_NSID,
-			                           val.len,
-			                           val.data,
+			                           nsid_len, nsid_data,
 			                           qdata->mm);
 			if (ret != KNOT_EOK) {
 				return ret;
@@ -299,14 +310,12 @@ static int prepare_answer(const knot_pkt_t *query, knot_pkt_t *resp, knot_layer_
 	/* Initialize response. */
 	int ret = knot_pkt_init_response(resp, query);
 	if (ret != KNOT_EOK) {
-		dbg_ns("%s: can't init response pkt (%d)\n", __func__, ret);
 		return ret;
 	}
 
 	/* Query MUST carry a question. */
 	const knot_dname_t *qname = knot_pkt_qname(query);
 	if (qname == NULL) {
-		dbg_ns("%s: query missing QNAME, FORMERR\n", __func__);
 		qdata->rcode = KNOT_RCODE_FORMERR;
 		return KNOT_EMALF;
 	}
@@ -317,7 +326,6 @@ static int prepare_answer(const knot_pkt_t *query, knot_pkt_t *resp, knot_layer_
 	memcpy(qdata->orig_qname, qname, query->qname_size);
 	ret = process_query_qname_case_lower((knot_pkt_t *)query);
 	if (ret != KNOT_EOK) {
-		dbg_ns("%s: can't convert QNAME to lowercase (%d)\n", __func__, ret);
 		return ret;
 	}
 	/* Find zone for QNAME. */
@@ -334,9 +342,9 @@ static int prepare_answer(const knot_pkt_t *query, knot_pkt_t *resp, knot_layer_
 	if (has_limit) {
 		resp->max_size = KNOT_WIRE_MIN_PKTSIZE;
 		if (knot_pkt_has_edns(query)) {
-			conf_val_t val = conf_get(conf(), C_SRV, C_MAX_UDP_PAYLOAD);
+			conf_val_t *max_payload = &conf()->cache.srv_max_udp_payload;
 			uint16_t client = knot_edns_get_payload(query->opt_rr);
-			uint16_t server = conf_int(&val);
+			uint16_t server = conf_int(max_payload);
 			uint16_t transfer = MIN(client, server);
 			resp->max_size = MAX(resp->max_size, transfer);
 		}
@@ -371,8 +379,6 @@ static int process_query_err(knot_layer_t *ctx, knot_pkt_t *pkt)
 {
 	assert(pkt && ctx);
 	struct query_data *qdata = QUERY_DATA(ctx);
-	dbg_ns("%s: making error response, rcode = %d (TSIG rcode = %d)\n",
-	       __func__, qdata->rcode, qdata->rcode_tsig);
 
 	/* Initialize response from query packet. */
 	knot_pkt_t *query = qdata->query;
@@ -387,8 +393,7 @@ static int process_query_err(knot_layer_t *ctx, knot_pkt_t *pkt)
 	}
 
 	/* Put OPT RR to the additional section. */
-	int ret = answer_edns_reserve(pkt, qdata);
-	if (ret == KNOT_EOK) {
+	if (answer_edns_reserve(pkt, qdata) == KNOT_EOK) {
 		(void) answer_edns_put(pkt, qdata);
 	}
 
@@ -427,9 +432,10 @@ static int ratelimit_apply(int state, knot_pkt_t *pkt, knot_layer_t *ctx)
 
 	/* Now it is slip or drop. */
 	conf_val_t val = conf_get(conf(), C_SRV, C_RATE_LIMIT_SLIP);
-	if (rrl_slip_roll(conf_int(&val))) {
+	int slip = conf_int(&val);
+	if (slip > 0 && rrl_slip_roll(slip)) {
 		/* Answer slips. */
-		if (process_query_err(ctx, pkt) != KNOT_EOK) {
+		if (process_query_err(ctx, pkt) != KNOT_STATE_DONE) {
 			return KNOT_STATE_FAIL;
 		}
 		knot_wire_set_tc(pkt->wire);
@@ -444,17 +450,17 @@ static int ratelimit_apply(int state, knot_pkt_t *pkt, knot_layer_t *ctx)
 static int process_query_out(knot_layer_t *ctx, knot_pkt_t *pkt)
 {
 	assert(pkt && ctx);
+
+	rcu_read_lock();
+
 	struct query_data *qdata = QUERY_DATA(ctx);
 	struct query_plan *plan = conf()->query_plan;
 	struct query_step *step = NULL;
 
-	rcu_read_lock();
-
 	/* Check parse state. */
 	knot_pkt_t *query = qdata->query;
 	int next_state = KNOT_STATE_PRODUCE;
 	if (query->parsed < query->size) {
-		dbg_ns("%s: incompletely parsed query, FORMERR\n", __func__);
 		knot_pkt_clear(pkt);
 		qdata->rcode = KNOT_RCODE_FORMERR;
 		next_state = KNOT_STATE_FAIL;
@@ -548,6 +554,7 @@ finish:
 	}
 
 	rcu_read_unlock();
+
 	return next_state;
 }
 
@@ -572,7 +579,21 @@ bool process_query_acl_check(const knot_dname_t *zone_name, acl_action_t action,
 	/* Check if authenticated. */
 	conf_val_t acl = conf_zone_get(conf(), C_ACL, zone_name);
 	if (!acl_allowed(&acl, action, query_source, &tsig)) {
-		dbg_ns("%s: no ACL match => NOTAUTH\n", __func__);
+		char addr_str[SOCKADDR_STRLEN] = { 0 };
+		sockaddr_tostr(addr_str, sizeof(addr_str), query_source);
+		const knot_lookup_t *act = knot_lookup_by_id((knot_lookup_t *)acl_actions,
+		                                             action);
+		char *key_name = knot_dname_to_str_alloc(tsig.name);
+
+		log_zone_debug(zone_name,
+		               "ACL, denied, action '%s', remote '%s', key %s%s%s",
+		               (act != NULL) ? act->name : "query",
+		               addr_str,
+		               (key_name != NULL) ? "'" : "",
+		               (key_name != NULL) ? key_name : "none",
+		               (key_name != NULL) ? "'" : "");
+		free(key_name);
+
 		qdata->rcode = KNOT_RCODE_NOTAUTH;
 		qdata->rcode_tsig = KNOT_TSIG_ERR_BADKEY;
 		return false;
@@ -605,8 +626,6 @@ int process_query_verify(struct query_data *qdata)
 	                                 query->size, &ctx->tsig_key);
 	process_query_qname_case_lower(query);
 
-	dbg_ns("%s: QUERY TSIG check result = %s\n", __func__, knot_strerror(ret));
-
 	/* Evaluate TSIG check results. */
 	switch(ret) {
 	case KNOT_EOK:
@@ -651,7 +670,6 @@ int process_query_sign_response(knot_pkt_t *pkt, struct query_data *qdata)
 	if (ctx->tsig_key.name != NULL && knot_tsig_can_sign(qdata->rcode_tsig)) {
 
 		/* Sign query response. */
-		dbg_ns("%s: signing response using key %p\n", __func__, &ctx->tsig_key);
 		size_t new_digest_len = dnssec_tsig_algorithm_size(ctx->tsig_key.algorithm);
 		if (ctx->pkt_count == 0) {
 			ret = knot_tsig_sign(pkt->wire, &pkt->size, pkt->max_size,
@@ -674,7 +692,6 @@ int process_query_sign_response(knot_pkt_t *pkt, struct query_data *qdata)
 	} else {
 		/* Copy TSIG from query and set RCODE. */
 		if (query->tsig_rr && qdata->rcode_tsig != KNOT_RCODE_NOERROR) {
-			dbg_ns("%s: appending original TSIG\n", __func__);
 			ret = knot_tsig_add(pkt->wire, &pkt->size, pkt->max_size,
 			                    qdata->rcode_tsig, query->tsig_rr);
 			if (ret != KNOT_EOK) {
@@ -687,7 +704,6 @@ int process_query_sign_response(knot_pkt_t *pkt, struct query_data *qdata)
 
 	/* Server failure in signing. */
 fail:
-	dbg_ns("%s: signing failed (%s)\n", __func__, knot_strerror(ret));
 	qdata->rcode = KNOT_RCODE_SERVFAIL;
 	qdata->rcode_tsig = KNOT_RCODE_NOERROR; /* Don't sign again. */
 	return ret;
