@@ -1,14 +1,4 @@
-/*!
- * \file process_query.h
- *
- * \author Marek Vavrusa <marek.vavrusa@nic.cz>
- *
- * \brief Query processor.
- *
- * \addtogroup query_processing
- * @{
- */
-/*  Copyright (C) 2013 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,12 +13,21 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
+/*!
+ * \file
+ *
+ * \brief Query processor.
+ *
+ * \addtogroup query_processing
+ * @{
+ */
 
 #pragma once
 
 #include "libknot/processing/layer.h"
 #include "knot/server/server.h"
 #include "knot/updates/acl.h"
+#include "contrib/sockaddr.h"
 
 /* Query processing module implementation. */
 const knot_layer_api_t *process_query_layer(void);
@@ -93,7 +92,7 @@ struct query_data {
 
 	/* Everything below should be kept on reset. */
 	struct process_query_param *param; /*!< Module parameters. */
-	mm_ctx_t *mm;                      /*!< Memory context. */
+	knot_mm_t *mm;                     /*!< Memory context. */
 };
 
 /*! \brief Visited wildcard node list. */
