@@ -1,10 +1,28 @@
+/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+
+    This program is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "knot/nameserver/query_module.h"
 #include "libknot/libknot.h"
-#include "libknot/internal/strlcpy.h"
+#include "contrib/mempattern.h"
+#include "contrib/openbsd/strlcpy.h"
 
 /* Compiled-in module headers. */
 #include "knot/modules/synth_record.h"
 #include "knot/modules/dnsproxy.h"
+#include "knot/modules/online_sign/module.h"
 #ifdef HAVE_ROSEDB
 #include "knot/modules/rosedb.h"
 #endif
@@ -16,22 +34,24 @@ typedef struct static_module {
 	const yp_name_t *name;
 	qmodule_load_t load;
 	qmodule_unload_t unload;
+	unsigned scope;
 } static_module_t;
 
 /*! \note All modules should be dynamically loaded later on. */
 static_module_t MODULES[] = {
-        { C_MOD_SYNTH_RECORD, &synth_record_load, &synth_record_unload },
-        { C_MOD_DNSPROXY,     &dnsproxy_load,     &dnsproxy_unload },
+        { C_MOD_SYNTH_RECORD, &synth_record_load, &synth_record_unload, MOD_SCOPE_ANY },
+        { C_MOD_DNSPROXY,     &dnsproxy_load,     &dnsproxy_unload,     MOD_SCOPE_ANY },
+        { C_MOD_ONLINE_SIGN,  &online_sign_load,  &online_sign_unload,  MOD_SCOPE_ZONE },
 #ifdef HAVE_ROSEDB
-        { C_MOD_ROSEDB,       &rosedb_load,       &rosedb_unload },
+        { C_MOD_ROSEDB,       &rosedb_load,       &rosedb_unload,       MOD_SCOPE_ANY },
 #endif
 #if USE_DNSTAP
-        { C_MOD_DNSTAP,       &dnstap_load,       &dnstap_unload },
+        { C_MOD_DNSTAP,       &dnstap_load,       &dnstap_unload,       MOD_SCOPE_ANY },
 #endif
         { NULL }
 };
 
-struct query_plan *query_plan_create(mm_ctx_t *mm)
+struct query_plan *query_plan_create(knot_mm_t *mm)
 {
 	struct query_plan *plan = mm_alloc(mm, sizeof(struct query_plan));
 	if (plan == NULL) {
@@ -62,7 +82,7 @@ void query_plan_free(struct query_plan *plan)
 	mm_free(plan->mm, plan);
 }
 
-static struct query_step *make_step(mm_ctx_t *mm, qmodule_process_t process,
+static struct query_step *make_step(knot_mm_t *mm, qmodule_process_t process,
                                     void *ctx)
 {
 	struct query_step *step = mm_alloc(mm, sizeof(struct query_step));
@@ -106,7 +126,7 @@ static static_module_t *find_module(const yp_name_t *name)
 }
 
 struct query_module *query_module_open(conf_t *config, conf_mod_id_t *mod_id,
-                                       mm_ctx_t *mm)
+                                       knot_mm_t *mm)
 {
 	if (config == NULL || mod_id == NULL) {
 		return NULL;
@@ -130,6 +150,7 @@ struct query_module *query_module_open(conf_t *config, conf_mod_id_t *mod_id,
 	module->id = mod_id;
 	module->load = found->load;
 	module->unload = found->unload;
+	module->scope = found->scope;
 
 	return module;
 }
