@@ -1,15 +1,4 @@
-/*!
- * \file update.h
- *
- * \author Marek Vavrusa <marek.vavrusa@nic.cz>
- * \author Jan Kadlec <jan.kadlec@nic.cz>
- *
- * \brief DDNS UPDATE processing.
- *
- * \addtogroup query_processing
- * @{
- */
-/*  Copyright (C) 2013 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,31 +13,33 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
+/*!
+ * \file
+ *
+ * \brief DDNS UPDATE processing.
+ *
+ * \addtogroup query_processing
+ * @{
+ */
 
 #pragma once
 
 #include "libknot/packet/pkt.h"
-#include "knot/zone/zonedb.h"
+#include "knot/zone/zone.h"
 
 struct query_data;
-struct zone;
 
 /*!
  * \brief UPDATE query processing module.
  *
  * \return KNOT_STATE_* processing states
  */
-int update_query_process(knot_pkt_t *pkt, struct query_data *qdata);
+int update_process_query(knot_pkt_t *pkt, struct query_data *qdata);
 
 /*!
  * \brief Processes serialized packet with DDNS. Function expects that the
  *        query is already authenticated and TSIG signature is verified.
- *
- * \param pkt    Prepared response packet.
- * \param qdata  Minimal query data context.
- *
- * \return KNOT_E*
  */
-int updates_execute(zone_t *zone);
+void updates_execute(conf_t *conf, zone_t *zone);
 
 /*! @} */
