@@ -1,4 +1,4 @@
-/*  Copyright (C) 2011 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,25 +14,23 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define __APPLE_USE_RFC_3542
+
+#include <stdlib.h>
+#include <assert.h>
 #include <urcu.h>
 
-#include <unistd.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <sys/stat.h>
-#include <errno.h>
-#include <assert.h>
-
 #include "libknot/errcode.h"
-#include "knot/common/debug.h"
-#include "knot/common/trim.h"
+#include "knot/common/log.h"
 #include "knot/server/server.h"
 #include "knot/server/udp-handler.h"
 #include "knot/server/tcp-handler.h"
-#include "knot/conf/conf.h"
-#include "knot/worker/pool.h"
 #include "knot/zone/timers.h"
 #include "knot/zone/zonedb-load.h"
+#include "knot/worker/pool.h"
+#include "contrib/net.h"
+#include "contrib/sockaddr.h"
+#include "contrib/trim.h"
 
 /*! \brief Minimal send/receive buffer sizes. */
 enum {
@@ -42,52 +40,33 @@ enum {
 	TCP_MIN_SNDSIZE = sizeof(uint16_t) + UINT16_MAX
 };
 
-/*! \brief Event scheduler loop. */
-static int evsched_run(dthread_t *thread)
+/*! \brief Unbind interface and clear the structure. */
+static void server_deinit_iface(iface_t *iface)
 {
-	evsched_t *s = (evsched_t*)thread->data;
-	if (!s) {
-		return KNOT_EINVAL;
-	}
-
-	/* Run event loop. */
-	event_t *ev = 0;
-	while((ev = evsched_begin_process(s))) {
-
-		/* Process termination event (NULL function). */
-		if (ev->cb == NULL) {
-			evsched_end_process(s);
-			evsched_event_free(ev);
-			break;
-		}
-
-		/* Process event. */
-		ev->cb(ev);
-		evsched_end_process(s);
-
-		/* Check for thread cancellation. */
-		if (dt_is_cancelled(thread)) {
-			break;
+	/* Free UDP handler. */
+	for (int i = 0; i < iface->fd_udp_count; i++) {
+		if (iface->fd_udp[i] > -1) {
+			close(iface->fd_udp[i]);
 		}
 	}
+	free(iface->fd_udp);
 
-	return KNOT_EOK;
+	/* Free TCP handler. */
+	if (iface->fd_tcp > -1) {
+		close(iface->fd_tcp);
+	}
+
+	memset(iface, 0, sizeof(*iface));
 }
 
 /*! \brief Unbind and dispose given interface. */
 static void server_remove_iface(iface_t *iface)
 {
-	/* Free UDP handler. */
-	if (iface->fd[IO_UDP] > -1) {
-		close(iface->fd[IO_UDP]);
+	if (!iface) {
+		return;
 	}
 
-	/* Free TCP handler. */
-	if (iface->fd[IO_TCP] > -1) {
-		close(iface->fd[IO_TCP]);
-	}
-
-	/* Free interface. */
+	server_deinit_iface(iface);
 	free(iface);
 }
 
@@ -118,6 +97,37 @@ static bool enlarge_net_buffers(int sock, int min_recvsize, int min_sndsize)
 	       setsockopt_min(sock, SO_SNDBUF, min_sndsize);
 }
 
+/*!
+ * \brief Enable source packet information retrieval.
+ */
+static bool enable_pktinfo(int sock, int family)
+{
+	int level = 0;
+	int option = 0;
+
+	switch (family) {
+	case AF_INET:
+		level = IPPROTO_IP;
+#if defined(IP_PKTINFO)
+		option = IP_PKTINFO; /* Linux */
+#elif defined(IP_RECVDSTADDR)
+		option = IP_RECVDSTADDR; /* BSD */
+#else
+		return false;
+#endif
+		break;
+	case AF_INET6:
+		level = IPPROTO_IPV6;
+		option = IPV6_RECVPKTINFO;
+		break;
+	default:
+		return false;
+	}
+
+	const int on = 1;
+	return setsockopt(sock, level, option, &on, sizeof(on)) == 0;
+}
+
 /*!
  * \brief Initialize new interface from config value.
  *
@@ -129,7 +139,7 @@ static bool enlarge_net_buffers(int sock, int min_recvsize, int min_sndsize)
  * \retval 0 if successful (EOK).
  * \retval <0 on errors (EACCES, EINVAL, ENOMEM, EADDRINUSE).
  */
-static int server_init_iface(iface_t *new_if, struct sockaddr_storage *addr)
+static int server_init_iface(iface_t *new_if, struct sockaddr_storage *addr, int udp_thread_count)
 {
 	/* Initialize interface. */
 	int ret = 0;
@@ -140,35 +150,67 @@ static int server_init_iface(iface_t *new_if, struct sockaddr_storage *addr)
 	char addr_str[SOCKADDR_STRLEN] = { 0 };
 	sockaddr_tostr(addr_str, sizeof(addr_str), addr);
 
-	/* Create bound UDP socket. */
+	int udp_socket_count = 1;
 	int bind_flags = 0;
-	int sock = net_bound_socket(SOCK_DGRAM, addr, bind_flags);
-	if (sock == KNOT_EADDRNOTAVAIL) {
-		bind_flags |= NET_BIND_NONLOCAL;
-		sock = net_bound_socket(SOCK_DGRAM, addr, bind_flags);
-		if (sock >= 0) {
-			log_warning("address '%s' is not available", addr_str);
+
+#ifdef ENABLE_REUSEPORT
+	udp_socket_count = udp_thread_count;
+	bind_flags |= NET_BIND_MULTIPLE;
+#endif
+
+	new_if->fd_udp = malloc(udp_socket_count * sizeof(int));
+	if (!new_if->fd_udp) {
+		return KNOT_ENOMEM;
+	}
+
+	/* Initialize the sockets to ensure safe early deinitialization. */
+	for (int i = 0; i < udp_socket_count; i++) {
+		new_if->fd_udp[new_if->fd_udp_count] = -1;
+	};
+	new_if->fd_tcp = -1;
+
+	bool warn_bind = false;
+	bool warn_bufsize = false;
+
+	/* Create bound UDP sockets. */
+	for (int i = 0; i < udp_socket_count; i++ ) {
+		int sock = net_bound_socket(SOCK_DGRAM, addr, bind_flags);
+		if (sock == KNOT_EADDRNOTAVAIL) {
+			bind_flags |= NET_BIND_NONLOCAL;
+			sock = net_bound_socket(SOCK_DGRAM, addr, bind_flags);
+			if (sock >= 0 && !warn_bind) {
+				log_warning("address '%s' is not available", addr_str);
+				warn_bind = true;
+			}
 		}
+
+		if (sock < 0) {
+			log_error("cannot bind address '%s' (%s)", addr_str,
+			          knot_strerror(sock));
+			server_deinit_iface(new_if);
+			return sock;
+		}
+
+		if (!enlarge_net_buffers(sock, UDP_MIN_RCVSIZE, UDP_MIN_SNDSIZE) &&
+		    !warn_bufsize) {
+			log_warning("failed to set network buffer sizes for UDP");
+			warn_bufsize = true;
+		}
+
+		if (sockaddr_is_any(addr) && !enable_pktinfo(sock, addr->ss_family)) {
+			log_warning("failed to enable received packet information retrieval");
+		}
+
+		new_if->fd_udp[new_if->fd_udp_count] = sock;
+		new_if->fd_udp_count += 1;
 	}
 
-	if (sock < 0) {
-		log_error("cannot bind address '%s' (%s)", addr_str, knot_strerror(sock));
-		return sock;
-	}
-
-	if (!enlarge_net_buffers(sock, UDP_MIN_RCVSIZE, UDP_MIN_SNDSIZE)) {
-		log_warning("failed to set network buffer sizes for UDP");
-	}
-
-	/* Set UDP as non-blocking. */
-	fcntl(sock, F_SETFL, O_NONBLOCK);
-
-	new_if->fd[IO_UDP] = sock;
-
 	/* Create bound TCP socket. */
-	sock = net_bound_socket(SOCK_STREAM, addr, bind_flags);
+	int sock = net_bound_socket(SOCK_STREAM, addr, bind_flags);
 	if (sock < 0) {
-		close(new_if->fd[IO_UDP]);
+		log_error("cannot bind address '%s' (%s)", addr_str,
+		          knot_strerror(sock));
+		server_deinit_iface(new_if);
 		return sock;
 	}
 
@@ -176,23 +218,13 @@ static int server_init_iface(iface_t *new_if, struct sockaddr_storage *addr)
 		log_warning("failed to set network buffer sizes for TCP");
 	}
 
-	new_if->fd[IO_TCP] = sock;
+	new_if->fd_tcp = sock;
 
 	/* Listen for incoming connections. */
 	ret = listen(sock, TCP_BACKLOG_SIZE);
 	if (ret < 0) {
-		close(new_if->fd[IO_UDP]);
-		close(new_if->fd[IO_TCP]);
 		log_error("failed to listen on TCP interface '%s'", addr_str);
-		return KNOT_ERROR;
-	}
-
-	/* accept() must not block */
-	if (fcntl(sock, F_SETFL, O_NONBLOCK) < 0) {
-		close(new_if->fd[IO_UDP]);
-		close(new_if->fd[IO_TCP]);
-		log_error("failed to listen on '%s' in non-blocking mode",
-			  addr_str);
+		server_deinit_iface(new_if);
 		return KNOT_ERROR;
 	}
 
@@ -271,7 +303,8 @@ static int reconfigure_sockets(conf_t *conf, server_t *s)
 
 			/* Create new interface. */
 			m = malloc(sizeof(iface_t));
-			if (server_init_iface(m, &addr) < 0) {
+			unsigned size = s->handlers[IO_UDP].handler.unit->size;
+			if (server_init_iface(m, &addr, size) < 0) {
 				free(m);
 				m = 0;
 			}
@@ -290,8 +323,8 @@ static int reconfigure_sockets(conf_t *conf, server_t *s)
 	/* Wait for readers that are reconfiguring right now. */
 	/*! \note This subsystem will be reworked in #239 */
 	for (unsigned proto = IO_UDP; proto <= IO_TCP; ++proto) {
-		dt_unit_t *tu = s->handler[proto].unit;
-		iohandler_t *ioh = &s->handler[proto];
+		dt_unit_t *tu = s->handlers[proto].handler.unit;
+		iohandler_t *ioh = &s->handlers[proto].handler;
 		for (unsigned i = 0; i < tu->size; ++i) {
 			while (ioh->thread_state[i] & ServerReload) {
 				sleep(1);
@@ -305,11 +338,11 @@ static int reconfigure_sockets(conf_t *conf, server_t *s)
 	/* Update TCP+UDP ifacelist (reload all threads). */
 	unsigned thread_count = 0;
 	for (unsigned proto = IO_UDP; proto <= IO_TCP; ++proto) {
-		dt_unit_t *tu = s->handler[proto].unit;
+		dt_unit_t *tu = s->handlers[proto].handler.unit;
 		for (unsigned i = 0; i < tu->size; ++i) {
 			ref_retain((ref_t *)newlist);
-			s->handler[proto].thread_state[i] |= ServerReload;
-			s->handler[proto].thread_id[i] = thread_count++;
+			s->handlers[proto].handler.thread_state[i] |= ServerReload;
+			s->handlers[proto].handler.thread_id[i] = thread_count++;
 			if (s->state & ServerRunning) {
 				dt_activate(tu->threads[i]);
 				dt_signalize(tu->threads[i], SIGALRM);
@@ -324,27 +357,20 @@ static int reconfigure_sockets(conf_t *conf, server_t *s)
 
 int server_init(server_t *server, int bg_workers)
 {
-	/* Clear the structure. */
-	dbg_server("%s(%p)\n", __func__, server);
 	if (server == NULL) {
 		return KNOT_EINVAL;
 	}
 
+	/* Clear the structure. */
 	memset(server, 0, sizeof(server_t));
 
 	/* Initialize event scheduler. */
 	if (evsched_init(&server->sched, server) != KNOT_EOK) {
 		return KNOT_ENOMEM;
 	}
-	server->iosched = dt_create(1, evsched_run, NULL, &server->sched);
-	if (server->iosched == NULL) {
-		evsched_deinit(&server->sched);
-		return KNOT_ENOMEM;
-	}
 
 	server->workers = worker_pool_create(bg_workers);
 	if (server->workers == NULL) {
-		dt_delete(&server->iosched);
 		evsched_deinit(&server->sched);
 		return KNOT_ENOMEM;
 	}
@@ -354,7 +380,6 @@ int server_init(server_t *server, int bg_workers)
 
 void server_deinit(server_t *server)
 {
-	dbg_server("%s(%p)\n", __func__, server);
 	if (server == NULL) {
 		return;
 	}
@@ -370,7 +395,6 @@ void server_deinit(server_t *server)
 
 	/* Free threads and event handlers. */
 	worker_pool_destroy(server->workers);
-	dt_delete(&server->iosched);
 
 	/* Free rate limits. */
 	rrl_destroy(server->rrl);
@@ -392,7 +416,7 @@ static int server_init_handler(server_t *server, int index, int thread_count,
                                runnable_t runnable, runnable_t destructor)
 {
 	/* Initialize */
-	iohandler_t *h = &server->handler[index];
+	iohandler_t *h = &server->handlers[index].handler;
 	memset(h, 0, sizeof(iohandler_t));
 	h->server = server;
 	h->unit = dt_create(thread_count, runnable, destructor, h);
@@ -435,94 +459,101 @@ static void server_free_handler(iohandler_t *h)
 	memset(h, 0, sizeof(iohandler_t));
 }
 
-int server_start(server_t *s, bool async)
-{
-	dbg_server("%s(%p, %d)\n", __func__, s, async);
-	if (s == 0) {
-		return KNOT_EINVAL;
-	}
-
-	/* Start workers. */
-	worker_pool_start(s->workers);
-
-	/* Wait for enqueued events if not asynchronous. */
-	if (!async) {
-		worker_pool_wait(s->workers);
-	}
-
-	/* Start evsched handler. */
-	dt_start(s->iosched);
-
-	/* Start I/O handlers. */
-	int ret = KNOT_EOK;
-	s->state |= ServerRunning;
-	if (s->tu_size > 0) {
-		for (unsigned i = 0; i < IO_COUNT; ++i) {
-			ret = dt_start(s->handler[i].unit);
-		}
-	}
-
-	return ret;
-}
-
-void server_wait(server_t *s)
-{
-	if (s == NULL) {
-		return;
-	}
-
-	dt_join(s->iosched);
-	worker_pool_join(s->workers);
-
-	if (s->tu_size == 0) {
-		return;
-	}
-
-	for (unsigned i = 0; i < IO_COUNT; ++i) {
-		server_free_handler(s->handler + i);
-	}
-}
-
-int server_reload(server_t *server, const char *cf)
+int server_start(server_t *server, bool async)
 {
 	if (server == NULL) {
 		return KNOT_EINVAL;
 	}
 
+	/* Start workers. */
+	worker_pool_start(server->workers);
+
+	/* Wait for enqueued events if not asynchronous. */
+	if (!async) {
+		worker_pool_wait(server->workers);
+	}
+
+	/* Start evsched handler. */
+	evsched_start(&server->sched);
+
+	/* Start I/O handlers. */
+	server->state |= ServerRunning;
+	for (int proto = IO_UDP; proto <= IO_TCP; ++proto) {
+		if (server->handlers[proto].size > 0) {
+			int ret = dt_start(server->handlers[proto].handler.unit);
+			if (ret != KNOT_EOK) {
+				return ret;
+			}
+		}
+	}
+
+	return KNOT_EOK;
+}
+
+void server_wait(server_t *server)
+{
+	if (server == NULL) {
+		return;
+	}
+
+	evsched_join(&server->sched);
+	worker_pool_join(server->workers);
+
+	for (int proto = IO_UDP; proto <= IO_TCP; ++proto) {
+		if (server->handlers[proto].size > 0) {
+			server_free_handler(&server->handlers[proto].handler);
+		}
+	}
+}
+
+int server_reload(server_t *server, const char *cf, bool refresh_hostname)
+{
+	if (server == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	// Check for no edit mode.
+	if (cf != NULL && conf()->io.txn != NULL) {
+		log_warning("reload aborted due to active config DB transaction");
+		return KNOT_CONF_ETXN;
+	}
+
+	conf_t *new_conf = NULL;
+	int ret = conf_clone(&new_conf);
+	if (ret != KNOT_EOK) {
+		log_error("failed to initialize configuration (%s)",
+		          knot_strerror(ret));
+		return ret;
+	}
+
 	if (cf != NULL) {
 		log_info("reloading configuration file '%s'", cf);
-		conf_t *new_conf = NULL;
-		int ret = conf_clone(&new_conf);
-		if (ret != KNOT_EOK) {
-			log_fatal("failed to initialize configuration (%s)",
-			          knot_strerror(ret));
-			return ret;
-		}
 
 		/* Import the configuration file. */
 		ret = conf_import(new_conf, cf, true);
 		if (ret != KNOT_EOK) {
-			log_fatal("failed to load configuration file (%s)",
+			log_error("failed to load configuration file (%s)",
 			          knot_strerror(ret));
-			conf_free(new_conf, false);
+			conf_free(new_conf);
 			return ret;
 		}
-
-		/* Run post-open config operations. */
-		int res = conf_post_open(new_conf);
-		if (res != KNOT_EOK) {
-			log_fatal("failed to use configuration (%s)",
-			          knot_strerror(res));
-			conf_free(new_conf, false);
-			return ret;
-		}
-
-		conf_update(new_conf);
 	} else {
 		log_info("reloading configuration database");
 	}
 
-	log_reconfigure(conf(), NULL);
+	/* Activate global query modules. */
+	conf_activate_modules(new_conf, NULL, &new_conf->query_modules,
+	                      &new_conf->query_plan);
+
+	/* Refresh hostname. */
+	if (refresh_hostname) {
+		conf_refresh_hostname(new_conf);
+	}
+
+	/* Update to the new config. */
+	conf_update(new_conf);
+
+	log_reconfigure(conf());
 	server_reconfigure(conf(), server);
 	server_update_zones(conf(), server);
 
@@ -535,11 +566,8 @@ void server_stop(server_t *server)
 {
 	log_info("stopping server");
 
-	/* Send termination event. */
-	event_t *term_ev = evsched_event_create(&server->sched, NULL, NULL);
-	evsched_schedule(term_ev, 0);
-	dt_stop(server->iosched);
-
+	/* Stop scheduler. */
+	evsched_stop(&server->sched);
 	/* Interrupt background workers. */
 	worker_pool_stop(server->workers);
 
@@ -547,49 +575,42 @@ void server_stop(server_t *server)
 	server->state &= ~ServerRunning;
 }
 
-/*! \brief Reconfigure UDP and TCP query processing threads. */
-static int reconfigure_threads(conf_t *conf, server_t *server)
+static int reset_handler(server_t *server, int index, unsigned size, runnable_t run)
 {
-	/* Estimate number of threads/manager. */
-	int ret = KNOT_EOK;
-	int tu_size = conf_udp_threads(conf);
-	if ((unsigned)tu_size != server->tu_size) {
+	if (server->handlers[index].size != size) {
 		/* Free old handlers */
-		if (server->tu_size > 0) {
-			for (unsigned i = 0; i < IO_COUNT; ++i) {
-				server_free_handler(server->handler + i);
-			}
+		if (server->handlers[index].size > 0) {
+			server_free_handler(&server->handlers[index].handler);
 		}
 
 		/* Initialize I/O handlers. */
-		ret = server_init_handler(server, IO_UDP, conf_udp_threads(conf),
-		                          &udp_master, NULL);
+		int ret = server_init_handler(server, index, size, run, NULL);
 		if (ret != KNOT_EOK) {
-			log_error("failed to create UDP threads (%s)",
-			          knot_strerror(ret));
-			return ret;
-		}
-
-		/* Create at least CONFIG_XFERS threads for TCP for faster
-		 * processing of massive bootstrap queries. */
-		ret = server_init_handler(server, IO_TCP, conf_tcp_threads(conf),
-		                          &tcp_master, NULL);
-		if (ret != KNOT_EOK) {
-			log_error("failed to create TCP threads (%s)",
-			          knot_strerror(ret));
 			return ret;
 		}
 
 		/* Start if server is running. */
 		if (server->state & ServerRunning) {
-			for (unsigned i = 0; i < IO_COUNT; ++i) {
-				ret = dt_start(server->handler[i].unit);
+			ret = dt_start(server->handlers[index].handler.unit);
+			if (ret != KNOT_EOK) {
+				return ret;
 			}
 		}
-		server->tu_size = tu_size;
+		server->handlers[index].size = size;
 	}
 
-	return ret;
+	return KNOT_EOK;
+}
+
+/*! \brief Reconfigure UDP and TCP query processing threads. */
+static int reconfigure_threads(conf_t *conf, server_t *server)
+{
+	int ret = reset_handler(server, IO_UDP, conf_udp_threads(conf), udp_master);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
+	return reset_handler(server, IO_TCP, conf_tcp_threads(conf), tcp_master);
 }
 
 static int reconfigure_rate_limits(conf_t *conf, server_t *server)
@@ -625,12 +646,10 @@ static int reconfigure_rate_limits(conf_t *conf, server_t *server)
 	return KNOT_EOK;
 }
 
-int server_reconfigure(conf_t *conf, void *data)
+void server_reconfigure(conf_t *conf, server_t *server)
 {
-	server_t *server = (server_t *)data;
-	dbg_server("%s(%p, %p)\n", __func__, conf, server);
-	if (server == NULL) {
-		return KNOT_EINVAL;
+	if (conf == NULL || server == NULL) {
+		return;
 	}
 
 	/* First reconfiguration. */
@@ -639,25 +658,23 @@ int server_reconfigure(conf_t *conf, void *data)
 	}
 
 	/* Reconfigure rate limits. */
-	int ret = KNOT_EOK;
+	int ret;
 	if ((ret = reconfigure_rate_limits(conf, server)) < 0) {
-		log_error("failed to reconfigure rate limits");
-		return ret;
+		log_error("failed to reconfigure rate limits (%s)",
+		          knot_strerror(ret));
 	}
 
 	/* Reconfigure server threads. */
 	if ((ret = reconfigure_threads(conf, server)) < 0) {
-		log_error("failed to reconfigure server threads");
-		return ret;
+		log_error("failed to reconfigure server threads (%s)",
+		          knot_strerror(ret));
 	}
 
 	/* Update bound sockets. */
 	if ((ret = reconfigure_sockets(conf, server)) < 0) {
-		log_error("failed to reconfigure server sockets");
-		return ret;
+		log_error("failed to reconfigure server sockets (%s)",
+		          knot_strerror(ret));
 	}
-
-	return ret;
 }
 
 static void reopen_timers_database(conf_t *conf, server_t *server)
@@ -667,17 +684,23 @@ static void reopen_timers_database(conf_t *conf, server_t *server)
 
 	conf_val_t val = conf_default_get(conf, C_STORAGE);
 	char *storage = conf_abs_path(&val, NULL);
-	int ret = open_timers_db(storage, &server->timers_db);
+	val = conf_default_get(conf, C_TIMER_DB);
+	char *timer_db = conf_abs_path(&val, storage);
 	free(storage);
+
+	int ret = open_timers_db(timer_db, &server->timers_db);
+	free(timer_db);
 	if (ret != KNOT_EOK && ret != KNOT_ENOTSUP) {
 		log_warning("cannot open persistent timers DB (%s)",
 		            knot_strerror(ret));
 	}
 }
 
-int server_update_zones(conf_t *conf, void *data)
+void server_update_zones(conf_t *conf, server_t *server)
 {
-	server_t *server = (server_t *)data;
+	if (conf == NULL || server == NULL) {
+		return;
+	}
 
 	/* Prevent emitting of new zone events. */
 	if (server->zone_db) {
@@ -691,7 +714,7 @@ int server_update_zones(conf_t *conf, void *data)
 
 	/* Reload zone database and free old zones. */
 	reopen_timers_database(conf, server);
-	int ret = zonedb_reload(conf, server);
+	zonedb_reload(conf, server);
 
 	/* Trim extra heap. */
 	mem_trim();
@@ -701,22 +724,36 @@ int server_update_zones(conf_t *conf, void *data)
 	if (server->zone_db) {
 		knot_zonedb_foreach(server->zone_db, zone_events_start);
 	}
-
-	return ret;
 }
 
-ref_t *server_set_ifaces(server_t *s, fdset_t *fds, int type)
+ref_t *server_set_ifaces(server_t *server, fdset_t *fds, int index, int thread_id)
 {
-	iface_t *i = NULL;
+	if (server == NULL || server->ifaces == NULL || fds == NULL) {
+		return NULL;
+	}
 
 	rcu_read_lock();
 	fdset_clear(fds);
-	if (s->ifaces) {
-		WALK_LIST(i, s->ifaces->l) {
-			fdset_add(fds, i->fd[type], POLLIN, NULL);
-		}
 
+	iface_t *i = NULL;
+	WALK_LIST(i, server->ifaces->l) {
+#ifdef ENABLE_REUSEPORT
+		int udp_id = thread_id % i->fd_udp_count;
+#else
+		int udp_id = 0;
+#endif
+		switch(index) {
+		case IO_TCP:
+			fdset_add(fds, i->fd_tcp, POLLIN, NULL);
+			break;
+		case IO_UDP:
+			fdset_add(fds, i->fd_udp[udp_id], POLLIN, NULL);
+			break;
+		default:
+			assert(0);
+		}
 	}
 	rcu_read_unlock();
-	return (ref_t *)s->ifaces;
+
+	return &server->ifaces->ref;
 }
