@@ -1,4 +1,4 @@
-/*  Copyright (C) 2011 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,13 +16,11 @@
 
 #include <assert.h>
 
+#include "knot/common/log.h"
 #include "knot/updates/apply.h"
-#include "knot/updates/changesets.h"
-#include "knot/zone/zone.h"
-#include "knot/zone/zonefile.h"
 #include "libknot/libknot.h"
-#include "libknot/internal/lists.h"
-#include "libknot/internal/macros.h"
+#include "contrib/macros.h"
+#include "contrib/mempattern.h"
 
 /* --------------------------- Update cleanup ------------------------------- */
 
@@ -32,7 +30,7 @@
  *          Freed data:
  *           - actual data inside knot_rrs_t. (the rest is part of the node)
  */
-static void rrs_list_clear(list_t *l, mm_ctx_t *mm)
+static void rrs_list_clear(list_t *l, knot_mm_t *mm)
 {
 	ptrnode_t *n;
 	node_t *nxt;
@@ -271,7 +269,7 @@ static int apply_remove(zone_contents_t *contents, changeset_t *chset)
 
 /*! \brief Adds a single RR into zone contents. */
 static int add_rr(const zone_contents_t *zone, zone_node_t *node,
-                  const knot_rrset_t *rr, changeset_t *chset, bool master)
+                  const knot_rrset_t *rr, changeset_t *chset)
 {
 	knot_rrset_t changed_rrset = node_rrset(node, rr->type);
 	if (!knot_rrset_empty(&changed_rrset)) {
@@ -302,12 +300,10 @@ static int add_rr(const zone_contents_t *zone, zone_node_t *node,
 		}
 
 		if (ret == KNOT_ETTL) {
-			// Handle possible TTL errors.
-			log_ttl_error(zone, node, rr);
-			if (!master) {
-				// TTL errors fatal only for master.
-				return KNOT_EOK;
-			}
+			log_zone_notice(zone->apex->owner,
+			                "rrset (type %u) TTL mismatch, updated to %u",
+			                rr->type, knot_rrset_ttl(rr));
+			return KNOT_EOK;
 		}
 	}
 
@@ -315,8 +311,7 @@ static int add_rr(const zone_contents_t *zone, zone_node_t *node,
 }
 
 /*! \brief Adds all RRs from changeset into zone contents. */
-static int apply_add(zone_contents_t *contents, changeset_t *chset,
-                     bool master)
+static int apply_add(zone_contents_t *contents, changeset_t *chset)
 {
 	changeset_iter_t itt;
 	changeset_iter_add(&itt, chset, false);
@@ -330,7 +325,7 @@ static int apply_add(zone_contents_t *contents, changeset_t *chset,
 			return KNOT_ENOMEM;
 		}
 
-		int ret = add_rr(contents, node, &rr, chset, master);
+		int ret = add_rr(contents, node, &rr, chset);
 		if (ret != KNOT_EOK) {
 			changeset_iter_clear(&itt);
 			return ret;
@@ -353,12 +348,11 @@ static int apply_replace_soa(zone_contents_t *contents, changeset_t *chset)
 
 	assert(!node_rrtype_exists(contents->apex, KNOT_RRTYPE_SOA));
 
-	return add_rr(contents, contents->apex, chset->soa_to, chset, false);
+	return add_rr(contents, contents->apex, chset->soa_to, chset);
 }
 
 /*! \brief Apply single change to zone contents structure. */
-static int apply_single(zone_contents_t *contents, changeset_t *chset,
-                           bool master)
+static int apply_single(zone_contents_t *contents, changeset_t *chset)
 {
 	/*
 	 * Applies one changeset to the zone. Checks if the changeset may be
@@ -377,7 +371,7 @@ static int apply_single(zone_contents_t *contents, changeset_t *chset,
 		return ret;
 	}
 
-	ret = apply_add(contents, chset, master);
+	ret = apply_add(contents, chset);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
@@ -453,7 +447,7 @@ int apply_changesets(zone_t *zone, list_t *chsets, zone_contents_t **new_content
 	 */
 	changeset_t *set = NULL;
 	WALK_LIST(set, *chsets) {
-		ret = apply_single(contents_copy, set, !zone_is_slave(zone));
+		ret = apply_single(contents_copy, set);
 		if (ret != KNOT_EOK) {
 			updates_rollback(chsets);
 			update_free_zone(&contents_copy);
@@ -492,7 +486,7 @@ int apply_changeset(zone_t *zone, changeset_t *change, zone_contents_t **new_con
 		return ret;
 	}
 
-	ret = apply_single(contents_copy, change, !zone_is_slave(zone));
+	ret = apply_single(contents_copy, change);
 	if (ret != KNOT_EOK) {
 		update_rollback(change);
 		update_free_zone(&contents_copy);
@@ -519,8 +513,7 @@ int apply_changesets_directly(zone_contents_t *contents, list_t *chsets)
 
 	changeset_t *set = NULL;
 	WALK_LIST(set, *chsets) {
-		const bool master = true; // Only DNSSEC changesets are applied directly.
-		int ret = apply_single(contents, set, master);
+		int ret = apply_single(contents, set);
 		if (ret != KNOT_EOK) {
 			updates_cleanup(chsets);
 			return ret;
@@ -541,8 +534,7 @@ int apply_changeset_directly(zone_contents_t *contents, changeset_t *ch)
 		return KNOT_EINVAL;
 	}
 
-	const bool master = true; // Only DNSSEC changesets are applied directly.
-	int ret = apply_single(contents, ch, master);
+	int ret = apply_single(contents, ch);
 	if (ret != KNOT_EOK) {
 		update_cleanup(ch);
 		return ret;
