@@ -1,16 +1,4 @@
-/*!
- * \file apply.h
- *
- * \author Lubos Slovak <lubos.slovak@nic.cz>
- * \author Jan Kadlec <jan.kadlec@nic.cz>
- *
- * \brief Changesets application and update helpers.
- *
- * \addtogroup xfr
- * @{
- */
-
-/*  Copyright (C) 2011 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,14 +13,20 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
+/*!
+ * \file
+ *
+ * \brief Changesets application and update helpers.
+ *
+ * \addtogroup ddns
+ * @{
+ */
 
 #pragma once
 
-#include <stdint.h>
-#include <string.h>
-
 #include "knot/zone/zone.h"
 #include "knot/updates/changesets.h"
+#include "contrib/ucw/lists.h"
 
 /*!
  * \brief Applies changesets *with* zone shallow copy.
