@@ -1,4 +1,4 @@
-/*  Copyright (C) 2011 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,13 +14,14 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <stdlib.h>
 #include <assert.h>
+#include <stdlib.h>
 #include <stdarg.h>
 
 #include "knot/updates/changesets.h"
-#include "libknot/rrset.h"
-#include "libknot/internal/macros.h"
+#include "libknot/libknot.h"
+#include "contrib/macros.h"
+#include "contrib/mempattern.h"
 
 /* -------------------- Changeset iterator helpers -------------------------- */
 
@@ -257,6 +258,14 @@ int changeset_merge(changeset_t *ch1, const changeset_t *ch2)
 	knot_rrset_free(&ch1->soa_to, NULL);
 	ch1->soa_to = soa_copy;
 
+	ptrnode_t *n;
+	WALK_LIST(n, ch2->old_data) {
+		ptrlist_add(&ch1->old_data, (void *)n->d, NULL);
+	};
+	WALK_LIST(n, ch2->new_data) {
+		ptrlist_add(&ch1->new_data, (void *)n->d, NULL);
+	};
+
 	return KNOT_EOK;
 }
 
@@ -297,6 +306,14 @@ void changeset_clear(changeset_t *ch)
 	knot_rrset_free(&ch->soa_from, NULL);
 	knot_rrset_free(&ch->soa_to, NULL);
 
+	node_t *n, *nxt;
+	WALK_LIST_DELSAFE(n, nxt, ch->old_data) {
+		mm_free(NULL, n);
+	};
+	WALK_LIST_DELSAFE(n, nxt, ch->new_data) {
+		mm_free(NULL, n);
+	};
+
 	// Delete binary data
 	free(ch->data);
 }
