@@ -20,10 +20,9 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "libknot/errcode.h"
+#include "libknot/libknot.h"
 #include "knot/server/dthreads.h"
 #include "knot/worker/pool.h"
-#include "libknot/dnssec/crypto.h"
 
 /*!
  * \brief Worker pool state.
@@ -88,13 +87,6 @@ static int worker_main(dthread_t *thread)
 	return KNOT_EOK;
 }
 
-static int worker_cleanup(dthread_t *thread)
-{
-	knot_crypto_cleanup_thread();
-
-	return KNOT_EOK;
-}
-
 /* -- public API ------------------------------------------------------------ */
 
 worker_pool_t *worker_pool_create(unsigned threads)
@@ -105,7 +97,7 @@ worker_pool_t *worker_pool_create(unsigned threads)
 	}
 
 	memset(pool, 0, sizeof(worker_pool_t));
-	pool->threads = dt_create(threads, worker_main, worker_cleanup, pool);
+	pool->threads = dt_create(threads, worker_main, NULL, pool);
 	if (pool->threads == NULL) {
 		goto fail;
 	}
