@@ -1,4 +1,4 @@
-/*  Copyright (C) 2014 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,17 +18,17 @@
 
 #include "dnssec/random.h"
 #include "libknot/libknot.h"
-#include "libknot/internal/mempool.h"
-#include "libknot/internal/macros.h"
-#include "libknot/processing/requestor.h"
+#include "libknot/yparser/yptrafo.h"
+#include "contrib/mempattern.h"
+#include "contrib/trim.h"
+#include "contrib/ucw/mempool.h"
+#include "contrib/wire.h"
 
 #include "knot/common/log.h"
-#include "knot/common/trim.h"
 #include "knot/server/udp-handler.h"
 #include "knot/server/tcp-handler.h"
 #include "knot/updates/changesets.h"
 #include "knot/dnssec/zone-events.h"
-#include "knot/zone/timers.h"
 #include "knot/zone/zone-load.h"
 #include "knot/zone/zonefile.h"
 #include "knot/zone/events/events.h"
@@ -43,14 +43,12 @@
 #define BOOTSTRAP_RETRY (30) /*!< Interval between AXFR bootstrap retries. */
 #define BOOTSTRAP_MAXTIME (24*60*60) /*!< Maximum AXFR retry cap of 24 hours. */
 
-/* ------------------------- zone query requesting -------------------------- */
-
 /*! \brief Zone event logging. */
 #define ZONE_QUERY_LOG(severity, zone, remote, operation, msg, ...) \
 	NS_PROC_LOG(severity, &(remote)->addr, zone->name, operation, msg, ##__VA_ARGS__)
 
 /*! \brief Create zone query packet. */
-static knot_pkt_t *zone_query(const zone_t *zone, uint16_t pkt_type, mm_ctx_t *mm)
+static knot_pkt_t *zone_query(const zone_t *zone, uint16_t pkt_type, knot_mm_t *mm)
 {
 	/* Determine query type and opcode. */
 	uint16_t query_type = KNOT_RRTYPE_SOA;
@@ -89,43 +87,59 @@ static knot_pkt_t *zone_query(const zone_t *zone, uint16_t pkt_type, mm_ctx_t *m
 	return pkt;
 }
 
-/*!
- * \brief Create a zone event query, send it, wait for the response and process it.
- *
- * \note Everything in this function is executed synchronously, returns when
- *       the query processing is either complete or an error occurs.
- */
-static int zone_query_execute(zone_t *zone, uint16_t pkt_type, const conf_remote_t *remote)
+/*! \brief Set EDNS section. */
+static int prepare_edns(conf_t *conf, zone_t *zone, knot_pkt_t *pkt)
 {
-	/* Create a memory pool for this task. */
-	int ret = KNOT_EOK;
-	mm_ctx_t mm;
-	mm_ctx_mempool(&mm, MM_DEFAULT_BLKSIZE);
+	conf_val_t val = conf_zone_get(conf, C_REQUEST_EDNS_OPTION, zone->name);
 
-	/* Create a query message. */
-	knot_pkt_t *query = zone_query(zone, pkt_type, &mm);
-	if (query == NULL) {
-		return KNOT_ENOMEM;
+	/* Check if an extra EDNS option is configured. */
+	size_t opt_len;
+	const uint8_t *opt_data = conf_data(&val, &opt_len);
+	if (opt_data == NULL) {
+		return KNOT_EOK;
 	}
 
-	/* Answer processing parameters. */
-	struct process_answer_param param = { 0 };
-	param.zone = zone;
-	param.query = query;
-	param.remote = &remote->addr;
+	knot_rrset_t opt_rr;
+	conf_val_t *max_payload = &conf->cache.srv_max_udp_payload;
+	int ret = knot_edns_init(&opt_rr, conf_int(max_payload), 0,
+	                         KNOT_EDNS_VERSION, &pkt->mm);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
 
+	ret = knot_edns_add_option(&opt_rr, wire_read_u64(opt_data),
+	                           yp_bin_len(opt_data + sizeof(uint64_t)),
+	                           yp_bin(opt_data + sizeof(uint64_t)), &pkt->mm);
+	if (ret != KNOT_EOK) {
+		knot_rrset_clear(&opt_rr, &pkt->mm);
+		return ret;
+	}
+
+	knot_pkt_begin(pkt, KNOT_ADDITIONAL);
+
+	ret = knot_pkt_put(pkt, KNOT_COMPR_HINT_NONE, &opt_rr, KNOT_PF_FREE);
+	if (ret != KNOT_EOK) {
+		knot_rrset_clear(&opt_rr, &pkt->mm);
+		return ret;
+	}
+
+	return KNOT_EOK;
+}
+
+/*! \brief Process query using requestor. */
+static int zone_query_request(knot_pkt_t *query, const conf_remote_t *remote,
+                              struct process_answer_param *param, knot_mm_t *mm)
+{
 	/* Create requestor instance. */
 	struct knot_requestor re;
-	knot_requestor_init(&re, &mm);
-	knot_requestor_overlay(&re, KNOT_STATE_ANSWER, &param);
-
-	const knot_tsig_key_t *key = remote->key.name != NULL ?
-	                             &remote->key : NULL;
-	tsig_init(&param.tsig_ctx, key);
-
-	ret = tsig_sign_packet(&param.tsig_ctx, query);
+	int ret = knot_requestor_init(&re, mm);
 	if (ret != KNOT_EOK) {
-		goto fail;
+		return ret;
+	}
+	ret = knot_requestor_overlay(&re, KNOT_STATE_ANSWER, param);
+	if (ret != KNOT_EOK) {
+		knot_requestor_clear(&re);
+		return ret;
 	}
 
 	/* Create a request. */
@@ -133,22 +147,80 @@ static int zone_query_execute(zone_t *zone, uint16_t pkt_type, const conf_remote
 	const struct sockaddr *src = (const struct sockaddr *)&remote->via;
 	struct knot_request *req = knot_request_make(re.mm, dst, src, query, 0);
 	if (req == NULL) {
-		ret = KNOT_ENOMEM;
-		goto fail;
+		knot_requestor_clear(&re);
+		return KNOT_ENOMEM;
 	}
 
 	/* Send the queries and process responses. */
 	ret = knot_requestor_enqueue(&re, req);
 	if (ret == KNOT_EOK) {
-		conf_val_t val = conf_get(conf(), C_SRV, C_TCP_REPLY_TIMEOUT);
-		struct timeval tv = { conf_int(&val), 0 };
-		ret = knot_requestor_exec(&re, &tv);
+		conf_val_t *val = &param->conf->cache.srv_tcp_reply_timeout;
+		int timeout = conf_int(val) * 1000;
+		ret = knot_requestor_exec(&re, timeout);
+	} else {
+		knot_request_free(req, re.mm);
 	}
 
-fail:
+	/* Cleanup. */
+	knot_requestor_clear(&re);
+
+	return ret;
+}
+
+/*!
+ * \brief Create a zone event query, send it, wait for the response and process it.
+ *
+ * \note Everything in this function is executed synchronously, returns when
+ *       the query processing is either complete or an error occurs.
+ */
+static int zone_query_execute(conf_t *conf, zone_t *zone, uint16_t pkt_type,
+                              const conf_remote_t *remote)
+{
+	/* Create a memory pool for this task. */
+	knot_mm_t mm;
+	mm_ctx_mempool(&mm, MM_DEFAULT_BLKSIZE);
+
+	/* Create a query message. */
+	knot_pkt_t *query = zone_query(zone, pkt_type, &mm);
+	if (query == NULL) {
+		mp_delete(mm.ctx);
+		return KNOT_ENOMEM;
+	}
+
+	/* Set EDNS section. */
+	int ret = prepare_edns(conf, zone, query);
+	if (ret != KNOT_EOK) {
+		knot_pkt_free(&query);
+		mp_delete(mm.ctx);
+		return ret;
+	}
+
+	/* Answer processing parameters. */
+	struct process_answer_param param = {
+		.zone = zone,
+		.conf = conf,
+		.query = query,
+		.remote = &remote->addr
+	};
+
+	const knot_tsig_key_t *key = remote->key.name != NULL ?
+	                             &remote->key : NULL;
+	tsig_init(&param.tsig_ctx, key);
+
+	ret = tsig_sign_packet(&param.tsig_ctx, query);
+	if (ret != KNOT_EOK) {
+		tsig_cleanup(&param.tsig_ctx);
+		knot_pkt_free(&query);
+		mp_delete(mm.ctx);
+		return ret;
+	}
+
+	/* Process the query. */
+	ret = zone_query_request(query, remote, &param, &mm);
+
 	/* Cleanup. */
 	tsig_cleanup(&param.tsig_ctx);
-	knot_requestor_clear(&re);
+	knot_pkt_free(&query);
 	mp_delete(mm.ctx);
 
 	return ret;
@@ -163,17 +235,18 @@ fail:
 	}
 
 /*! \brief Execute zone transfer request. */
-static int zone_query_transfer(zone_t *zone, const conf_remote_t *master, uint16_t pkt_type)
+static int zone_query_transfer(conf_t *conf, zone_t *zone, const conf_remote_t *master,
+                               uint16_t pkt_type)
 {
 	assert(zone);
 	assert(master);
 
-	int ret = zone_query_execute(zone, pkt_type, master);
+	int ret = zone_query_execute(conf, zone, pkt_type, master);
 	if (ret != KNOT_EOK) {
 		/* IXFR failed, revert to AXFR. */
 		if (pkt_type == KNOT_QUERY_IXFR) {
 			ZONE_XFER_LOG(LOG_NOTICE, pkt_type, "fallback to AXFR");
-			return zone_query_transfer(zone, master, KNOT_QUERY_AXFR);
+			return zone_query_transfer(conf, zone, master, KNOT_QUERY_AXFR);
 		}
 
 		/* Log connection errors. */
@@ -218,70 +291,71 @@ static const knot_rdataset_t *zone_soa(zone_t *zone)
 }
 
 /*! \brief Fetch SOA expire timer and add a timeout grace period. */
-static uint32_t soa_graceful_expire(const knot_rdataset_t *soa)
+static uint32_t soa_graceful_expire(conf_t *conf, const knot_rdataset_t *soa)
 {
 	// Allow for timeouts.  Otherwise zones with very short
 	// expiry may expire before the timeout is reached.
-	conf_val_t val = conf_get(conf(), C_SRV, C_TCP_IDLE_TIMEOUT);
-	return knot_soa_expire(soa) + 2 * conf_int(&val);
+	conf_val_t *val = &conf->cache.srv_tcp_reply_timeout;
+	return knot_soa_expire(soa) + 2 * conf_int(val);
 }
 
 /*! \brief Schedule expire event, unless it is already scheduled. */
-static void start_expire_timer(zone_t *zone, const knot_rdataset_t *soa)
+static void start_expire_timer(conf_t *conf, zone_t *zone, const knot_rdataset_t *soa)
 {
 	if (zone_events_is_scheduled(zone, ZONE_EVENT_EXPIRE)) {
 		return;
 	}
 
-	zone_events_schedule(zone, ZONE_EVENT_EXPIRE, soa_graceful_expire(soa));
+	zone_events_schedule(zone, ZONE_EVENT_EXPIRE, soa_graceful_expire(conf, soa));
 }
 
-/* -- zone events handling callbacks --------------------------------------- */
-
-int event_reload(zone_t *zone)
+int event_load(conf_t *conf, zone_t *zone)
 {
 	assert(zone);
 
 	/* Take zone file mtime and load it. */
-	char *filename = conf_zonefile(conf(), zone->name);
+	char *filename = conf_zonefile(conf, zone->name);
 	time_t mtime = zonefile_mtime(filename);
 	free(filename);
 	uint32_t dnssec_refresh = time(NULL);
-	zone_contents_t *contents = zone_load_contents(conf(), zone->name);
-	if (!contents) {
-		return KNOT_ERROR;
+
+	zone_contents_t *contents = NULL;
+	int ret = zone_load_contents(conf, zone->name, &contents);
+	if (ret != KNOT_EOK) {
+		goto fail;
 	}
 
 	/* Store zonefile serial and apply changes from the journal. */
 	zone->zonefile_serial = zone_contents_serial(contents);
-	int result = zone_load_journal(conf(), zone, contents);
-	if (result != KNOT_EOK) {
+	ret = zone_load_journal(conf, zone, contents);
+	if (ret != KNOT_EOK) {
 		goto fail;
 	}
 
 	/* Post load actions - calculate delta, sign with DNSSEC... */
 	/*! \todo issue #242 dnssec signing should occur in the special event */
-	result = zone_load_post(conf(), contents, zone, &dnssec_refresh);
-	if (result != KNOT_EOK) {
-		if (result == KNOT_ESPACE) {
+	ret = zone_load_post(conf, zone, contents, &dnssec_refresh);
+	if (ret != KNOT_EOK) {
+		if (ret == KNOT_ESPACE) {
 			log_zone_error(zone->name, "journal size is too small "
 			               "to fit the changes");
 		} else {
 			log_zone_error(zone->name, "failed to store changes into "
-			               "journal (%s)", knot_strerror(result));
+			               "journal (%s)", knot_strerror(ret));
 		}
 		goto fail;
 	}
 
 	/* Check zone contents consistency. */
-	result = zone_load_check(conf(), contents);
-	if (result != KNOT_EOK) {
+	ret = zone_load_check(conf, contents);
+	if (ret != KNOT_EOK) {
 		goto fail;
 	}
 
 	/* Everything went alright, switch the contents. */
 	zone->zonefile_mtime = mtime;
 	zone_contents_t *old = zone_switch_contents(zone, contents);
+	zone->flags &= ~ZONE_EXPIRED;
 	uint32_t old_serial = zone_contents_serial(old);
 	if (old != NULL) {
 		synchronize_rcu();
@@ -289,7 +363,7 @@ int event_reload(zone_t *zone)
 	}
 
 	/* Schedule notify and refresh after load. */
-	if (zone_is_slave(zone)) {
+	if (zone_is_slave(conf, zone)) {
 		zone_events_schedule(zone, ZONE_EVENT_REFRESH, ZONE_EVENT_NOW);
 	}
 	if (!zone_contents_is_empty(contents)) {
@@ -298,13 +372,13 @@ int event_reload(zone_t *zone)
 	}
 
 	/* Schedule zone resign. */
-	conf_val_t val = conf_zone_get(conf(), C_DNSSEC_SIGNING, zone->name);
+	conf_val_t val = conf_zone_get(conf, C_DNSSEC_SIGNING, zone->name);
 	if (conf_bool(&val)) {
 		schedule_dnssec(zone, dnssec_refresh);
 	}
 
 	/* Periodic execution. */
-	val = conf_zone_get(conf(), C_ZONEFILE_SYNC, zone->name);
+	val = conf_zone_get(conf, C_ZONEFILE_SYNC, zone->name);
 	int64_t sync_timeout = conf_int(&val);
 	if (sync_timeout >= 0) {
 		zone_events_schedule(zone, ZONE_EVENT_FLUSH, sync_timeout);
@@ -314,19 +388,25 @@ int event_reload(zone_t *zone)
 	log_zone_info(zone->name, "loaded, serial %u -> %u",
 	              old_serial, current_serial);
 
-	return zone_events_write_persistent(zone);
+	return KNOT_EOK;
 
 fail:
 	zone_contents_deep_free(&contents);
-	return result;
+
+	/* Try to bootstrap the zone if local error. */
+	if (zone_is_slave(conf, zone) && !zone_events_is_scheduled(zone, ZONE_EVENT_XFER)) {
+		zone_events_schedule(zone, ZONE_EVENT_XFER, ZONE_EVENT_NOW);
+	}
+
+	return ret;
 }
 
-static int try_refresh(zone_t *zone, const conf_remote_t *master, void *ctx)
+static int try_refresh(conf_t *conf, zone_t *zone, const conf_remote_t *master, void *ctx)
 {
 	assert(zone);
 	assert(master);
 
-	int ret = zone_query_execute(zone, KNOT_QUERY_NORMAL, master);
+	int ret = zone_query_execute(conf, zone, KNOT_QUERY_NORMAL, master);
 	if (ret != KNOT_EOK) {
 		ZONE_QUERY_LOG(LOG_WARNING, zone, master, "refresh, outgoing",
 		               "failed (%s)", knot_strerror(ret));
@@ -335,12 +415,12 @@ static int try_refresh(zone_t *zone, const conf_remote_t *master, void *ctx)
 	return ret;
 }
 
-int event_refresh(zone_t *zone)
+int event_refresh(conf_t *conf, zone_t *zone)
 {
 	assert(zone);
 
 	/* Ignore if not slave zone. */
-	if (!zone_is_slave(zone)) {
+	if (!zone_is_slave(conf, zone)) {
 		return KNOT_EOK;
 	}
 
@@ -350,27 +430,27 @@ int event_refresh(zone_t *zone)
 		return KNOT_EOK;
 	}
 
-	int ret = zone_master_try(zone, try_refresh, NULL, "refresh");
+	int ret = zone_master_try(conf, zone, try_refresh, NULL, "refresh");
 	const knot_rdataset_t *soa = zone_soa(zone);
 	if (ret != KNOT_EOK) {
 		log_zone_error(zone->name, "refresh, failed (%s)",
 		               knot_strerror(ret));
 		/* Schedule next retry. */
 		zone_events_schedule(zone, ZONE_EVENT_REFRESH, knot_soa_retry(soa));
-		start_expire_timer(zone, soa);
+		start_expire_timer(conf, zone, soa);
 	} else {
 		/* SOA query answered, reschedule refresh timer. */
 		zone_events_schedule(zone, ZONE_EVENT_REFRESH, knot_soa_refresh(soa));
 	}
 
-	return zone_events_write_persistent(zone);
+	return KNOT_EOK;
 }
 
 struct transfer_data {
 	uint16_t pkt_type;
 };
 
-static int try_xfer(zone_t *zone, const conf_remote_t *master, void *_data)
+static int try_xfer(conf_t *conf, zone_t *zone, const conf_remote_t *master, void *_data)
 {
 	assert(zone);
 	assert(master);
@@ -378,15 +458,15 @@ static int try_xfer(zone_t *zone, const conf_remote_t *master, void *_data)
 
 	struct transfer_data *data = _data;
 
-	return zone_query_transfer(zone, master, data->pkt_type);
+	return zone_query_transfer(conf, zone, master, data->pkt_type);
 }
 
-int event_xfer(zone_t *zone)
+int event_xfer(conf_t *conf, zone_t *zone)
 {
 	assert(zone);
 
 	/* Ignore if not slave zone. */
-	if (!zone_is_slave(zone)) {
+	if (!zone_is_slave(conf, zone)) {
 		return KNOT_EOK;
 	}
 
@@ -404,7 +484,7 @@ int event_xfer(zone_t *zone)
 	}
 
 	/* Execute zone transfer. */
-	int ret = zone_master_try(zone, try_xfer, &data, err_str);
+	int ret = zone_master_try(conf, zone, try_xfer, &data, err_str);
 	zone_clear_preferred_master(zone);
 	if (ret != KNOT_EOK) {
 		log_zone_error(zone->name, "%s, failed (%s)", err_str,
@@ -415,7 +495,7 @@ int event_xfer(zone_t *zone)
 		} else {
 			const knot_rdataset_t *soa = zone_soa(zone);
 			zone_events_schedule(zone, ZONE_EVENT_XFER, knot_soa_retry(soa));
-			start_expire_timer(zone, soa);
+			start_expire_timer(conf, zone, soa);
 		}
 
 		return KNOT_EOK;
@@ -428,7 +508,7 @@ int event_xfer(zone_t *zone)
 	zone_events_schedule(zone, ZONE_EVENT_REFRESH, knot_soa_refresh(soa));
 	zone_events_schedule(zone, ZONE_EVENT_NOTIFY,  ZONE_EVENT_NOW);
 	zone_events_cancel(zone, ZONE_EVENT_EXPIRE);
-	conf_val_t val = conf_zone_get(conf(), C_ZONEFILE_SYNC, zone->name);
+	conf_val_t val = conf_zone_get(conf, C_ZONEFILE_SYNC, zone->name);
 	int64_t sync_timeout = conf_int(&val);
 	if (sync_timeout == 0) {
 		zone_events_schedule(zone, ZONE_EVENT_FLUSH, ZONE_EVENT_NOW);
@@ -449,13 +529,12 @@ int event_xfer(zone_t *zone)
 	return KNOT_EOK;
 }
 
-int event_update(zone_t *zone)
+int event_update(conf_t *conf, zone_t *zone)
 {
 	assert(zone);
 
 	/* Process update list - forward if zone has master, or execute. */
-	int ret = updates_execute(zone);
-	UNUSED(ret); /* Don't care about the Knot code, RCODEs are set. */
+	updates_execute(conf, zone);
 
 	/* Trim extra heap. */
 	mem_trim();
@@ -474,7 +553,7 @@ int event_update(zone_t *zone)
 	return KNOT_EOK;
 }
 
-int event_expire(zone_t *zone)
+int event_expire(conf_t *conf, zone_t *zone)
 {
 	assert(zone);
 
@@ -484,6 +563,7 @@ int event_expire(zone_t *zone)
 	/* Expire zonefile information. */
 	zone->zonefile_mtime = 0;
 	zone->zonefile_serial = 0;
+	zone->flags |= ZONE_EXPIRED;
 	zone_contents_deep_free(&expired);
 
 	log_zone_info(zone->name, "zone expired");
@@ -494,12 +574,12 @@ int event_expire(zone_t *zone)
 	return KNOT_EOK;
 }
 
-int event_flush(zone_t *zone)
+int event_flush(conf_t *conf, zone_t *zone)
 {
 	assert(zone);
 
 	/* Reschedule. */
-	conf_val_t val = conf_zone_get(conf(), C_ZONEFILE_SYNC, zone->name);
+	conf_val_t val = conf_zone_get(conf, C_ZONEFILE_SYNC, zone->name);
 	int64_t sync_timeout = conf_int(&val);
 	if (sync_timeout > 0) {
 		zone_events_schedule(zone, ZONE_EVENT_FLUSH, sync_timeout);
@@ -510,10 +590,10 @@ int event_flush(zone_t *zone)
 		return KNOT_EOK;
 	}
 
-	return zone_flush_journal(zone);
+	return zone_flush_journal(conf, zone);
 }
 
-int event_notify(zone_t *zone)
+int event_notify(conf_t *conf, zone_t *zone)
 {
 	assert(zone);
 
@@ -523,14 +603,14 @@ int event_notify(zone_t *zone)
 	}
 
 	/* Walk through configured remotes and send messages. */
-	conf_val_t notify = conf_zone_get(conf(), C_NOTIFY, zone->name);
+	conf_val_t notify = conf_zone_get(conf, C_NOTIFY, zone->name);
 	while (notify.code == KNOT_EOK) {
-		conf_val_t addr = conf_id_get(conf(), C_RMT, C_ADDR, &notify);
+		conf_val_t addr = conf_id_get(conf, C_RMT, C_ADDR, &notify);
 		size_t addr_count = conf_val_count(&addr);
 
 		for (int i = 0; i < addr_count; i++) {
-			conf_remote_t slave = conf_remote(conf(), &notify, i);
-			int ret = zone_query_execute(zone, KNOT_QUERY_NOTIFY, &slave);
+			conf_remote_t slave = conf_remote(conf, &notify, i);
+			int ret = zone_query_execute(conf, zone, KNOT_QUERY_NOTIFY, &slave);
 			if (ret == KNOT_EOK) {
 				ZONE_QUERY_LOG(LOG_INFO, zone, &slave,
 				               "NOTIFY, outgoing", "serial %u",
@@ -549,7 +629,7 @@ int event_notify(zone_t *zone)
 	return KNOT_EOK;
 }
 
-int event_dnssec(zone_t *zone)
+int event_dnssec(conf_t *conf, zone_t *zone)
 {
 	assert(zone);
 
@@ -589,7 +669,7 @@ int event_dnssec(zone_t *zone)
 		}
 
 		/* Write change to journal. */
-		ret = zone_change_store(zone, &ch);
+		ret = zone_change_store(conf, zone, &ch);
 		if (ret != KNOT_EOK) {
 			log_zone_error(zone->name, "DNSSEC, failed to sign zone (%s)",
 				       knot_strerror(ret));
@@ -600,6 +680,7 @@ int event_dnssec(zone_t *zone)
 
 		/* Switch zone contents. */
 		zone_contents_t *old_contents = zone_switch_contents(zone, new_contents);
+		zone->flags &= ~ZONE_EXPIRED;
 		synchronize_rcu();
 		update_free_zone(&old_contents);
 
@@ -611,7 +692,7 @@ int event_dnssec(zone_t *zone)
 	schedule_dnssec(zone, refresh_at);
 	if (zone_changed) {
 		zone_events_schedule(zone, ZONE_EVENT_NOTIFY, ZONE_EVENT_NOW);
-		conf_val_t val = conf_zone_get(conf(), C_ZONEFILE_SYNC, zone->name);
+		conf_val_t val = conf_zone_get(conf, C_ZONEFILE_SYNC, zone->name);
 		if (conf_int(&val) == 0) {
 			zone_events_schedule(zone, ZONE_EVENT_FLUSH, ZONE_EVENT_NOW);
 		}
