@@ -1,4 +1,4 @@
-/*  Copyright (C) 2014 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,24 +16,25 @@
 
 #pragma once
 
-#include "knot/zone/events/events.h"
+#include "knot/conf/conf.h"
+#include "knot/zone/zone.h"
 
-/*! \brief Reloads the potentially changed zone. */
-int event_reload(zone_t *zone);
+/*! \brief Loads or reloads potentially changed zone. */
+int event_load(conf_t *conf, zone_t *zone);
 /*! \brief Sends a SOA query to master. */
-int event_refresh(zone_t *zone);
+int event_refresh(conf_t *conf, zone_t *zone);
 /*! \brief Initiates transfer with master. */
-int event_xfer(zone_t *zone);
+int event_xfer(conf_t *conf, zone_t *zone);
 /*! \brief Processes DDNS updates in the zone's DDNS queue. */
-int event_update(zone_t *zone);
+int event_update(conf_t *conf, zone_t *zone);
 /*! \brief Empties in-memory zone contents. */
-int event_expire(zone_t *zone);
+int event_expire(conf_t *conf, zone_t *zone);
 /*! \brief Flushes zone contents into text file. */
-int event_flush(zone_t *zone);
+int event_flush(conf_t *conf, zone_t *zone);
 /*! \brief Sends notify to slaves. */
-int event_notify(zone_t *zone);
+int event_notify(conf_t *conf, zone_t *zone);
 /*! \brief (re)Signs the zone using its DNSSEC keys. */
-int event_dnssec(zone_t *zone);
+int event_dnssec(conf_t *conf, zone_t *zone);
 
 /*! \brief Progressive bootstrap retry timer. */
 uint32_t bootstrap_next(uint32_t timer);
