@@ -1,4 +1,4 @@
-/*  Copyright (C) 2014 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,14 +14,10 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "libknot/dname.h"
-#include "libknot/internal/namedb/namedb.h"
-#include "libknot/internal/namedb/namedb_lmdb.h"
-#include "libknot/internal/mem.h"
+#include "libknot/libknot.h"
 #include "knot/zone/timers.h"
-#include "knot/zone/zonedb.h"
-
-/* ---- Knot-internal event code to db key lookup tables ------------------ - */
+#include "contrib/wire.h"
+#include "contrib/wire_ctx.h"
 
 #define PERSISTENT_EVENT_COUNT 3
 
@@ -63,40 +59,56 @@ static void clear_timers(time_t *timers)
 }
 
 /*! \brief Stores timers for persistent events. */
-static int store_timers(namedb_txn_t *txn, zone_t *zone)
+static int store_timers(zone_t *zone, knot_db_txn_t *txn)
 {
 	// Create key
-	namedb_val_t key = { .len = knot_dname_size(zone->name), .data = zone->name };
+	knot_db_val_t key = { .len = knot_dname_size(zone->name), .data = zone->name };
 
 	// Create value
 	uint8_t packed_timer[EVENT_KEY_PAIR_SIZE * PERSISTENT_EVENT_COUNT];
-	size_t offset = 0;
+
+	wire_ctx_t w = wire_ctx_init(packed_timer, sizeof(packed_timer));
+
 	for (zone_event_type_t event = 0; event < ZONE_EVENT_COUNT; ++event) {
 		if (!event_persistent(event)) {
 			continue;
 		}
 
-		// Write event key and timer to buffer
-		packed_timer[offset] = event_id_to_key[event];
-		offset += 1;
-		wire_write_u64(packed_timer + offset,
-		                    (int64_t)zone_events_get_time(zone, event));
-		offset += sizeof(uint64_t);
+		// Key
+		wire_ctx_write_u8(&w, event_id_to_key[event]);
+
+		// Value
+		time_t value = zone_events_get_time(zone, event);
+		if (event == ZONE_EVENT_EXPIRE && zone->flags & ZONE_EXPIRED) {
+			/*
+			 * WORKAROUND. The current timer database contains
+			 * time stamps for running timers. The expiration
+			 * in past indicates that the zone expired. We need
+			 * to preserve this status across server restarts.
+			 */
+			value = 1;
+		}
+		wire_ctx_write_u64(&w, value);
 	}
-	namedb_val_t val = { .len = sizeof(packed_timer), .data = packed_timer };
+
+	if (w.error != KNOT_EOK) {
+		return w.error;
+	}
+
+	knot_db_val_t val = { .len = sizeof(packed_timer), .data = packed_timer };
 
 	// Store
-	return namedb_lmdb_api()->insert(txn, &key, &val, 0);
+	return knot_db_lmdb_api()->insert(txn, &key, &val, 0);
 }
 
 /*! \brief Reads timers for persistent events. */
-static int read_timers(namedb_txn_t *txn, const zone_t *zone, time_t *timers)
+static int read_timers(knot_db_txn_t *txn, const zone_t *zone, time_t *timers)
 {
-	const namedb_api_t *db_api = namedb_lmdb_api();
+	const knot_db_api_t *db_api = knot_db_lmdb_api();
 	assert(db_api);
 
-	namedb_val_t key = { .len = knot_dname_size(zone->name), .data = zone->name };
-	namedb_val_t val;
+	knot_db_val_t key = { .len = knot_dname_size(zone->name), .data = zone->name };
+	knot_db_val_t val;
 
 	int ret = db_api->find(txn, &key, &val, 0);
 	if (ret != KNOT_EOK && ret != KNOT_ENOENT) {
@@ -124,56 +136,52 @@ static int read_timers(namedb_txn_t *txn, const zone_t *zone, time_t *timers)
 	return KNOT_EOK;
 }
 
-/* -------- API ------------------------------------------------------------- */
-
-int open_timers_db(const char *storage, namedb_t **db_ptr)
+int open_timers_db(const char *path, knot_db_t **timer_db)
 {
-	if (storage == NULL || db_ptr == NULL) {
+	if (path == NULL || timer_db == NULL) {
 		return KNOT_EINVAL;
 	}
 
-	struct namedb_lmdb_opts opts = NAMEDB_LMDB_OPTS_INITIALIZER;
-	const namedb_api_t *db_api = namedb_lmdb_api();
+	const knot_db_api_t *db_api = knot_db_lmdb_api();
 	if (db_api == NULL) {
 		return KNOT_ENOTSUP;
 	}
 
-	opts.path = sprintf_alloc("%s/timers", storage);
-	if (opts.path == NULL) {
-		return KNOT_ENOMEM;
-	}
+	struct knot_db_lmdb_opts opts = KNOT_DB_LMDB_OPTS_INITIALIZER;
+	opts.mapsize = (size_t)TIMER_MAPSIZE * 1024 * 1024;
+	opts.path = path;
 
-	int ret = db_api->init(db_ptr, NULL, &opts);
-
-	free((char *)opts.path);
-
-	return ret;
+	return db_api->init(timer_db, NULL, &opts);
 }
 
-void close_timers_db(namedb_t *timer_db)
+void close_timers_db(knot_db_t *timer_db)
 {
 	if (timer_db == NULL) {
 		return;
 	}
 
-	const namedb_api_t *db_api = namedb_lmdb_api();
+	const knot_db_api_t *db_api = knot_db_lmdb_api();
 	assert(db_api);
 
 	db_api->deinit(timer_db);
 }
 
-int read_zone_timers(namedb_t *timer_db, const zone_t *zone, time_t *timers)
+int read_zone_timers(knot_db_t *timer_db, const zone_t *zone, time_t *timers)
 {
 	if (timer_db == NULL) {
 		clear_timers(timers);
 		return KNOT_EOK;
 	}
 
-	const namedb_api_t *db_api = namedb_lmdb_api();
+	if (zone == NULL || timers == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	const knot_db_api_t *db_api = knot_db_lmdb_api();
 	assert(db_api);
 
-	namedb_txn_t txn;
-	int ret = db_api->txn_begin(timer_db, &txn, NAMEDB_RDONLY);
+	knot_db_txn_t txn;
+	int ret = db_api->txn_begin(timer_db, &txn, KNOT_DB_RDONLY);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
@@ -187,41 +195,45 @@ int read_zone_timers(namedb_t *timer_db, const zone_t *zone, time_t *timers)
 	return KNOT_EOK;
 }
 
-int write_zone_timers(namedb_t *timer_db, zone_t *zone)
+int write_timer_db(knot_db_t *timer_db, knot_zonedb_t *zone_db)
 {
 	if (timer_db == NULL) {
 		return KNOT_EOK;
 	}
 
-	const namedb_api_t *db_api = namedb_lmdb_api();
+	if (zone_db == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	const knot_db_api_t *db_api = knot_db_lmdb_api();
 	assert(db_api);
 
-	namedb_txn_t txn;
-	int ret = db_api->txn_begin(timer_db, &txn, 0);
+	knot_db_txn_t txn;
+	int ret = db_api->txn_begin(timer_db, &txn, KNOT_DB_SORTED);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
 
-	ret = store_timers(&txn, zone);
-	if (ret != KNOT_EOK) {
-		db_api->txn_abort(&txn);
-		return ret;
-	}
+	knot_zonedb_foreach(zone_db, store_timers, &txn);
 
 	return db_api->txn_commit(&txn);
 }
 
-int sweep_timer_db(namedb_t *timer_db, knot_zonedb_t *zone_db)
+int sweep_timer_db(knot_db_t *timer_db, knot_zonedb_t *zone_db)
 {
 	if (timer_db == NULL) {
 		return KNOT_EOK;
 	}
 
-	const namedb_api_t *db_api = namedb_lmdb_api();
+	if (zone_db == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	const knot_db_api_t *db_api = knot_db_lmdb_api();
 	assert(db_api);
 
-	namedb_txn_t txn;
-	int ret = db_api->txn_begin(timer_db, &txn, NAMEDB_SORTED);
+	knot_db_txn_t txn;
+	int ret = db_api->txn_begin(timer_db, &txn, KNOT_DB_SORTED);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
@@ -231,14 +243,14 @@ int sweep_timer_db(namedb_t *timer_db, knot_zonedb_t *zone_db)
 		return KNOT_EOK;
 	}
 
-	namedb_iter_t *it = db_api->iter_begin(&txn, 0);
+	knot_db_iter_t *it = db_api->iter_begin(&txn, 0);
 	if (it == NULL) {
 		db_api->txn_abort(&txn);
 		return KNOT_ERROR;
 	}
 
 	while (it) {
-		namedb_val_t key;
+		knot_db_val_t key;
 		ret = db_api->iter_key(it, &key);
 		if (ret != KNOT_EOK) {
 			db_api->txn_abort(&txn);
