@@ -1,4 +1,4 @@
-/*  Copyright (C) 2011 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,25 +16,17 @@
 
 #include <assert.h>
 #include <stdlib.h>
-#include <stdio.h>
 
 #include "knot/zone/zone-tree.h"
-#include "knot/zone/node.h"
-#include "knot/common/debug.h"
-#include "libknot/internal/trie/hat-trie.h"
-#include "libknot/internal/macros.h"
-
-/*----------------------------------------------------------------------------*/
-/* API functions                                                              */
-/*----------------------------------------------------------------------------*/
+#include "libknot/consts.h"
+#include "libknot/errcode.h"
+#include "contrib/macros.h"
 
 zone_tree_t* zone_tree_create()
 {
 	return hattrie_create();
 }
 
-/*----------------------------------------------------------------------------*/
-
 size_t zone_tree_weight(const zone_tree_t* tree)
 {
 	return hattrie_weight(tree);
@@ -45,8 +37,6 @@ int zone_tree_is_empty(const zone_tree_t *tree)
 	return zone_tree_weight(tree) == 0;
 }
 
-/*----------------------------------------------------------------------------*/
-
 int zone_tree_insert(zone_tree_t *tree, zone_node_t *node)
 {
 	if (tree == NULL) {
@@ -61,8 +51,6 @@ int zone_tree_insert(zone_tree_t *tree, zone_node_t *node)
 	return KNOT_EOK;
 }
 
-/*----------------------------------------------------------------------------*/
-
 int zone_tree_find(zone_tree_t *tree, const knot_dname_t *owner,
                           const zone_node_t **found)
 {
@@ -73,8 +61,6 @@ int zone_tree_find(zone_tree_t *tree, const knot_dname_t *owner,
 	return zone_tree_get(tree, owner, (zone_node_t **)found);
 }
 
-/*----------------------------------------------------------------------------*/
-
 int zone_tree_get(zone_tree_t *tree, const knot_dname_t *owner,
                          zone_node_t **found)
 {
@@ -99,8 +85,6 @@ int zone_tree_get(zone_tree_t *tree, const knot_dname_t *owner,
 	return KNOT_EOK;
 }
 
-/*----------------------------------------------------------------------------*/
-
 int zone_tree_find_less_or_equal(zone_tree_t *tree,
                                  const knot_dname_t *owner,
                                  const zone_node_t **found,
@@ -119,8 +103,6 @@ int zone_tree_find_less_or_equal(zone_tree_t *tree,
 	return ret;
 }
 
-/*----------------------------------------------------------------------------*/
-
 int zone_tree_get_less_or_equal(zone_tree_t *tree,
                                 const knot_dname_t *owner,
                                 zone_node_t **found,
@@ -170,29 +152,9 @@ int zone_tree_get_less_or_equal(zone_tree_t *tree,
 		*previous = (*previous)->prev;
 	}
 
-dbg_zone_exec_detail(
-		char *name = knot_dname_to_str_alloc(owner);
-		char *name_f = (*found != NULL)
-			? knot_dname_to_str_alloc((*found)->owner)
-			: "none";
-
-		dbg_zone_detail("Searched for owner %s in zone tree.\n",
-				name);
-		dbg_zone_detail("Exact match: %d\n", exact_match);
-		dbg_zone_detail("Found node: %p: %s.\n", *found, name_f);
-		dbg_zone_detail("Previous node: %p.\n", *previous);
-
-		free(name);
-		if (*found != NULL) {
-			free(name_f);
-		}
-);
-
 	return exact_match;
 }
 
-/*----------------------------------------------------------------------------*/
-
 zone_node_t *zone_tree_get_next(zone_tree_t *tree,
                                 const knot_dname_t *owner)
 {
@@ -225,8 +187,6 @@ zone_node_t *zone_tree_get_next(zone_tree_t *tree,
 	}
 }
 
-/*----------------------------------------------------------------------------*/
-
 int zone_tree_remove(zone_tree_t *tree,
                      const knot_dname_t *owner,
                      zone_node_t **removed)
@@ -253,8 +213,6 @@ int zone_tree_remove(zone_tree_t *tree,
 	return KNOT_EOK;
 }
 
-/*----------------------------------------------------------------------------*/
-
 int zone_tree_apply_inorder(zone_tree_t *tree,
                             zone_tree_apply_cb_t function,
                             void *data)
@@ -282,8 +240,6 @@ int zone_tree_apply_inorder(zone_tree_t *tree,
 	return result;
 }
 
-/*----------------------------------------------------------------------------*/
-
 int zone_tree_apply(zone_tree_t *tree,
                     zone_tree_apply_cb_t function,
                     void *data)
@@ -299,8 +255,6 @@ int zone_tree_apply(zone_tree_t *tree,
 	return hattrie_apply_rev(tree, (int (*)(value_t*,void*))function, data);
 }
 
-/*----------------------------------------------------------------------------*/
-
 void zone_tree_free(zone_tree_t **tree)
 {
 	if (tree == NULL || *tree == NULL) {
@@ -310,8 +264,6 @@ void zone_tree_free(zone_tree_t **tree)
 	*tree = NULL;
 }
 
-/*----------------------------------------------------------------------------*/
-
 static int zone_tree_free_node(zone_node_t **node, void *data)
 {
 	UNUSED(data);
