@@ -1,4 +1,4 @@
-/*  Copyright (C) 2011 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,11 +14,11 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 /*!
- * \file zone.h
+ * \file
  *
  * \brief Zone structure and API for manipulating it.
  *
- * \addtogroup libknot
+ * \addtogroup zone
  * @{
  */
 
@@ -28,11 +28,8 @@
 #include <stdbool.h>
 #include <stdint.h>
 
-#include "knot/common/evsched.h"
-#include "knot/common/ref.h"
 #include "knot/conf/conf.h"
 #include "knot/server/journal.h"
-#include "knot/updates/acl.h"
 #include "knot/zone/events/events.h"
 #include "knot/zone/contents.h"
 #include "libknot/dname.h"
@@ -46,7 +43,8 @@ struct process_query_param;
 typedef enum zone_flag_t {
 	ZONE_FORCE_AXFR   = 1 << 0, /* Force AXFR as next transfer. */
 	ZONE_FORCE_RESIGN = 1 << 1, /* Force zone resign. */
-	ZONE_FORCE_FLUSH  = 1 << 2  /* Force zone flush. */
+	ZONE_FORCE_FLUSH  = 1 << 2, /* Force zone flush. */
+	ZONE_EXPIRED      = 1 << 3, /* Zone is expired. */
 } zone_flag_t;
 
 /*!
@@ -82,8 +80,6 @@ typedef struct zone
 	struct query_plan *query_plan;
 } zone_t;
 
-/*----------------------------------------------------------------------------*/
-
 /*!
  * \brief Creates new zone with emtpy zone content.
  *
@@ -96,7 +92,7 @@ zone_t* zone_new(const knot_dname_t *name);
 /*!
  * \brief Deallocates the zone structure.
  *
- * \note The function also deallocates all bound structures (config, contents, etc.).
+ * \note The function also deallocates all bound structures (contents, etc.).
  *
  * \param zone Zone to be freed.
  */
@@ -107,22 +103,24 @@ void zone_free(zone_t **zone_ptr);
  * \ref #223 New zone API
  * \todo get rid of this
  */
-int zone_changes_store(zone_t *zone, list_t *chgs);
-int zone_change_store(zone_t *zone, changeset_t *change);
+int zone_changes_store(conf_t *conf, zone_t *zone, list_t *chgs);
+int zone_change_store(conf_t *conf, zone_t *zone, changeset_t *change);
 /*!
  * \brief Atomically switch the content of the zone.
  */
 zone_contents_t *zone_switch_contents(zone_t *zone, zone_contents_t *new_contents);
 
 /*! \brief Checks if the zone is slave. */
-bool zone_is_slave(const zone_t *zone);
+bool zone_is_slave(conf_t *conf, const zone_t *zone);
 
 /*! \brief Sets the address as a preferred master address. */
 void zone_set_preferred_master(zone_t *zone, const struct sockaddr_storage *addr);
+
 /*! \brief Clears the current preferred master address. */
 void zone_clear_preferred_master(zone_t *zone);
 
-typedef int (*zone_master_cb)(zone_t *zone, const conf_remote_t *remote, void *data);
+typedef int (*zone_master_cb)(conf_t *conf, zone_t *zone, const conf_remote_t *remote,
+                              void *data);
 
 /*!
  * \brief Perform an action with a first working master server.
@@ -133,11 +131,11 @@ typedef int (*zone_master_cb)(zone_t *zone, const conf_remote_t *remote, void *d
  *
  * \return Error code from the last callback.
  */
-int zone_master_try(zone_t *zone, zone_master_cb callback, void *callback_data,
-                    const char *err_str);
+int zone_master_try(conf_t *conf, zone_t *zone, zone_master_cb callback,
+                    void *callback_data, const char *err_str);
 
 /*! \brief Synchronize zone file with journal. */
-int zone_flush_journal(zone_t *zone);
+int zone_flush_journal(conf_t *conf, zone_t *zone);
 
 /*! \brief Enqueue UPDATE request for processing. */
 int zone_update_enqueue(zone_t *zone, knot_pkt_t *pkt, struct process_query_param *param);
