@@ -1,4 +1,4 @@
-/*  Copyright (C) 2013 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2015 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,22 +15,17 @@
 */
 
 #include <assert.h>
-#include <sys/stat.h>
 #include <urcu.h>
 
 #include "knot/zone/zonedb-load.h"
 #include "knot/zone/zone-load.h"
-#include "knot/conf/conf.h"
 #include "knot/zone/zone.h"
 #include "knot/zone/zonefile.h"
 #include "knot/zone/zonedb.h"
 #include "knot/zone/timers.h"
-#include "knot/server/server.h"
 #include "knot/common/log.h"
 #include "libknot/libknot.h"
 
-/*- zone file status --------------------------------------------------------*/
-
 /*!
  * \brief Zone file status.
  */
@@ -45,12 +40,12 @@ typedef enum {
 /*!
  * \brief Check zone file status.
  *
- * \param old_zone  Previous instance of the zone (can be NULL).
  * \param conf      Zone configuration.
+ * \param old_zone  Previous instance of the zone (can be NULL).
  *
  * \return Zone status.
  */
-static zone_status_t zone_file_status(const zone_t *old_zone, conf_t *conf,
+static zone_status_t zone_file_status(conf_t *conf, const zone_t *old_zone,
                                       const knot_dname_t *name)
 {
 	assert(conf);
@@ -81,8 +76,6 @@ static zone_status_t zone_file_status(const zone_t *old_zone, conf_t *conf,
 	}
 }
 
-/*- zone loading/updating ---------------------------------------------------*/
-
 /*!
  * \brief Log message about loaded zone (name and status).
  *
@@ -140,12 +133,18 @@ static zone_t *create_zone_reload(conf_t *conf, const knot_dname_t *name,
 	}
 	zone->contents = old_zone->contents;
 
-	const zone_status_t zstatus = zone_file_status(old_zone, conf, name);
+	zone_status_t zstatus;
+	if (zone_is_slave(conf, zone) && old_zone->flags & ZONE_EXPIRED) {
+		zone->flags |= ZONE_EXPIRED;
+		zstatus = ZONE_STATUS_FOUND_CURRENT;
+	} else {
+		zstatus = zone_file_status(conf, old_zone, name);
+	}
 
 	switch (zstatus) {
 	case ZONE_STATUS_FOUND_UPDATED:
 		/* Enqueueing makes the first zone load waitable. */
-		zone_events_enqueue(zone, ZONE_EVENT_RELOAD);
+		zone_events_enqueue(zone, ZONE_EVENT_LOAD);
 		/* Replan DDNS processing if there are pending updates. */
 		zone_events_replan_ddns(zone, old_zone);
 		break;
@@ -153,9 +152,7 @@ static zone_t *create_zone_reload(conf_t *conf, const knot_dname_t *name,
 		zone->zonefile_mtime = old_zone->zonefile_mtime;
 		zone->zonefile_serial = old_zone->zonefile_serial;
 		/* Reuse events from old zone. */
-		zone_events_update(zone, old_zone);
-		/* Write updated timers. */
-		zone_events_write_persistent(zone);
+		zone_events_update(conf, zone, old_zone);
 		break;
 	default:
 		assert(0);
@@ -169,29 +166,38 @@ static bool slave_event(zone_event_type_t event)
 	return event == ZONE_EVENT_EXPIRE || event == ZONE_EVENT_REFRESH;
 }
 
-static void reuse_events(zone_t *zone, const time_t *timers)
+static int reuse_events(conf_t *conf, knot_db_t *timer_db, zone_t *zone)
 {
+	// Get persistent timers
+
+	time_t timers[ZONE_EVENT_COUNT] = { 0 };
+	int ret = read_zone_timers(timer_db, zone, timers);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
 	for (zone_event_type_t event = 0; event < ZONE_EVENT_COUNT; ++event) {
 		if (timers[event] == 0) {
 			// Timer unset.
 			continue;
 		}
-		if (slave_event(event) && !zone_is_slave(zone)) {
+		if (slave_event(event) && !zone_is_slave(conf, zone)) {
 			// Slave-only event.
 			continue;
 		}
 
+		if (event == ZONE_EVENT_EXPIRE && timers[event] <= time(NULL)) {
+			zone->flags |= ZONE_EXPIRED;
+			continue;
+		}
+
 		zone_events_schedule_at(zone, event, timers[event]);
 	}
+
+	return KNOT_EOK;
 }
 
-static bool zone_expired(const time_t *timers)
-{
-	const time_t now = time(NULL);
-	return now <= timers[ZONE_EVENT_EXPIRE];
-}
-
-static zone_t *create_zone_new(conf_t * conf, const knot_dname_t *name,
+static zone_t *create_zone_new(conf_t *conf, const knot_dname_t *name,
                                server_t *server)
 {
 	zone_t *zone = create_zone_from(name, server);
@@ -199,11 +205,7 @@ static zone_t *create_zone_new(conf_t * conf, const knot_dname_t *name,
 		return NULL;
 	}
 
-	time_t timers[ZONE_EVENT_COUNT];
-	memset(timers, 0, sizeof(timers));
-
-	// Get persistent timers
-	int ret = read_zone_timers(server->timers_db, zone, timers);
+	int ret = reuse_events(conf, server->timers_db, zone);
 	if (ret != KNOT_EOK) {
 		log_zone_error(zone->name, "cannot read zone timers (%s)",
 		               knot_strerror(ret));
@@ -211,19 +213,19 @@ static zone_t *create_zone_new(conf_t * conf, const knot_dname_t *name,
 		return NULL;
 	}
 
-	reuse_events(zone, timers);
-
-	const zone_status_t zstatus = zone_file_status(NULL, conf, name);
+	zone_status_t zstatus = zone_file_status(conf, NULL, name);
+	if (zone->flags & ZONE_EXPIRED) {
+		assert(zone_is_slave(conf, zone));
+		zstatus = ZONE_STATUS_BOOSTRAP;
+	}
 
 	switch (zstatus) {
 	case ZONE_STATUS_FOUND_NEW:
-		if (!zone_expired(timers)) {
-			/* Enqueueing makes the first zone load waitable. */
-			zone_events_enqueue(zone, ZONE_EVENT_RELOAD);
-		}
+		/* Enqueueing makes the first zone load waitable. */
+		zone_events_enqueue(zone, ZONE_EVENT_LOAD);
 		break;
 	case ZONE_STATUS_BOOSTRAP:
-		if (timers[ZONE_EVENT_REFRESH] == 0) {
+		if (zone_events_get_time(zone, ZONE_EVENT_REFRESH) == 0) {
 			// Plan immediate refresh if not already planned.
 			zone_events_schedule(zone, ZONE_EVENT_REFRESH, ZONE_EVENT_NOW);
 		}
@@ -284,31 +286,21 @@ static knot_zonedb_t *create_zonedb(conf_t *conf, server_t *server)
 		return NULL;
 	}
 
-	conf_iter_t iter = conf_iter(conf, C_ZONE);
-	while (iter.code == KNOT_EOK) {
+	for (conf_iter_t iter = conf_iter(conf, C_ZONE); iter.code == KNOT_EOK;
+	     conf_iter_next(conf, &iter)) {
 		conf_val_t id = conf_iter_id(conf, &iter);
 		zone_t *old_zone = knot_zonedb_find(db_old, conf_dname(&id));
 		zone_t *zone = create_zone(conf, conf_dname(&id), server, old_zone);
 		if (!zone) {
 			log_zone_error(id.data, "zone cannot be created");
-			conf_iter_next(conf, &iter);
 			continue;
 		}
 
-		int ret = conf_activate_modules(conf, zone->name, &zone->query_modules,
-		                                &zone->query_plan);
-		if (ret != KNOT_EOK) {
-			zone_free(&zone);
-			log_zone_error(id.data, "cannot activate modules");
-			conf_iter_next(conf, &iter);
-			continue;
-		}
+		conf_activate_modules(conf, zone->name, &zone->query_modules,
+		                      &zone->query_plan);
 
 		knot_zonedb_insert(db_new, zone);
-
-		conf_iter_next(conf, &iter);
 	}
-	conf_iter_finish(conf, &iter);
 
 	return db_new;
 }
@@ -322,10 +314,10 @@ static knot_zonedb_t *create_zonedb(conf_t *conf, server_t *server)
  * \param db_new New zone database.
  * \param db_old Old zone database.
   */
-static int remove_old_zonedb(const knot_zonedb_t *db_new, knot_zonedb_t *db_old)
+static void remove_old_zonedb(const knot_zonedb_t *db_new, knot_zonedb_t *db_old)
 {
 	if (db_old == NULL) {
-		return KNOT_EOK;
+		return;
 	}
 
 	knot_zonedb_iter_t it;
@@ -343,27 +335,20 @@ static int remove_old_zonedb(const knot_zonedb_t *db_new, knot_zonedb_t *db_old)
 	}
 
 	knot_zonedb_deep_free(&db_old);
-
-	return KNOT_EOK;
 }
 
-/*- public API functions ----------------------------------------------------*/
-
-/*!
- * \brief Update zone database according to configuration.
- */
-int zonedb_reload(conf_t *conf, struct server *server)
+void zonedb_reload(conf_t *conf, server_t *server)
 {
 	/* Check parameters */
 	if (conf == NULL || server == NULL) {
-		return KNOT_EINVAL;
+		return;
 	}
 
 	/* Insert all required zones to the new zone DB. */
 	knot_zonedb_t *db_new = create_zonedb(conf, server);
 	if (db_new == NULL) {
 		log_error("failed to create new zone database");
-		return KNOT_ENOMEM;
+		return;
 	}
 
 	/* Rebuild zone database search stack. */
@@ -384,5 +369,5 @@ int zonedb_reload(conf_t *conf, struct server *server)
 	 * No new thread can access these zones in the old DB, as the
 	 * databases are already switched.
 	 */
-	return remove_old_zonedb(db_new, db_old);
+	remove_old_zonedb(db_new, db_old);
 }
