@@ -17,23 +17,11 @@
 #include <stdlib.h>
 #include <assert.h>
 
-#include <urcu.h>
-
 #include "knot/zone/zonedb.h"
-#include "knot/server/server.h"
-#include "knot/zone/zone.h"
-#include "knot/zone/zonedb.h"
-#include "libknot/dname.h"
 #include "libknot/packet/wire.h"
-#include "knot/zone/node.h"
-#include "knot/common/debug.h"
-#include "libknot/internal/mempattern.h"
-#include "libknot/internal/mempool.h"
-#include "libknot/internal/macros.h"
-
-/*----------------------------------------------------------------------------*/
-/* Non-API functions                                                          */
-/*----------------------------------------------------------------------------*/
+#include "contrib/macros.h"
+#include "contrib/mempattern.h"
+#include "contrib/ucw/mempool.h"
 
 /*! \brief Discard zone in zone database. */
 static void discard_zone(zone_t *zone)
@@ -43,7 +31,7 @@ static void discard_zone(zone_t *zone)
 	/* Flush if bootstrapped or if the journal doesn't exist. */
 	if (zone->zonefile_mtime == 0 || !journal_exists(journal_file)) {
 		pthread_mutex_lock(&zone->journal_lock);
-		zone_flush_journal(zone);
+		zone_flush_journal(conf(), zone);
 		pthread_mutex_unlock(&zone->journal_lock);
 	}
 
@@ -52,14 +40,10 @@ static void discard_zone(zone_t *zone)
 	zone_free(&zone);
 }
 
-/*----------------------------------------------------------------------------*/
-/* API functions                                                              */
-/*----------------------------------------------------------------------------*/
-
 knot_zonedb_t *knot_zonedb_new(uint32_t size)
 {
 	/* Create memory pool context. */
-	mm_ctx_t mm = {0};
+	knot_mm_t mm = {0};
 	mm_ctx_mempool(&mm, MM_DEFAULT_BLKSIZE);
 	knot_zonedb_t *db = mm.alloc(mm.ctx, sizeof(knot_zonedb_t));
 	if (db == NULL) {
@@ -73,12 +57,10 @@ knot_zonedb_t *knot_zonedb_new(uint32_t size)
 		return NULL;
 	}
 
-	memcpy(&db->mm, &mm, sizeof(mm_ctx_t));
+	memcpy(&db->mm, &mm, sizeof(knot_mm_t));
 	return db;
 }
 
-/*----------------------------------------------------------------------------*/
-
 int knot_zonedb_insert(knot_zonedb_t *db, zone_t *zone)
 {
 	if (db == NULL || zone == NULL) {
@@ -93,8 +75,6 @@ int knot_zonedb_insert(knot_zonedb_t *db, zone_t *zone)
 	return hhash_insert(db->hash, (const char*)zone->name, name_size, zone);
 }
 
-/*----------------------------------------------------------------------------*/
-
 int knot_zonedb_del(knot_zonedb_t *db, const knot_dname_t *zone_name)
 {
 	if (db == NULL || zone_name == NULL) {
@@ -108,8 +88,6 @@ int knot_zonedb_del(knot_zonedb_t *db, const knot_dname_t *zone_name)
 	return hhash_del(db->hash, (const char*)zone_name, name_size);
 }
 
-/*----------------------------------------------------------------------------*/
-
 int knot_zonedb_build_index(knot_zonedb_t *db)
 {
 	if (db == NULL) {
@@ -132,8 +110,6 @@ int knot_zonedb_build_index(knot_zonedb_t *db)
 	return KNOT_EOK;
 }
 
-/*----------------------------------------------------------------------------*/
-
 static value_t *find_name(knot_zonedb_t *db, const knot_dname_t *dname, uint16_t size)
 {
 	assert(db);
@@ -142,8 +118,6 @@ static value_t *find_name(knot_zonedb_t *db, const knot_dname_t *dname, uint16_t
 	return hhash_find(db->hash, (const char*)dname, size);
 }
 
-/*----------------------------------------------------------------------------*/
-
 zone_t *knot_zonedb_find(knot_zonedb_t *db, const knot_dname_t *zone_name)
 {
 	int name_size = knot_dname_size(zone_name);
@@ -159,8 +133,6 @@ zone_t *knot_zonedb_find(knot_zonedb_t *db, const knot_dname_t *zone_name)
 	return *ret;
 }
 
-/*----------------------------------------------------------------------------*/
-
 zone_t *knot_zonedb_find_suffix(knot_zonedb_t *db, const knot_dname_t *dname)
 {
 	if (db == NULL || dname == NULL) {
@@ -192,8 +164,6 @@ zone_t *knot_zonedb_find_suffix(knot_zonedb_t *db, const knot_dname_t *dname)
 	return NULL;
 }
 
-/*----------------------------------------------------------------------------*/
-
 size_t knot_zonedb_size(const knot_zonedb_t *db)
 {
 	if (db == NULL) {
@@ -203,8 +173,6 @@ size_t knot_zonedb_size(const knot_zonedb_t *db)
 	return db->hash->weight;
 }
 
-/*----------------------------------------------------------------------------*/
-
 void knot_zonedb_free(knot_zonedb_t **db)
 {
 	if (db == NULL || *db == NULL) {
@@ -215,8 +183,6 @@ void knot_zonedb_free(knot_zonedb_t **db)
 	*db = NULL;
 }
 
-/*----------------------------------------------------------------------------*/
-
 void knot_zonedb_deep_free(knot_zonedb_t **db)
 {
 	if (db == NULL || *db == NULL) {
