@@ -14,13 +14,9 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 /*!
- * \file zone-load.h
- *
- * \author Marek Vavrusa <marek.vavrusa@nic.cz>
- * \author Jan Kadlec <jan.kadlec@nic.cz>
- *
- * \brief Zone loading
+ * \file
  *
+ * \addtogroup zone
  * @{
  */
 
@@ -37,9 +33,8 @@
  */
 typedef struct zcreator {
 	zone_contents_t *z;  /*!< Created zone. */
-	bool master;              /*!< Master flag. True if server is a primary
-	                               master for the zone. */
-	int ret;                  /*!< Return value. */
+	bool master;         /*!< True if server is a primary master for the zone. */
+	int ret;             /*!< Return value. */
 } zcreator_t;
 
 /*!
@@ -47,11 +42,10 @@ typedef struct zcreator {
  */
 typedef struct zloader {
 	char *source;                /*!< Zone source file. */
-	char *origin;                /*!< Zone's origin string. */
 	bool semantic_checks;        /*!< Do semantic checks. */
 	err_handler_t *err_handler;  /*!< Semantic checks error handler. */
-	zs_scanner_t *scanner;       /*!< Zone scanner. */
 	zcreator_t *creator;         /*!< Loader context. */
+	zs_scanner_t scanner;        /*!< Zone scanner. */
 } zloader_t;
 
 /*!
@@ -103,20 +97,4 @@ void zonefile_close(zloader_t *loader);
  */
 int zcreator_step(zcreator_t *zl, const knot_rrset_t *rr);
 
-/*!
- * \brief Scanner error processing function.
- * \param scanner  Scanner to use.
- */
-void process_error(zs_scanner_t *scanner);
-
-/*!
- * \brief Logs TTL mismatch error.
- *
- * \param zone    Related zone.
- * \param node    Node with TTL mismatch.
- * \param rr      RR that caused the mismatch.
- */
-void log_ttl_error(const zone_contents_t *zone, const zone_node_t *node,
-                   const knot_rrset_t *rr);
-
 /*! @} */
