@@ -18,26 +18,29 @@
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdlib.h>
-#include "libknot/common.h"
+
+#include "dnssec/error.h"
+#include "dnssec/kasp.h"
+#include "dnssec/key.h"
+#include "dnssec/sign.h"
+#include "libknot/attribute.h"
 #include "libknot/descriptor.h"
-#include "libknot/dnssec/key.h"
-#include "libknot/dnssec/policy.h"
 #include "libknot/dnssec/rrset-sign.h"
-#include "libknot/dnssec/sign.h"
-#include "libknot/errcode.h"
+#include "libknot/libknot.h"
+#include "libknot/packet/rrset-wire.h"
 #include "libknot/packet/wire.h"
 #include "libknot/rrset.h"
 #include "libknot/rrtype/rrsig.h"
-#include "libknot/packet/rrset-wire.h"
+#include "contrib/wire_ctx.h"
 
 #define RRSIG_RDATA_SIGNER_OFFSET 18
 
 /*- Creating of RRSIGs -------------------------------------------------------*/
 
 /*!
- * \brief Get size of RRSIG RDATA for a given key.
+ * \brief Get size of RRSIG RDATA for a given key without signature.
  */
-size_t knot_rrsig_rdata_size(const knot_dnssec_key_t *key)
+static size_t rrsig_rdata_header_size(const dnssec_key_t *key)
 {
 	if (!key) {
 		return 0;
@@ -59,47 +62,54 @@ size_t knot_rrsig_rdata_size(const knot_dnssec_key_t *key)
 
 	// variable part
 
-	assert(key->name);
-	size += knot_dname_size(key->name);
-	size += knot_dnssec_sign_size(key);
+	const uint8_t *signer = dnssec_key_get_dname(key);
+	assert(signer);
+	size += knot_dname_size(signer);
 
 	return size;
 }
 
 /*!
  * \brief Write RRSIG RDATA except signature.
+ *
+ * \note This can be also used for SIG(0) if proper parameters are supplied.
+ *
+ * \param rdata_len     Length of RDATA.
+ * \param rdata         Pointer to RDATA.
+ * \param key           Key used for signing.
+ * \param covered_type  Type of the covered RR.
+ * \param owner_labels  Number of labels covered by the signature.
+ * \param sig_incepted  Timestamp of signature inception.
+ * \param sig_expires   Timestamp of signature expiration.
  */
-int knot_rrsig_write_rdata(uint8_t *rdata, const knot_dnssec_key_t *key,
-                           uint16_t covered_type, uint8_t owner_labels,
-                           uint32_t owner_ttl,  uint32_t sig_incepted,
-                           uint32_t sig_expires)
+static int rrsig_write_rdata(uint8_t *rdata, size_t rdata_len,
+                             const dnssec_key_t *key,
+                             uint16_t covered_type, uint8_t owner_labels,
+                             uint32_t owner_ttl,  uint32_t sig_incepted,
+                             uint32_t sig_expires)
 {
 	if (!rdata || !key || sig_incepted >= sig_expires) {
 		return KNOT_EINVAL;
 	}
 
-	uint8_t *w = rdata;
+	uint8_t algorithm = dnssec_key_get_algorithm(key);
+	uint16_t keytag = dnssec_key_get_keytag(key);
+	const uint8_t *signer = dnssec_key_get_dname(key);
+	assert(signer);
 
-	knot_wire_write_u16(w, covered_type);	// type covered
-	w += sizeof(uint16_t);
-	*w = key->algorithm;			// algorithm
-	w += sizeof(uint8_t);
-	*w = owner_labels;			// labels
-	w += sizeof(uint8_t);
-	knot_wire_write_u32(w, owner_ttl);	// original TTL
-	w += sizeof(uint32_t);
-	knot_wire_write_u32(w, sig_expires);	// signature expiration
-	w += sizeof(uint32_t);
-	knot_wire_write_u32(w, sig_incepted);	// signature inception
-	w += sizeof(uint32_t);
-	knot_wire_write_u16(w, key->keytag);	// key fingerprint
-	w += sizeof(uint16_t);
+	wire_ctx_t wire = wire_ctx_init(rdata, rdata_len);
 
-	assert(w == rdata + RRSIG_RDATA_SIGNER_OFFSET);
-	assert(key->name);
-	memcpy(w, key->name, knot_dname_size(key->name)); // signer
+	wire_ctx_write_u16(&wire, covered_type);	// type covered
+	wire_ctx_write_u8(&wire, algorithm);		// algorithm
+	wire_ctx_write_u8(&wire, owner_labels);	// labels
+	wire_ctx_write_u32(&wire, owner_ttl);		// original TTL
+	wire_ctx_write_u32(&wire, sig_expires);	// signature expiration
+	wire_ctx_write_u32(&wire, sig_incepted);	// signature inception
+	wire_ctx_write_u16(&wire, keytag);		// key fingerprint
+	assert(wire_ctx_offset(&wire) == RRSIG_RDATA_SIGNER_OFFSET);
+	wire_ctx_write(&wire, signer, knot_dname_size(signer));	// signer
 
-	return KNOT_EOK;
+	return wire.error;
 }
 
 /*- Computation of signatures ------------------------------------------------*/
@@ -114,26 +124,33 @@ int knot_rrsig_write_rdata(uint8_t *rdata, const knot_dnssec_key_t *key,
  *
  * \return Error code, KNOT_EOK if successful.
  */
-static int sign_ctx_add_self(knot_dnssec_sign_context_t *ctx,
-                             const uint8_t *rdata)
+static int sign_ctx_add_self(dnssec_sign_ctx_t *ctx, const uint8_t *rdata)
 {
 	assert(ctx);
 	assert(rdata);
 
 	int result;
 
-	// add static header
-	result = knot_dnssec_sign_add(ctx, rdata, RRSIG_RDATA_SIGNER_OFFSET);
-	if (result != KNOT_EOK) {
+	// static header
+
+	dnssec_binary_t header = { 0 };
+	header.data = (uint8_t *)rdata;
+	header.size = RRSIG_RDATA_SIGNER_OFFSET;
+
+	result = dnssec_sign_add(ctx, &header);
+	if (result != DNSSEC_EOK) {
 		return result;
 	}
 
-	// add signer name
-	const uint8_t *signer_ptr = rdata + RRSIG_RDATA_SIGNER_OFFSET;
-	knot_dname_t *signer = knot_dname_copy(signer_ptr, NULL);
-	knot_dname_to_lower(signer);
-	result = knot_dnssec_sign_add(ctx, signer, knot_dname_size(signer));
-	knot_dname_free(&signer, NULL);
+	// signer name
+
+	const uint8_t *rdata_signer = rdata + RRSIG_RDATA_SIGNER_OFFSET;
+	dnssec_binary_t signer = { 0 };
+	signer.data = knot_dname_copy(rdata_signer, NULL);
+	signer.size = knot_dname_size(signer.data);
+
+	result = dnssec_sign_add(ctx, &signer);
+	free(signer.data);
 
 	return result;
 }
@@ -148,8 +165,7 @@ static int sign_ctx_add_self(knot_dnssec_sign_context_t *ctx,
  *
  * \return Error code, KNOT_EOK if successful.
  */
-static int sign_ctx_add_records(knot_dnssec_sign_context_t *ctx,
-                                const knot_rrset_t *covered)
+static int sign_ctx_add_records(dnssec_sign_ctx_t *ctx, const knot_rrset_t *covered)
 {
 	// huge block of rrsets can be optionally created
 	uint8_t *rrwf = malloc(KNOT_WIRE_MAX_PKTSIZE);
@@ -163,7 +179,10 @@ static int sign_ctx_add_records(knot_dnssec_sign_context_t *ctx,
 		return written;
 	}
 
-	int result = knot_dnssec_sign_add(ctx, rrwf, written);
+	dnssec_binary_t rrset_wire = { 0 };
+	rrset_wire.size = written;
+	rrset_wire.data = rrwf;
+	int result = dnssec_sign_add(ctx, &rrset_wire);
 	free(rrwf);
 
 	return result;
@@ -183,7 +202,7 @@ static int sign_ctx_add_records(knot_dnssec_sign_context_t *ctx,
  *
  * \return Error code, KNOT_EOK if successful.
  */
-static int sign_ctx_add_data(knot_dnssec_sign_context_t *ctx,
+static int sign_ctx_add_data(dnssec_sign_ctx_t *ctx,
                              const uint8_t *rrsig_rdata,
                              const knot_rrset_t *covered)
 {
@@ -196,10 +215,10 @@ static int sign_ctx_add_data(knot_dnssec_sign_context_t *ctx,
 }
 
 /*!
- * \brief Create RRSIG RDATA (all fields except signature are filled).
+ * \brief Create RRSIG RDATA.
  *
  * \param[in]  rrsigs        RR set with RRSIGS.
- * \param[in]  rrsigs        DNSSEC signing context.
+ * \param[in]  ctx           DNSSEC signing context.
  * \param[in]  covered       RR covered by the signature.
  * \param[in]  key           Key used for signing.
  * \param[in]  sig_incepted  Timestamp of signature inception.
@@ -207,79 +226,83 @@ static int sign_ctx_add_data(knot_dnssec_sign_context_t *ctx,
  *
  * \return Error code, KNOT_EOK if succesful.
  */
-static int rrsigs_create_rdata(knot_rrset_t *rrsigs,
-                               knot_dnssec_sign_context_t *context,
+static int rrsigs_create_rdata(knot_rrset_t *rrsigs, dnssec_sign_ctx_t *ctx,
                                const knot_rrset_t *covered,
-                               const knot_dnssec_key_t *key,
-                               uint32_t sig_incepted, uint32_t sig_expires)
+                               const dnssec_key_t *key,
+                               uint32_t sig_incepted, uint32_t sig_expires,
+                               knot_mm_t *mm)
 {
 	assert(rrsigs);
 	assert(rrsigs->type == KNOT_RRTYPE_RRSIG);
 	assert(!knot_rrset_empty(covered));
 	assert(key);
 
-	size_t size = knot_rrsig_rdata_size(key);
-	assert(size != 0);
+	size_t header_size = rrsig_rdata_header_size(key);
+	assert(header_size != 0);
 
 	uint8_t owner_labels = knot_dname_labels(covered->owner, NULL);
 	if (knot_dname_is_wildcard(covered->owner)) {
 		owner_labels -= 1;
 	}
 
-	uint8_t result[size];
+	uint8_t header[header_size];
 	const knot_rdata_t *covered_data = knot_rdataset_at(&covered->rrs, 0);
-	int res = knot_rrsig_write_rdata(result, key, covered->type, owner_labels,
-	                                 knot_rdata_ttl(covered_data),
-	                                 sig_incepted, sig_expires);
+	int res = rrsig_write_rdata(header, header_size,
+	                            key, covered->type, owner_labels,
+	                            knot_rdata_ttl(covered_data),
+	                            sig_incepted, sig_expires);
 	assert(res == KNOT_EOK);
 
-	res = knot_dnssec_sign_new(context);
+	res = dnssec_sign_init(ctx);
 	if (res != KNOT_EOK) {
 		return res;
 	}
 
-	res = sign_ctx_add_data(context, result, covered);
+	res = sign_ctx_add_data(ctx, header, covered);
 	if (res != KNOT_EOK) {
 		return res;
 	}
 
-	const size_t signature_offset = RRSIG_RDATA_SIGNER_OFFSET + knot_dname_size(key->name);
-	uint8_t *signature = result + signature_offset;
-	const size_t signature_size = size - signature_offset;
-
-	res = knot_dnssec_sign_write(context, signature, signature_size);
-	if (res != KNOT_EOK) {
+	dnssec_binary_t signature = { 0 };
+	res = dnssec_sign_write(ctx, &signature);
+	if (res != DNSSEC_EOK) {
 		return res;
 	}
+	assert(signature.size > 0);
 
-	return knot_rrset_add_rdata(rrsigs, result, size,
-	                            knot_rdata_ttl(covered_data), NULL);
+	size_t rrsig_size = header_size + signature.size;
+	uint8_t rrsig[rrsig_size];
+	memcpy(rrsig, header, header_size);
+	memcpy(rrsig + header_size, signature.data, signature.size);
+
+	dnssec_binary_free(&signature);
+
+	return knot_rrset_add_rdata(rrsigs, rrsig, rrsig_size,
+	                            knot_rdata_ttl(covered_data), mm);
 }
 
-/*!
- * \brief Create RRSIG RR for given RR set.
- */
+_public_
 int knot_sign_rrset(knot_rrset_t *rrsigs, const knot_rrset_t *covered,
-                    const knot_dnssec_key_t *key,
-                    knot_dnssec_sign_context_t *sign_ctx,
-                    const knot_dnssec_policy_t *policy)
+                    const dnssec_key_t *key, dnssec_sign_ctx_t *sign_ctx,
+                    const kdnssec_ctx_t *dnssec_ctx, knot_mm_t *mm)
 {
-	if (knot_rrset_empty(covered) || !key || !sign_ctx || !policy ||
+	if (knot_rrset_empty(covered) || !key || !sign_ctx || !dnssec_ctx ||
 	    rrsigs->type != KNOT_RRTYPE_RRSIG ||
 	    !knot_dname_is_equal(rrsigs->owner, covered->owner)
 	) {
 		return KNOT_EINVAL;
 	}
 
-	uint32_t sig_incept = policy->now;
-	uint32_t sig_expire = sig_incept + policy->sign_lifetime;
+	uint32_t sig_incept = dnssec_ctx->now;
+	uint32_t sig_expire = sig_incept + dnssec_ctx->policy->rrsig_lifetime;
 
 	return rrsigs_create_rdata(rrsigs, sign_ctx, covered, key, sig_incept,
-	                           sig_expire);
+	                           sig_expire, mm);
 }
 
+_public_
 int knot_synth_rrsig(uint16_t type, const knot_rdataset_t *rrsig_rrs,
-                     knot_rdataset_t *out_sig, mm_ctx_t *mm)
+                knot_rdataset_t *out_sig, knot_mm_t *mm)
 {
 	if (rrsig_rrs == NULL) {
 		return KNOT_ENOENT;
@@ -315,33 +338,32 @@ int knot_synth_rrsig(uint16_t type, const knot_rdataset_t *rrsig_rrs,
  * \return Signature is expired or should be replaced soon.
  */
 static bool is_expired_signature(const knot_rrset_t *rrsigs, size_t pos,
-                                 const knot_dnssec_policy_t *policy)
+                                 uint32_t now, uint32_t refresh_before)
 {
 	assert(!knot_rrset_empty(rrsigs));
 	assert(rrsigs->type == KNOT_RRTYPE_RRSIG);
-	assert(policy);
 
-	uint32_t expiration = knot_rrsig_sig_expiration(&rrsigs->rrs, pos);
+	uint32_t expire_at = knot_rrsig_sig_expiration(&rrsigs->rrs, pos);
+	uint32_t expire_in = expire_at > now ? expire_at - now : 0;
 
-	return (expiration <= policy->refresh_before);
+	return expire_in <= refresh_before;
 }
 
-/*!
- * \brief Check if RRSIG signature is valid.
- */
-int knot_is_valid_signature(const knot_rrset_t *covered,
-                            const knot_rrset_t *rrsigs, size_t pos,
-                            const knot_dnssec_key_t *key,
-                            knot_dnssec_sign_context_t *ctx,
-                            const knot_dnssec_policy_t *policy)
+_public_
+int knot_check_signature(const knot_rrset_t *covered,
+                    const knot_rrset_t *rrsigs, size_t pos,
+                    const dnssec_key_t *key,
+                    dnssec_sign_ctx_t *sign_ctx,
+                    const kdnssec_ctx_t *dnssec_ctx)
 {
-	if (knot_rrset_empty(covered) ||
-	    knot_rrset_empty(rrsigs) || !key || !ctx || !policy) {
+	if (knot_rrset_empty(covered) || knot_rrset_empty(rrsigs) || !key ||
+	    !sign_ctx || !dnssec_ctx) {
 		return KNOT_EINVAL;
 	}
 
-	if (is_expired_signature(rrsigs, pos, policy)) {
-		return KNOT_DNSSEC_EINVALID_SIGNATURE;
+	if (is_expired_signature(rrsigs, pos, dnssec_ctx->now,
+	                         dnssec_ctx->policy->rrsig_refresh_before)) {
+		return DNSSEC_INVALID_SIGNATURE;
 	}
 
 	// identify fields in the signature being validated
@@ -352,25 +374,23 @@ int knot_is_valid_signature(const knot_rrset_t *covered,
 		return KNOT_EINVAL;
 	}
 
-	uint8_t *signature = NULL;
-	size_t signature_size = 0;
-	knot_rrsig_signature(&rrsigs->rrs, pos, &signature, &signature_size);
-	if (!signature) {
+	dnssec_binary_t signature = { 0 };
+	knot_rrsig_signature(&rrsigs->rrs, pos, &signature.data, &signature.size);
+	if (!signature.data) {
 		return KNOT_EINVAL;
 	}
 
 	// perform the validation
 
-	int result = knot_dnssec_sign_new(ctx);
+	int result = dnssec_sign_init(sign_ctx);
 	if (result != KNOT_EOK) {
 		return result;
 	}
 
-	result = sign_ctx_add_data(ctx, rdata, covered);
+	result = sign_ctx_add_data(sign_ctx, rdata, covered);
 	if (result != KNOT_EOK) {
 		return result;
 	}
 
-	return knot_dnssec_sign_verify(ctx, signature, signature_size);
+	return dnssec_sign_verify(sign_ctx, &signature);
 }
-
