@@ -18,6 +18,7 @@
 #include <stdlib.h>
 #include <stdbool.h>
 
+#include "libknot/attribute.h"
 #include "libknot/packet/pkt.h"
 #include "libknot/descriptor.h"
 #include "libknot/errcode.h"
@@ -25,7 +26,9 @@
 #include "libknot/tsig-op.h"
 #include "libknot/packet/wire.h"
 #include "libknot/packet/rrset-wire.h"
-#include "libknot/internal/macros.h"
+#include "contrib/mempattern.h"
+#include "contrib/wire.h"
+#include "contrib/wire_ctx.h"
 
 /*! \brief Packet RR array growth step. */
 #define NEXT_RR_ALIGN 16
@@ -67,6 +70,10 @@ static void pkt_free_data(knot_pkt_t *pkt)
 	/* Reset special types. */
 	pkt->opt_rr = NULL;
 	pkt->tsig_rr = NULL;
+
+	/* Reset TSIG wire reference. */
+	pkt->tsig_wire.pos = NULL;
+	pkt->tsig_wire.len = 0;
 }
 
 /*! \brief Allocate new wireformat of given length. */
@@ -164,14 +171,14 @@ static int pkt_rr_array_alloc(knot_pkt_t *pkt, uint16_t count)
 }
 
 /*! \brief Clear the packet and switch wireformat pointers (possibly allocate new). */
-static int pkt_init(knot_pkt_t *pkt, void *wire, uint16_t len, mm_ctx_t *mm)
+static int pkt_init(knot_pkt_t *pkt, void *wire, uint16_t len, knot_mm_t *mm)
 {
 	assert(pkt);
 
 	memset(pkt, 0, sizeof(knot_pkt_t));
 
 	/* No data to free, set memory context. */
-	memcpy(&pkt->mm, mm, sizeof(mm_ctx_t));
+	memcpy(&pkt->mm, mm, sizeof(knot_mm_t));
 
 	/* Initialize wire. */
 	int ret = KNOT_EOK;
@@ -213,7 +220,7 @@ static void pkt_clear_payload(knot_pkt_t *pkt)
 }
 
 /*! \brief Allocate new packet using memory context. */
-static knot_pkt_t *pkt_new_mm(void *wire, uint16_t len, mm_ctx_t *mm)
+static knot_pkt_t *pkt_new_mm(void *wire, uint16_t len, knot_mm_t *mm)
 {
 	assert(mm);
 
@@ -231,10 +238,10 @@ static knot_pkt_t *pkt_new_mm(void *wire, uint16_t len, mm_ctx_t *mm)
 }
 
 _public_
-knot_pkt_t *knot_pkt_new(void *wire, uint16_t len, mm_ctx_t *mm)
+knot_pkt_t *knot_pkt_new(void *wire, uint16_t len, knot_mm_t *mm)
 {
 	/* Default memory allocator if NULL. */
-	mm_ctx_t _mm;
+	knot_mm_t _mm;
 	if (mm == NULL) {
 		mm_ctx_init(&_mm);
 		mm = &_mm;
@@ -243,6 +250,28 @@ knot_pkt_t *knot_pkt_new(void *wire, uint16_t len, mm_ctx_t *mm)
 	return pkt_new_mm(wire, len, mm);
 }
 
+static int append_tsig(knot_pkt_t *dst, const knot_pkt_t *src)
+{
+	/* Check if a wire TSIG is available. */
+	if (src->tsig_wire.pos != NULL) {
+		if (dst->max_size < src->size + src->tsig_wire.len) {
+			return KNOT_ESPACE;
+		}
+		memcpy(dst->wire + dst->size, src->tsig_wire.pos,
+		       src->tsig_wire.len);
+		dst->size += src->tsig_wire.len;
+
+		/* Increment arcount. */
+		knot_wire_set_arcount(dst->wire,
+		                      knot_wire_get_arcount(dst->wire) + 1);
+	} else {
+		return knot_tsig_append(dst->wire, &dst->size, dst->max_size,
+		                        src->tsig_rr);
+	}
+
+	return KNOT_EOK;
+}
+
 _public_
 int knot_pkt_copy(knot_pkt_t *dst, const knot_pkt_t *src)
 {
@@ -253,20 +282,18 @@ int knot_pkt_copy(knot_pkt_t *dst, const knot_pkt_t *src)
 	if (dst->max_size < src->size) {
 		return KNOT_ESPACE;
 	}
-
+	memcpy(dst->wire, src->wire, src->size);
 	dst->size = src->size;
-	memcpy(dst->wire, src->wire, dst->size);
 
-	/* Copy TSIG RR back to wire. */
+	/* Append TSIG record. */
 	if (src->tsig_rr) {
-		int ret = knot_tsig_append(dst->wire, &dst->size, dst->max_size,
-		                           src->tsig_rr);
+		int ret = append_tsig(dst, src);
 		if (ret != KNOT_EOK) {
 			return ret;
 		}
 	}
 
-	/* Invalidate arrays . */
+	/* Invalidate arrays. */
 	dst->rr = NULL;
 	dst->rr_info = NULL;
 	dst->rrset_count = 0;
@@ -483,27 +510,28 @@ int knot_pkt_put_question(knot_pkt_t *pkt, const knot_dname_t *qname, uint16_t q
 	assert(pkt->rrset_count == 0);
 
 	/* Copy name wireformat. */
-	uint8_t *dst = pkt->wire + KNOT_WIRE_HEADER_SIZE;
-	int qname_len = knot_dname_to_wire(dst, qname, pkt->max_size - pkt->size);
+	wire_ctx_t wire = wire_ctx_init(pkt->wire, pkt->max_size);
+	wire_ctx_set_offset(&wire, KNOT_WIRE_HEADER_SIZE);
+
+	int qname_len = knot_dname_to_wire(wire.position,
+	                                   qname, wire_ctx_available(&wire));
 	if (qname_len < 0) {
 		return qname_len;
 	}
-
-	/* Check size limits. */
-	size_t question_len = 2 * sizeof(uint16_t) + qname_len;
-	if (qname_len < 0 || pkt->size + question_len > pkt->max_size) {
-		return KNOT_ESPACE;
-	}
+	wire_ctx_skip(&wire, qname_len);
 
 	/* Copy QTYPE & QCLASS */
-	dst += qname_len;
-	wire_write_u16(dst, qtype);
-	dst += sizeof(uint16_t);
-	wire_write_u16(dst, qclass);
+	wire_ctx_write_u16(&wire, qtype);
+	wire_ctx_write_u16(&wire, qclass);
+
+	/* Check errors. */
+	if (wire.error != KNOT_EOK) {
+		return wire.error;
+	}
 
 	/* Update question count and sizes. */
 	knot_wire_set_qdcount(pkt->wire, 1);
-	pkt->size += question_len;
+	pkt->size = wire_ctx_offset(&wire);
 	pkt->qname_size = qname_len;
 
 	/* Start writing ANSWER. */
@@ -597,6 +625,16 @@ const knot_rrset_t *knot_pkt_rr(const knot_pktsection_t *section, uint16_t i)
 	return section->pkt->rr + section->pos + i;
 }
 
+_public_
+uint16_t knot_pkt_rr_offset(const knot_pktsection_t *section, uint16_t i)
+{
+	if (section == NULL || section->pkt == NULL) {
+		return -1;
+	}
+
+	return section->pkt->rr_info[section->pos + i].pos;
+}
+
 _public_
 int knot_pkt_parse(knot_pkt_t *pkt, unsigned flags)
 {
@@ -689,6 +727,8 @@ static int check_rr_constraints(knot_pkt_t *pkt, knot_rrset_t *rr, size_t rr_siz
 		if (!(flags & KNOT_PF_KEEPWIRE)) {
 			pkt->parsed -= rr_size;
 			pkt->size -= rr_size;
+			pkt->tsig_wire.pos = pkt->wire + pkt->parsed;
+			pkt->tsig_wire.len = rr_size;
 			knot_wire_set_id(pkt->wire, knot_tsig_rdata_orig_id(rr));
 			knot_wire_set_arcount(pkt->wire, knot_wire_get_arcount(pkt->wire) - 1);
 		}
@@ -703,7 +743,7 @@ static int check_rr_constraints(knot_pkt_t *pkt, knot_rrset_t *rr, size_t rr_siz
 	return KNOT_EOK;
 }
 
-#undef CHECK_AR_RECORD
+#undef CHECK_AR_CONSTRAINTS
 
 _public_
 int knot_pkt_parse_rr(knot_pkt_t *pkt, unsigned flags)
@@ -783,6 +823,11 @@ int knot_pkt_parse_payload(knot_pkt_t *pkt, unsigned flags)
 	size_t rr_count = knot_wire_get_ancount(pkt->wire) +
 	                  knot_wire_get_nscount(pkt->wire) +
 	                  knot_wire_get_arcount(pkt->wire);
+
+	if (rr_count > pkt->size / KNOT_WIRE_RR_MIN_SIZE) {
+		return KNOT_EMALF;
+	}
+
 	int ret = pkt_rr_array_alloc(pkt, rr_count);
 	if (ret != KNOT_EOK) {
 		return ret;
