@@ -21,6 +21,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "libknot/attribute.h"
 #include "libknot/packet/rrset-wire.h"
 #include "libknot/consts.h"
 #include "libknot/descriptor.h"
@@ -29,7 +30,9 @@
 #include "libknot/packet/wire.h"
 #include "libknot/rrset.h"
 #include "libknot/rrtype/naptr.h"
-#include "libknot/internal/macros.h"
+#include "contrib/macros.h"
+#include "contrib/wire.h"
+#include "contrib/wire_ctx.h"
 
 #define RR_HEADER_SIZE 10
 
@@ -284,7 +287,12 @@ static int write_owner(const knot_rrset_t *rrset, uint8_t **dst, size_t *dst_ava
 	assert(dst && *dst);
 	assert(dst_avail);
 
-	uint16_t owner_pointer = compr_get_ptr(compr, KNOT_COMPR_HINT_OWNER);
+	/* Check for zero label owner (don't compress). */
+
+	uint16_t owner_pointer = 0;
+	if (*rrset->owner != '\0') {
+		owner_pointer = compr_get_ptr(compr, KNOT_COMPR_HINT_OWNER);
+	}
 
 	/* Check size */
 
@@ -333,34 +341,25 @@ static int write_fixed_header(const knot_rrset_t *rrset, uint16_t rrset_index,
 	assert(dst && *dst);
 	assert(dst_avail);
 
-	/* Check capacity */
-
-	size_t size = sizeof(uint16_t)  // type
-	            + sizeof(uint16_t)  // class
-	            + sizeof(uint32_t); // ttl
-
-	if (size > *dst_avail) {
-		return KNOT_ESPACE;
-	}
-
 	/* Write result */
 
 	uint32_t ttl = knot_rdata_ttl(knot_rdataset_at(&rrset->rrs, rrset_index));
-	uint8_t *write = *dst;
 
-	wire_write_u16(write, rrset->type);
-	write += sizeof(uint16_t);
-	wire_write_u16(write, rrset->rclass);
-	write += sizeof(uint16_t);
-	wire_write_u32(write, ttl);
-	write += sizeof(uint32_t);
+	wire_ctx_t write = wire_ctx_init(*dst, *dst_avail);
 
-	assert(write == *dst + size);
+	wire_ctx_write_u16(&write, rrset->type);
+	wire_ctx_write_u16(&write, rrset->rclass);
+	wire_ctx_write_u32(&write, ttl);
+
+	/* Check write */
+	if (write.error != KNOT_EOK) {
+		return write.error;
+	}
 
 	/* Update buffer */
 
-	*dst = write;
-	*dst_avail -= size;
+	*dst = write.position;
+	*dst_avail = wire_ctx_available(&write);
 
 	return KNOT_EOK;
 }
@@ -393,7 +392,6 @@ static int compress_rdata_dname(const uint8_t **src, size_t *src_avail,
 	int written = knot_compr_put_dname(dname, *dst, dname_max(*dst_avail),
 	                                   put_compr);
 	if (written < 0) {
-		assert(written == KNOT_ESPACE);
 		return written;
 	}
 
@@ -514,9 +512,7 @@ int knot_rrset_to_wire(const knot_rrset_t *rrset, uint8_t *wire, uint16_t max_si
 		}
 	}
 
-	size_t written = write - wire;
-
-	return written;
+	return write - wire;
 }
 
 /*- RRSet from wire ---------------------------------------------------------*/
@@ -525,7 +521,7 @@ int knot_rrset_to_wire(const knot_rrset_t *rrset, uint8_t *wire, uint16_t max_si
  * \brief Parse header of one RR from packet wireformat.
  */
 static int parse_header(const uint8_t *pkt_wire, size_t *pos, size_t pkt_size,
-                        mm_ctx_t *mm, knot_rrset_t *rrset, uint32_t *ttl,
+                        knot_mm_t *mm, knot_rrset_t *rrset, uint32_t *ttl,
                         uint16_t *rdlen)
 {
 	assert(pkt_wire);
@@ -544,16 +540,22 @@ static int parse_header(const uint8_t *pkt_wire, size_t *pos, size_t pkt_size,
 		return KNOT_EMALF;
 	}
 
-	uint16_t type = wire_read_u16(pkt_wire + *pos);
-	*pos += sizeof(uint16_t);
-	uint16_t rclass = wire_read_u16(pkt_wire + *pos);
-	*pos += sizeof(uint16_t);
-	*ttl = wire_read_u32(pkt_wire + *pos);
-	*pos += sizeof(uint32_t);
-	*rdlen = wire_read_u16(pkt_wire + *pos);
-	*pos += sizeof(uint16_t);
+	wire_ctx_t wire = wire_ctx_init_const(pkt_wire, pkt_size);
+	wire_ctx_set_offset(&wire, *pos);
 
-	if (pkt_size - *pos < *rdlen) {
+	uint16_t type = wire_ctx_read_u16(&wire);
+	uint16_t rclass = wire_ctx_read_u16(&wire);
+	*ttl = wire_ctx_read_u32(&wire);
+	*rdlen = wire_ctx_read_u16(&wire);
+
+	*pos = wire_ctx_offset(&wire);
+
+	if (wire.error != KNOT_EOK) {
+		knot_dname_free(&owner, mm);
+		return wire.error;
+	}
+
+	if (wire_ctx_available(&wire) < *rdlen) {
 		knot_dname_free(&owner, mm);
 		return KNOT_EMALF;
 	}
@@ -611,7 +613,7 @@ static bool allow_zero_rdata(const knot_rrset_t *rr,
  * \brief Parse RDATA part of one RR from packet wireformat.
  */
 static int parse_rdata(const uint8_t *pkt_wire, size_t *pos, size_t pkt_size,
-                       mm_ctx_t *mm, uint32_t ttl, uint16_t rdlength,
+                       knot_mm_t *mm, uint32_t ttl, uint16_t rdlength,
                        knot_rrset_t *rrset)
 {
 	assert(pkt_wire);
@@ -644,7 +646,7 @@ static int parse_rdata(const uint8_t *pkt_wire, size_t *pos, size_t pkt_size,
 	if (buffer_size < 0) {
 		return buffer_size;
 	}
-	
+
 	if (buffer_size > MAX_RDLENGTH) {
 		/* DNAME compression caused RDATA overflow. */
 		return KNOT_EMALF;
@@ -680,7 +682,7 @@ static int parse_rdata(const uint8_t *pkt_wire, size_t *pos, size_t pkt_size,
 		knot_rdataset_unreserve(rrs, mm);
 		return ret;
 	}
-	
+
 	/* Update position pointer. */
 	*pos += rdlength;
 
@@ -689,7 +691,7 @@ static int parse_rdata(const uint8_t *pkt_wire, size_t *pos, size_t pkt_size,
 
 _public_
 int knot_rrset_rr_from_wire(const uint8_t *pkt_wire, size_t *pos,
-                            size_t pkt_size, mm_ctx_t *mm,
+                            size_t pkt_size, knot_mm_t *mm,
                             knot_rrset_t *rrset)
 {
 	if (!pkt_wire || !pos || !rrset || *pos > pkt_size) {
