@@ -32,7 +32,7 @@
 
 #include "libknot/dname.h"
 #include "libknot/rrset.h"
-#include "libknot/internal/mempattern.h"
+#include "libknot/mm_ctx.h"
 
 struct knot_compr;
 
@@ -61,6 +61,6 @@ int knot_rrset_to_wire(const knot_rrset_t *rrset, uint8_t *wire, uint16_t max_si
 * \return KNOT_E*
 */
 int knot_rrset_rr_from_wire(const uint8_t *pkt_wire, size_t *pos,
-                            size_t pkt_size, mm_ctx_t *mm, knot_rrset_t *rrset);
+                            size_t pkt_size, knot_mm_t *mm, knot_rrset_t *rrset);
 
 /*! @} */
