@@ -1,13 +1,3 @@
-/*!
- * \file rdataset.h
- *
- * \author Jan Kadlec <jan.kadlec@nic.cz>
- *
- * \brief API for manipulating RR arrays.
- *
- * \addtogroup libknot
- * @{
- */
 /*  Copyright (C) 2014 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
@@ -23,6 +13,14 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
+/*!
+ * \file
+ *
+ * \brief API for manipulating RR arrays.
+ *
+ * \addtogroup libknot
+ * @{
+ */
 
 #pragma once
 
@@ -30,8 +28,8 @@
 #include <stdint.h>
 #include <stdbool.h>
 
+#include "libknot/mm_ctx.h"
 #include "libknot/rdata.h"
-#include "libknot/internal/mempattern.h"
 
 /*!< \brief Set of RRs. */
 typedef struct knot_rdataset {
@@ -52,7 +50,7 @@ void knot_rdataset_init(knot_rdataset_t *rrs);
  * \param rrs  Structure to be cleared.
  * \param mm   Memory context used to create allocations.
  */
-void knot_rdataset_clear(knot_rdataset_t *rrs, mm_ctx_t *mm);
+void knot_rdataset_clear(knot_rdataset_t *rrs, knot_mm_t *mm);
 
 /*!
  * \brief Deep copies RRS structure. All data are duplicated.
@@ -61,7 +59,7 @@ void knot_rdataset_clear(knot_rdataset_t *rrs, mm_ctx_t *mm);
  * \param mm   Memory context.
  * \return KNOT_E*
  */
-int knot_rdataset_copy(knot_rdataset_t *dst, const knot_rdataset_t *src, mm_ctx_t *mm);
+int knot_rdataset_copy(knot_rdataset_t *dst, const knot_rdataset_t *src, knot_mm_t *mm);
 
 /* ----------------------- RRs getters/setters ------------------------------ */
 
@@ -80,6 +78,21 @@ knot_rdata_t *knot_rdataset_at(const knot_rdataset_t *rrs, size_t pos);
  */
 size_t knot_rdataset_size(const knot_rdataset_t *rrs);
 
+/*!
+ * \brief Returns the common TTL of the RR set.
+ * \note Actually returns the TTL of the first RR.
+ * \param rrs  RR array.
+ * \return TTL.
+ */
+uint32_t knot_rdataset_ttl(const knot_rdataset_t *rrs);
+
+/*!
+ * \brief Sets TTL of all the RRs in the RR set.
+ * \param rrs  RR array.
+ * \param ttl  TTL to set.
+ */
+void knot_rdataset_set_ttl(knot_rdataset_t *rrs, uint32_t ttl);
+
 /* ----------------------- RRs RR manipulation ------------------------------ */
 
 /*!
@@ -89,7 +102,7 @@ size_t knot_rdataset_size(const knot_rdataset_t *rrs);
  * \param mm   Memory context.
  * \return KNOT_E*
  */
-int knot_rdataset_add(knot_rdataset_t *rrs, const knot_rdata_t *rr, mm_ctx_t *mm);
+int knot_rdataset_add(knot_rdataset_t *rrs, const knot_rdata_t *rr, knot_mm_t *mm);
 
 /*!
  * \brief Reserves space at the end of the RRS structure.
@@ -98,7 +111,7 @@ int knot_rdataset_add(knot_rdataset_t *rrs, const knot_rdata_t *rr, mm_ctx_t *mm
  * \param mm    Memory context.
  * \return KNOT_E*
  */
-int knot_rdataset_reserve(knot_rdataset_t *rrs, size_t size, mm_ctx_t *mm);
+int knot_rdataset_reserve(knot_rdataset_t *rrs, size_t size, knot_mm_t *mm);
 
 /*!
  * \brief Removes the last RR from RRS structure, i.e. does the opposite of _reserve.
@@ -106,7 +119,7 @@ int knot_rdataset_reserve(knot_rdataset_t *rrs, size_t size, mm_ctx_t *mm);
  * \param mm   Memory context.
  * \return KNOT_E*
  */
-int knot_rdataset_unreserve(knot_rdataset_t *rrs, mm_ctx_t *mm);
+int knot_rdataset_unreserve(knot_rdataset_t *rrs, knot_mm_t *mm);
 
 /* ---------------------- RRs set-like operations --------------------------- */
 
@@ -139,7 +152,7 @@ bool knot_rdataset_member(const knot_rdataset_t *rrs, const knot_rdata_t *rr,
  * \return KNOT_E*
  */
 int knot_rdataset_merge(knot_rdataset_t *rrs1, const knot_rdataset_t *rrs2,
-			mm_ctx_t *mm);
+			knot_mm_t *mm);
 
 /*!
  * \brief RRS set-like intersection. Full compare is done.
@@ -150,7 +163,7 @@ int knot_rdataset_merge(knot_rdataset_t *rrs1, const knot_rdataset_t *rrs2,
  * \return KNOT_E*
  */
 int knot_rdataset_intersect(const knot_rdataset_t *a, const knot_rdataset_t *b,
-                            knot_rdataset_t *out, mm_ctx_t *mm);
+                            knot_rdataset_t *out, knot_mm_t *mm);
 
 /*!
  * \brief Does set-like RRS subtraction. \a from RRS is changed. Both sets must
@@ -161,7 +174,7 @@ int knot_rdataset_intersect(const knot_rdataset_t *a, const knot_rdataset_t *b,
  * \return KNOT_E*
  */
 int knot_rdataset_subtract(knot_rdataset_t *from, const knot_rdataset_t *what,
-                           mm_ctx_t *mm);
+                           knot_mm_t *mm);
 
 /*!
  * \brief Sorts the dataset. Removes the element if found to be duplicate.
@@ -170,7 +183,7 @@ int knot_rdataset_subtract(knot_rdataset_t *from, const knot_rdataset_t *what,
  * \param mm    Memory context used to remove the element if duplicate.
  * \return KNOT_E*
  */
-int knot_rdataset_sort_at(knot_rdataset_t *rrs, size_t pos, mm_ctx_t *mm);
+int knot_rdataset_sort_at(knot_rdataset_t *rrs, size_t pos, knot_mm_t *mm);
 
 /*! \brief Accession helpers. */
 #define KNOT_RDATASET_CHECK(rrs, pos, code) \
