@@ -1,14 +1,3 @@
-/*!
- * \file nsec3.h
- *
- * \author Lubos Slovak <lubos.slovak@nic.cz>
- * \author Jan Vcelak <jan.vcelak@nic.cz>
- *
- * \brief Functions for computation of NSEC3 hashes.
- *
- * \addtogroup libknot
- * @{
- */
 /*  Copyright (C) 2011 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
@@ -24,6 +13,14 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
+/*!
+ * \file
+ *
+ * \brief Functions for computation of NSEC3 hashes.
+ *
+ * \addtogroup libknot
+ * @{
+ */
 
 #pragma once
 
@@ -31,7 +28,7 @@
 #include <stdint.h>
 #include <string.h>
 
-#include "libknot/consts.h"
+#include "libknot/codes.h"
 #include "libknot/rdataset.h"
 #include "libknot/rrtype/nsec3param.h"
 
@@ -49,12 +46,7 @@ uint8_t knot_nsec3_flags(const knot_rdataset_t *rrs, size_t pos)
 	return *knot_rdata_offset(rrs, pos, 1);
 }
 
-static inline
-uint16_t knot_nsec3_iterations(const knot_rdataset_t *rrs, size_t pos)
-{
-	KNOT_RDATASET_CHECK(rrs, pos, return 0);
-	return wire_read_u16(knot_rdata_offset(rrs, pos, 2));
-}
+uint16_t knot_nsec3_iterations(const knot_rdataset_t *rrs, size_t pos);
 
 static inline
 uint8_t knot_nsec3_salt_length(const knot_rdataset_t *rrs, size_t pos)
