@@ -1,13 +1,3 @@
-/*!
- * \file opt.h
- *
- * \author Lubos Slovak <lubos.slovak@nic.cz>
- *
- * \brief Functions for manipulating the EDNS OPT pseudo-RR.
- *
- * \addtogroup libknot
- * @{
- */
 /*  Copyright (C) 2014 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
@@ -23,16 +13,19 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
+/*!
+ * \file
+ *
+ * \brief Functions for manipulating the EDNS OPT pseudo-RR.
+ *
+ * \addtogroup libknot
+ * @{
+ */
 
 #pragma once
 
-#include <stdint.h>
-
 #include "libknot/rrset.h"
 
-/* Forward declaration. */
-struct knot_packet;
-
 /*! \brief Constants related to EDNS. */
 enum knot_edns_const {
 	/*! \brief Supported EDNS version. */
@@ -57,23 +50,13 @@ enum knot_edns_const {
 	/*! \brief NSID option code. */
 	KNOT_EDNS_OPTION_NSID          = 3,
 	/*! \brief EDNS client subnet option code. */
-	KNOT_EDNS_OPTION_CLIENT_SUBNET = 8,
-	/*! \brief Extended RCODE BADVERS. */
-	KNOT_EDNS_RCODE_BADVERS = 16,
+	KNOT_EDNS_OPTION_CLIENT_SUBNET = 8
 };
 
 /* Helpers for splitting extended RCODE. */
 #define KNOT_EDNS_RCODE_HI(rc) ((rc >> 4) & 0x00ff)
 #define KNOT_EDNS_RCODE_LO(rc) (rc & 0x000f)
 
-/*!
- * \brief EDNS DO flag.
- *
- * \note Use only with unsigned 2-byte variables.
- * \warning Flags are represented in machine byte order.
- */
-static const uint16_t KNOT_EDNS_FLAG_DO = (uint16_t)1 << 15;
-
 /*----------------------------------------------------------------------------*/
 /* EDNS OPT RR handling functions.                                            */
 /*----------------------------------------------------------------------------*/
@@ -89,7 +72,7 @@ static const uint16_t KNOT_EDNS_FLAG_DO = (uint16_t)1 << 15;
  * \return KNOT_EOK or an error
  */
 int knot_edns_init(knot_rrset_t *opt_rr, uint16_t max_pld,
-                  uint8_t ext_rcode, uint8_t ver, mm_ctx_t *mm);
+                  uint8_t ext_rcode, uint8_t ver, knot_mm_t *mm);
 
 /*!
  * \brief Returns size of the OPT RR in wire format.
@@ -246,7 +229,7 @@ void knot_edns_set_do(knot_rrset_t *opt_rr);
  * \retval KNOT_ENOMEM
  */
 int knot_edns_add_option(knot_rrset_t *opt_rr, uint16_t code,
-                         uint16_t length, const uint8_t *data, mm_ctx_t *mm);
+                         uint16_t length, const uint8_t *data, knot_mm_t *mm);
 
 /*!
  * \brief Checks if the OPT RR contains Option with the specified code.
