@@ -24,12 +24,13 @@
 #include "utils/common/netio.h"
 #include "utils/common/params.h"
 #include "libknot/libknot.h"
-#include "libknot/internal/lists.h"
-#include "libknot/internal/print.h"
-#include "libknot/internal/sockaddr.h"
-#include "libknot/internal/strlcat.h"
+#include "contrib/print.h"
+#include "contrib/sockaddr.h"
+#include "contrib/openbsd/strlcat.h"
+#include "contrib/ucw/lists.h"
+#include "contrib/wire_ctx.h"
 
-static lookup_table_t rtypes[] = {
+static knot_lookup_t rtypes[] = {
 	{ KNOT_RRTYPE_A,      "has IPv4 address" },
 	{ KNOT_RRTYPE_NS,     "nameserver is" },
 	{ KNOT_RRTYPE_CNAME,  "is an alias for" },
@@ -50,21 +51,21 @@ static lookup_table_t rtypes[] = {
 static void print_header(const knot_pkt_t *packet, const style_t *style,
                          const uint16_t ext_rcode)
 {
-	char    flags[64] = "";
+	char flags[64] = "";
 	uint8_t opcode_id;
 	const char *rcode_str = "Unknown";
 	const char *opcode_str = "Unknown";
-	lookup_table_t *rcode, *opcode;
+	const knot_lookup_t *rcode, *opcode;
 
 	// Get RCODE from Header and check for Extended RCODE from OPT RR.
-	rcode = lookup_by_id(knot_rcode_names, ext_rcode);
+	rcode = knot_lookup_by_id(knot_rcode_names, ext_rcode);
 	if (rcode != NULL) {
 		rcode_str = rcode->name;
 	}
 
 	// Get OPCODE.
 	opcode_id = knot_wire_get_opcode(packet->wire);
-	opcode = lookup_by_id(knot_opcode_names, opcode_id);
+	opcode = knot_lookup_by_id(knot_opcode_names, opcode_id);
 	if (opcode != NULL) {
 		opcode_str = opcode->name;
 	}
@@ -135,7 +136,7 @@ static void print_footer(const size_t total_len,
                          const bool   incoming)
 {
 	struct tm tm;
-	char      date[64];
+	char date[64];
 
 	// Get current timestamp.
 	if (exec_time == 0) {
@@ -183,10 +184,10 @@ static void print_edns_client_subnet(const uint8_t *data, const uint16_t len)
 	struct in_addr addr4;
 	struct in6_addr addr6;
 	knot_addr_family_t family;
-	uint8_t  src_mask, dst_mask;
-	uint8_t  addr[IPV6_PREFIXLEN / 8] = { 0 };
+	uint8_t src_mask, dst_mask;
+	uint8_t addr[IPV6_PREFIXLEN / 8] = { 0 };
 	uint16_t addr_len = sizeof(addr);
-	char     addr_str[SOCKADDR_STRLEN] = { '\0' };
+	char addr_str[SOCKADDR_STRLEN] = { '\0' };
 
 	int ret = knot_edns_client_subnet_parse(data, len, &family, addr,
 	                                        &addr_len, &src_mask, &dst_mask);
@@ -214,13 +215,13 @@ static void print_edns_client_subnet(const uint8_t *data, const uint16_t len)
 
 static void print_section_opt(const knot_rrset_t *rr, const uint8_t rcode)
 {
-	uint8_t        ercode = knot_edns_get_ext_rcode(rr);
-	uint16_t       ext_rcode_id = knot_edns_whole_rcode(ercode, rcode);
-	const char     *ext_rcode_str = "Unused";
-	lookup_table_t *ext_rcode;
+	uint8_t ercode = knot_edns_get_ext_rcode(rr);
+	uint16_t ext_rcode_id = knot_edns_whole_rcode(ercode, rcode);
+	const char *ext_rcode_str = "Unused";
+	const knot_lookup_t *ext_rcode;
 
 	if (ercode > 0) {
-		ext_rcode = lookup_by_id(knot_rcode_names, ext_rcode_id);
+		ext_rcode = knot_lookup_by_id(knot_rcode_names, ext_rcode_id);
 		if (ext_rcode != NULL) {
 			ext_rcode_str = ext_rcode->name;
 		} else {
@@ -235,16 +236,18 @@ static void print_section_opt(const knot_rrset_t *rr, const uint8_t rcode)
 	       ext_rcode_str);
 
 	knot_rdata_t *rdata = knot_rdataset_at(&rr->rrs, 0);
-	assert(rdata != NULL);
+	wire_ctx_t wire = wire_ctx_init_const(knot_rdata_data(rdata),
+	                                      knot_rdata_rdlen(rdata));
 
-	uint16_t data_len = knot_rdata_rdlen(rdata);
-	uint8_t *data = knot_rdata_data(rdata);
-	int pos = 0;
+	while (wire_ctx_available(&wire) >= KNOT_EDNS_OPTION_HDRLEN) {
+		uint16_t opt_code = wire_ctx_read_u16(&wire);
+		uint16_t opt_len = wire_ctx_read_u16(&wire);
+		uint8_t *opt_data = wire.position;
 
-	while (pos < data_len - KNOT_EDNS_OPTION_HDRLEN) {
-		uint16_t opt_code = wire_read_u16(data + pos);
-		uint16_t opt_len = wire_read_u16(data + pos + 2);
-		uint8_t *opt_data = data + pos + 4;
+		if (wire.error != KNOT_EOK) {
+			WARN("invalid OPT record data\n");
+			return;
+		}
 
 		switch (opt_code) {
 		case KNOT_EDNS_OPTION_NSID:
@@ -264,7 +267,11 @@ static void print_section_opt(const knot_rrset_t *rr, const uint8_t rcode)
 			short_hex_print(opt_data, opt_len);
 		}
 
-		pos += 4 + opt_len;
+		wire_ctx_skip(&wire, opt_len);
+	}
+
+	if (wire_ctx_available(&wire) > 0) {
+		WARN("invalid OPT record data\n");
 	}
 }
 
@@ -274,7 +281,7 @@ static void print_section_question(const knot_dname_t *owner,
                                    const style_t      *style)
 {
 	size_t buflen = 8192;
-	char   *buf = calloc(buflen, 1);
+	char *buf = calloc(buflen, 1);
 
 	// Don't print zero TTL.
 	knot_dump_style_t qstyle = style->style;
@@ -298,7 +305,7 @@ static void print_section_full(const knot_rrset_t *rrsets,
                                const bool         no_tsig)
 {
 	size_t buflen = 8192;
-	char   *buf = calloc(buflen, 1);
+	char *buf = calloc(buflen, 1);
 
 	for (size_t i = 0; i < count; i++) {
 		// Ignore OPT records.
@@ -338,7 +345,7 @@ static void print_section_dig(const knot_rrset_t *rrsets,
                               const style_t      *style)
 {
 	size_t buflen = 8192;
-	char   *buf = calloc(buflen, 1);
+	char *buf = calloc(buflen, 1);
 
 	for (size_t i = 0; i < count; i++) {
 		const knot_rrset_t *rrset = &rrsets[i];
@@ -372,19 +379,19 @@ static void print_section_host(const knot_rrset_t *rrsets,
                                const style_t      *style)
 {
 	size_t buflen = 8192;
-	char   *buf = calloc(buflen, 1);
+	char *buf = calloc(buflen, 1);
 
 	for (size_t i = 0; i < count; i++) {
 		const knot_rrset_t *rrset = &rrsets[i];
-		lookup_table_t     *descr;
-		char               type[32] = "NULL";
-		char               *owner;
+		const knot_lookup_t *descr;
+		char type[32] = "NULL";
+		char *owner;
 
 		owner = knot_dname_to_str_alloc(rrset->owner);
 		if (style->style.ascii_to_idn != NULL) {
 			style->style.ascii_to_idn(&owner);
 		}
-		descr = lookup_by_id(rtypes, rrset->type);
+		descr = knot_lookup_by_id(rtypes, rrset->type);
 
 		uint16_t rrset_rdata_count = rrset->rrs.rr_count;
 		for (uint16_t j = 0; j < rrset_rdata_count; j++) {
@@ -433,15 +440,14 @@ static void print_error_host(const uint16_t   code,
 	const char *rcode_str = "Unknown";
 	char type[32] = "Unknown";
 	char *owner;
-
-	lookup_table_t *rcode;
+	const knot_lookup_t *rcode;
 
 	owner = knot_dname_to_str_alloc(knot_pkt_qname(packet));
 	if (style->style.ascii_to_idn != NULL) {
 		style->style.ascii_to_idn(&owner);
 	}
 
-	rcode = lookup_by_id(knot_rcode_names, code);
+	rcode = knot_lookup_by_id(knot_rcode_names, code);
 	if (rcode != NULL) {
 		rcode_str = rcode->name;
 	}
