@@ -26,19 +26,11 @@
 
 #pragma once
 
-#include "utils/common/netio.h"		// net_t
-#include "utils/common/params.h"	// style_t
-#include "libknot/libknot.h"
-#include "libknot/dnssec/key.h"
-#include "libknot/dnssec/sign.h"
+#include <time.h>
 
-/*! \brief Holds data required between signing and signature verification. */
-typedef struct {
-	knot_tsig_key_t   tsig_key;
-	knot_dnssec_key_t dnssec_key;
-	uint8_t           *digest;
-	size_t            digest_size;
-} sign_context_t;
+#include "utils/common/netio.h"
+#include "utils/common/params.h"
+#include "libknot/libknot.h"
 
 /*!
  * \brief Allocates empty packet and sets packet size and random id.
@@ -74,7 +66,7 @@ void print_data_xfr(const knot_pkt_t *packet, const style_t *style);
  * \param rr_count	Total number of answer records.
  * \param net		Connection information.
  * \param elapsed	Total elapsed time.
- * \param exec_time     Time of the packet creation.
+ * \param exec_time	Time of the packet creation.
  * \param style		Style of the otput.
  */
 void print_footer_xfr(const size_t  total_len,
@@ -92,7 +84,7 @@ void print_footer_xfr(const size_t  total_len,
  * \param net		Connection information.
  * \param size		Original packet wire size.
  * \param elapsed	Total elapsed time.
- * \param exec_time     Time of the packet creation.
+ * \param exec_time	Time of the packet creation.
  * \param incoming	Indicates if the packet is input.
  * \param style		Style of the otput.
  */
@@ -104,39 +96,4 @@ void print_packet(const knot_pkt_t *packet,
                   const bool       incoming,
                   const style_t    *style);
 
-/*!
- * \brief Cleans up sign context.
- *
- * \param ctx	Sign context.
- */
-void free_sign_context(sign_context_t *ctx);
-
-/*!
- * \brief Signs outgoing DNS packet.
- *
- * \param pkt		Packet to sign.
- * \param sign_ctx	Sign context.
- * \param key_params	Key parameters.
- *
- * \retval KNOT_EOK	if success.
- * \retval error code	if error.
- */
-int sign_packet(knot_pkt_t              *pkt,
-                sign_context_t          *sign_ctx,
-                const knot_key_params_t *key_params);
-
-/*!
- * \brief Verifies signature for incoming DNS packet.
- *
- * \param pkt		Packet verify sign.
- * \param sign_ctx	Sign context.
- * \param key_params	Key parameters.
- *
- * \retval KNOT_EOK	if success.
- * \retval error code	if error.
- */
-int verify_packet(const knot_pkt_t        *pkt,
-                  const sign_context_t    *sign_ctx,
-                  const knot_key_params_t *key_params);
-
 /*! @} */
