@@ -27,7 +27,7 @@
 #pragma once
 
 #include "utils/common/netio.h"
-#include "libknot/internal/lists.h"
+#include "contrib/ucw/lists.h"
 
 srv_info_t* parse_nameserver(const char *str, const char *def_port);
 
