@@ -14,16 +14,14 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "utils/common/token.h"
-
+#include <ctype.h>
 #include <stdio.h>
-#include <stdlib.h>			// free
-#include <ctype.h>			// isspace
-#include <string.h>			// memcmp
+#include <stdlib.h>
+#include <string.h>
 
-#include "libknot/errcode.h"		// KNOT_EOK
-#include "common/getline.h"		// knot_getline
-#include "utils/common/msg.h"		// ERR
+#include "utils/common/token.h"
+#include "utils/common/msg.h"
+#include "libknot/libknot.h"
 
 int tok_scan(const char* lp, const char **tbl, int *lpm)
 {
@@ -111,25 +109,3 @@ const char* tok_skipspace(const char *lp)
 
 	while (isspace((unsigned char)(*lp))) ++lp; return lp;
 }
-
-int tok_process_lines(FILE *fp, lparse_f cb, void *arg)
-{
-	if (fp == NULL || cb == NULL) {
-		DBG_NULL;
-		return KNOT_EINVAL;
-	}
-
-	int ret = KNOT_EOK;
-
-	/* Parse lines. */
-	char *buf = NULL;
-	size_t buflen = 0;
-	ssize_t rb = 0;
-	while ((rb = knot_getline(&buf, &buflen, fp)) != -1) {
-		ret = cb(buf, rb, arg);
-		if (ret != KNOT_EOK) break;
-	}
-
-	free(buf);
-	return ret;
-}
