@@ -32,15 +32,13 @@
 #include "utils/common/exec.h"
 #include "utils/common/sign.h"
 #include "libknot/libknot.h"
-#include "libknot/internal/sockaddr.h"
+#include "contrib/sockaddr.h"
 
 #if USE_DNSTAP
-# include "dnstap/reader.h"
-# include "dnstap/writer.h"
+# include "contrib/dnstap/reader.h"
+# include "contrib/dnstap/writer.h"
 #endif // USE_DNSTAP
 
-#define KDIG_VERSION "kdig, version " PACKAGE_VERSION "\n"
-
 /*! \brief Operation mode of kdig. */
 typedef enum {
 	/*!< Standard 1-message query/reply. */
@@ -114,8 +112,6 @@ struct query {
 	int32_t		wait;
 	/*!< Ignore truncated response. */
 	bool		ignore_tc;
-	/*!< Stop querying if servfail. */
-	bool		servfail_stop;
 	/*!< Class number (16unsigned + -1 uninitialized). */
 	int32_t		class_num;
 	/*!< Type number (16unsigned + -1 uninitialized). */
