@@ -24,11 +24,8 @@
  * @{
  */
 
-#ifndef _ZSCANNER__TESTS_H_
-#define _ZSCANNER__TESTS_H_
+#pragma once
 
 int test__date_to_timestamp();
 
-#endif // _ZSCANNER__TESTS_H_
-
 /*! @} */
