@@ -45,6 +45,7 @@ static void help(void)
 	       "     0        Empty output.\n"
 	       "     1        Debug output (DEFAULT).\n"
 	       "     2        Test output.\n"
+	       " -s           State parsing mode.\n"
 	       " -t           Launch unit tests.\n"
 	       " -h           Print this help.\n");
 }
@@ -69,26 +70,91 @@ static int time_test()
 	return EXIT_SUCCESS;
 }
 
+int state_parsing(
+	zs_scanner_t *s)
+{
+	zs_scanner_t *ss;
+
+	while (zs_parse_record(s) == 0) {
+		switch (s->state) {
+		case ZS_STATE_DATA:
+			if (s->process.record != NULL) {
+				s->process.record(s);
+			}
+			break;
+		case ZS_STATE_ERROR:
+			if (s->process.error != NULL) {
+				s->process.error(s);
+			}
+			if (s->error.fatal) {
+				return -1;
+			}
+			break;
+		case ZS_STATE_INCLUDE:
+			ss = malloc(sizeof(zs_scanner_t));
+			if (ss == NULL) {
+				return -1;
+			}
+			if (zs_init(ss, (char *)s->buffer, s->default_class, s->default_ttl) != 0 ||
+			    zs_set_input_file(ss, (char *)(s->include_filename)) != 0 ||
+			    zs_set_processing(ss, s->process.record, s->process.error, s->process.data) != 0 ||
+			    state_parsing(ss) != 0 ||
+			    ss->error.counter > 0) {
+				if (ss->error.counter > 0) {
+					s->error.code = ZS_UNPROCESSED_INCLUDE;
+				} else {
+					s->error.code = ss->error.code;
+				}
+
+				s->error.counter++;
+				s->error.fatal = true;
+				if (s->process.error != NULL) {
+					s->process.error(s);
+				}
+
+				zs_deinit(ss);
+				free(ss);
+				return -1;
+			}
+			zs_deinit(ss);
+			free(ss);
+			break;
+		case ZS_STATE_NONE:
+			break;
+		case ZS_STATE_EOF:
+			// Set the next input string block.
+			break;
+		case ZS_STATE_STOP:
+			return 0;
+		}
+	}
+
+	return 0;
+}
+
 int main(int argc, char *argv[])
 {
-	// Parsed command line arguments.
-	int c = 0, li = 0;
-	int mode = DEFAULT_MODE, test = 0;
+	int mode = DEFAULT_MODE, state = 0, test = 0;
 
 	// Command line long options.
 	struct option opts[] = {
-		{ "mode",	required_argument,	0,	'm' },
-		{ "test",	no_argument,		0,	't' },
-		{ "help",	no_argument,		0,	'h' },
-		{ 0, 		0, 			0,	0 }
+		{ "mode",  required_argument, NULL, 'm' },
+		{ "state", no_argument,       NULL, 's' },
+		{ "test",  no_argument,       NULL, 't' },
+		{ "help",  no_argument,       NULL, 'h' },
+		{ NULL }
 	};
 
-	// Command line options processing.
-	while ((c = getopt_long(argc, argv, "m:th", opts, &li)) != -1) {
-		switch (c) {
+	// Parsed command line arguments.
+	int opt = 0, li = 0;
+	while ((opt = getopt_long(argc, argv, "m:sth", opts, &li)) != -1) {
+		switch (opt) {
 		case 'm':
 			mode = atoi(optarg);
 			break;
+		case 's':
+			state = 1;
+			break;
 		case 't':
 			test = 1;
 			break;
@@ -111,65 +177,70 @@ int main(int argc, char *argv[])
 		return EXIT_FAILURE;
 	}
 
-	const char   *origin = argv[optind];
-	const char   *zone_file = argv[optind + 1];
-	zs_scanner_t *s;
+	const char *origin = argv[optind];
+	const char *zone_file = argv[optind + 1];
 
-	// Create appropriate zone scanner.
+	// Create a zone scanner.
+	zs_scanner_t *s = malloc(sizeof(zs_scanner_t));
+	if (s == NULL) {
+		printf("Scanner create error!\n");
+		return EXIT_FAILURE;
+	}
+	if (zs_init(s, origin, DEFAULT_CLASS, DEFAULT_TTL) != 0) {
+		printf("Scanner init error!\n");
+		free(s);
+		return EXIT_FAILURE;
+	}
+	if (zs_set_input_file(s, zone_file) != 0) {
+		printf("Scanner file error!\n");
+		zs_deinit(s);
+		free(s);
+		return EXIT_FAILURE;
+	}
+
+	// Set the processing mode.
+	int ret;
 	switch (mode) {
 	case 0:
-		s = zs_scanner_create(origin,
-		                      DEFAULT_CLASS,
-		                      DEFAULT_TTL,
-		                      NULL,
-		                      NULL,
-		                      NULL);
+		ret = 0;
 		break;
 	case 1:
-		s = zs_scanner_create(origin,
-		                      DEFAULT_CLASS,
-		                      DEFAULT_TTL,
-		                      &debug_process_record,
-		                      &debug_process_error,
-		                      NULL);
+		ret = zs_set_processing(s, debug_process_record, debug_process_error, NULL);
 		break;
 	case 2:
-		s = zs_scanner_create(origin,
-		                      DEFAULT_CLASS,
-		                      DEFAULT_TTL,
-		                      &test_process_record,
-		                      &test_process_error,
-		                      NULL);
+		ret = zs_set_processing(s, test_process_record, test_process_error, NULL);
 		break;
 	default:
 		printf("Bad mode number!\n");
 		help();
 		return EXIT_FAILURE;
 	}
-
-	// Check parser creation.
-	if (s == NULL) {
-		printf("Scanner create error!\n");
+	if (ret != 0) {
+		printf("Processing setup error!\n");
 		return EXIT_FAILURE;
 	}
 
 	// Parse the file.
-	int ret = zs_scanner_parse_file(s, zone_file);
+	ret = state ? state_parsing(s) : zs_parse_all(s);
 	if (ret == 0) {
 		if (mode == DEFAULT_MODE) {
 			printf("Zone file has been processed successfully\n");
 		}
-		zs_scanner_free(s);
+
+		zs_deinit(s);
+		free(s);
 		return EXIT_SUCCESS;
 	} else {
-		if (s->error_counter > 0 && mode == DEFAULT_MODE) {
+		if (s->error.counter > 0 && mode == DEFAULT_MODE) {
 			printf("Zone processing has stopped with "
 			       "%"PRIu64" warnings/errors!\n",
-			       s->error_counter);
+			       s->error.counter);
 		} else if (mode == DEFAULT_MODE) {
-			printf("%s\n", zs_strerror(s->error_code));
+			printf("%s\n", zs_strerror(s->error.code));
 		}
-		zs_scanner_free(s);
+
+		zs_deinit(s);
+		free(s);
 		return EXIT_FAILURE;
 	}
 }
