@@ -25,10 +25,10 @@
 
 int main(int argc, char *argv[])
 {
-	plan(122);
+	plan(125);
 
-	const    rdata_descriptor_t *descr;
-	char     name[BUF_LEN];
+	const    knot_rdata_descriptor_t *descr;
+	char     name[BUF_LEN] = { 0 };
 	int      ret;
 	uint16_t num;
 
@@ -105,7 +105,6 @@ int main(int argc, char *argv[])
 	ok(ret != -1, "get TYPE256 ret");
 	ok(strcmp(name, "TYPE256") == 0, "get TYPE256 name");
 
-
 	// Class num to string:
 	// 7. CLASS0
 	ret = knot_rrclass_to_string(0, name, BUF_LEN);
@@ -277,15 +276,21 @@ int main(int argc, char *argv[])
 	ok(knot_rrtype_is_metatype(KNOT_RRTYPE_ANY) != 0,
 	   "rrtype is ANY");
 
-	// knot_rrtype_is_ddns_forbidden
-	ok(knot_rrtype_is_ddns_forbidden(0) == 0,
-	   "rrtype is not ddns forbidden");
-	ok(knot_rrtype_is_ddns_forbidden(KNOT_RRTYPE_RRSIG) != 0,
+	// knot_rrtype_is_dnssec
+	ok(knot_rrtype_is_dnssec(0) == 0,
+	   "rrtype is not DNSSEC");
+	ok(knot_rrtype_is_dnssec(KNOT_RRTYPE_DNSKEY) != 0,
+	   "rrtype is DNSKEY");
+	ok(knot_rrtype_is_dnssec(KNOT_RRTYPE_RRSIG) != 0,
 	   "rrtype is RRSIG");
-	ok(knot_rrtype_is_ddns_forbidden(KNOT_RRTYPE_NSEC) != 0,
+	ok(knot_rrtype_is_dnssec(KNOT_RRTYPE_NSEC) != 0,
 	   "rrtype is NSEC");
-	ok(knot_rrtype_is_ddns_forbidden(KNOT_RRTYPE_NSEC3) != 0,
+	ok(knot_rrtype_is_dnssec(KNOT_RRTYPE_NSEC3) != 0,
 	   "rrtype is NSEC3");
+	ok(knot_rrtype_is_dnssec(KNOT_RRTYPE_NSEC3PARAM) != 0,
+	   "rrtype is NSEC3PARAM");
+	ok(knot_rrtype_is_dnssec(KNOT_RRTYPE_CDNSKEY) != 0,
+	   "rrtype is CDNSKEY");
 
 	// knot_rrtype_additional_needed
 	ok(knot_rrtype_additional_needed(0) == 0,
