@@ -20,14 +20,17 @@
 #include <stdint.h>
 #include <string.h>
 #include <pthread.h>
+#include <poll.h>
 
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netdb.h>
 
-#include "libknot/internal/net.h"
+#include "libknot/errcode.h"
+#include "contrib/net.h"
+#include "contrib/sockaddr.h"
 
-const struct timeval TIMEOUT = { .tv_sec = 1 };
+const int TIMEOUT = 2000;
 
 static struct sockaddr_storage localhost(void)
 {
@@ -43,8 +46,7 @@ static struct sockaddr_storage localhost(void)
 }
 
 struct data {
-	int fd;
-	struct timeval timeout;
+	int server_fd;
 	uint8_t *buffer;
 	size_t size;
 	int result;
@@ -53,7 +55,24 @@ struct data {
 static void *thr_receive(void *data)
 {
 	struct data *d = data;
-	d->result = tcp_recv_msg(d->fd, d->buffer, d->size, &d->timeout);
+
+	struct pollfd pfd = { .fd = d->server_fd, .events = POLLIN };
+	int r = poll(&pfd, 1, TIMEOUT);
+	if (r != 1) {
+		d->result = KNOT_ETIMEOUT;
+		return NULL;
+	}
+
+	int client = accept(d->server_fd, NULL, NULL);
+	if (client < 0) {
+		d->result = KNOT_ECONN;
+		return NULL;
+	}
+
+	d->result = net_dns_tcp_recv(client, d->buffer, d->size, TIMEOUT);
+
+	close(client);
+
 	return NULL;
 }
 
@@ -64,6 +83,7 @@ int main(int argc, char *argv[])
 	int r;
 
 	// create TCP server
+
 	struct sockaddr_storage addr = localhost();
 	int server = net_bound_socket(SOCK_STREAM, &addr, 0);
 	ok(server >= 0, "server: bind socket");
@@ -76,31 +96,21 @@ int main(int argc, char *argv[])
 	r = getsockname(server, sa, &salen);
 	ok(r == 0, "server: get bound address");
 
-	r = fcntl(server, F_SETFL, O_NONBLOCK);
-	ok(r == 0, "server: set non-blocking mode");
-
 	// create TCP client
 
-	int client = net_connected_socket(SOCK_STREAM, &addr, NULL, 0);
+	int client = net_connected_socket(SOCK_STREAM, &addr, NULL);
 	ok(client >= 0, "client: connect to server");
 
-	r = fcntl(client, F_SETFL, O_NONBLOCK);
-	ok(r == 0, "client: set non-blocking mode");
-
 	int optval = 8192;
 	socklen_t optlen = sizeof(optval);
 	r = setsockopt(client, SOL_SOCKET, SO_SNDBUF, &optval, optlen);
 	ok(r == 0, "client: configure small send buffer");
 
-	// accept TCP connection
-
-	int accepted = accept(server, NULL, NULL);
-	ok(accepted >= 0, "server: accepted connection");
+	// accept TCP connection on the background
 
 	uint8_t recvbuf[UINT16_MAX] = { 0 };
 	struct data recv_data = {
-		.fd = accepted,
-		.timeout = TIMEOUT,
+		.server_fd = server,
 		.buffer = recvbuf,
 		.size = sizeof(recvbuf)
 	};
@@ -115,9 +125,8 @@ int main(int argc, char *argv[])
 	for (size_t i = 0; i < sizeof(sndbuf); i++) {
 		sndbuf[i] = i;
 	}
-	struct timeval timeout = TIMEOUT;
-	r = tcp_send_msg(client, sndbuf, sizeof(sndbuf), &timeout);
-	ok(r == sizeof(sndbuf), "client: tcp_send_msg() with short-write");
+	r = net_dns_tcp_send(client, sndbuf, sizeof(sndbuf), TIMEOUT);
+	ok(r == sizeof(sndbuf), "client: net_dns_tcp_send() with short-write");
 
 	// receive message
 
@@ -126,14 +135,10 @@ int main(int argc, char *argv[])
 
 	ok(recv_data.result == sizeof(recvbuf) &&
 	   memcmp(sndbuf, recvbuf, sizeof(recvbuf)) == 0,
-	   "server: tcp_recv_msg() complete and valid data");
+	   "server: net_dns_tcp_recv() complete and valid data");
 
 	// clean up
 
-	if (accepted >= 0) {
-		close(accepted);
-	}
-
 	if (server >= 0) {
 		close(server);
 	}
