@@ -14,15 +14,16 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <assert.h>
 #include <tap/basic.h>
 #include <string.h>
 #include <stdlib.h>
 
-#include "libknot/internal/mempool.h"
 #include "libknot/descriptor.h"
 #include "libknot/packet/wire.h"
 #include "knot/nameserver/process_answer.h"
 #include "fake_server.h"
+#include "contrib/ucw/mempool.h"
 
 /* @note Test helpers. */
 #define TEST_RESET() \
@@ -111,7 +112,7 @@ int main(int argc, char *argv[])
 	plan(3 + TEST_COUNT);
 
 	/* Create processing context. */
-	mm_ctx_t mm;
+	knot_mm_t mm;
 	mm_ctx_mempool(&mm, MM_DEFAULT_BLKSIZE);
 
 	knot_layer_t proc;
@@ -157,7 +158,7 @@ int main(int argc, char *argv[])
 	/* Cleanup. */
 	mp_delete(mm.ctx);
 	server_deinit(&server);
-	conf_free(conf(), false);
+	conf_free(conf());
 
 	return 0;
 }
